
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
      program snfdefine
      !----------------------------------------------------------------
      ! this program modifies the control file to prepare a correct
      ! dscf/ridft/grad/rdgrad/moloch/escf - run
      ! and writes an restart file, such that only irreducible atoms
      ! will be calculated. A define - run might be necessary after
      ! snf_define.
      !
      ! written by Carsten Kind, Uni Erlangen, 2001
      ! modified and extended by Johannes Neugebauer, Uni Erlangen, 2001
      ! modified and extended by Markus Reiher, Uni Erlangen, Nov. 2001, June 2003
      ! modified by Markus Reiher, Uni Bonn, April 2004
      ! modified by C. Herrmann, Uni Bonn, February 2005, 
      ! Uni Jena, April-June 2005
      !----------------------------------------------------------------

!-------------------------------------------------------------
! modules used in snf_define:
!-------------------------------------------------------------
  use f90_kind
  use snf_molecule,       only: type_atom,print_molecule,destroy_molecule,&
       &                        fiso_mass,find_mass,print_isomass,&
       &                        isotope_equal,atom_equal,setisomass,testlin
  use snf_char_table,     only: type_char_table,create_char_table,&
       &                        destroy_char_table,print_char_table,checkpg
  use snf_restart,        only: write_restart,prepare_dummy_restart,&
       &                        backup_restart,check_restart_status,read_restart,&
       &                        prepare_restartfb
  use snf_control,        only: type_ctrl,read_control,print_ctrl,init_ctrl,&
       &                        write_control, copy_tm_to_ctrl,copy_ctrl_to_tm,&
       &                        destroy_control,get_energy,&
       &                        get_energy_mp2,copy_ctrl_to_dal,get_spin, &
       &                        get_dalenergy,get_dalescf,read_control2, copy_ctrl_to_adf,copy_ctrl_to_gauss
  use snf_irred_dist,     only: create_irred_dist,prepare_restart2
  use snf_irred_atoms,    only: create_irred_atoms,destroy_irred_atoms, &
       &                        print_irred_atoms,check_geometry,get_point_grp,&
       &                        is_point_grp,get_rpoint_grp
  use snf_dalton,         only: init_dal_ctrl,write_dal_ctrl
  use snf_rdalmol,        only: read_dal_coord,checkdalgeo,read_dal_ctrl,test_dal_sym
  use snf_adf,            only: init_adf_ctrl 
  use snf_adfmol,         only: rwadfmol, read_adf_coord, read_adf_ctrl,check_adf_settings
  use snf_gauss,          only: init_gauss_ctrl 
  use snf_gaussmol,       only: rwgaussmol, read_gauss_coord
  use snf_turbomole,      only: read_tm_coord,read_tm_ctrl,modify_tm_ctrl,&
       &                        init_tm_ctrl,write_tm_coord
  use snf_tools,          only: string2integer,string2real,clean,prog_type,&
       &                        testfile,bfile,rkeyint,rkeystr,rkeywd,split,&
       &                        calc_type,ramanpos,vroapos,getdfreq,llen
  use snf_error,          only: serror
  use snf_param

!-------------------------------------------------------------
! variables
!-------------------------------------------------------------
      implicit none
      include '../snfdc/VERSION.h'

      type(type_ctrl)          :: &
           & ctrl,ctrl_restart     ! essential control flags and parameters
      type(type_atom), pointer :: &
           & molecule(:)           ! geometry atoms and atomic masses of the molecule
      type(type_char_table)    :: &
           & char_table            ! character table
      integer,pointer          :: &
           &  istepfl(:), &        ! points to calculate(=6*nirred_atoms)
           &  oistepfl(:), &       !
           & atom_index(:), &      ! index of irreducible atom (1:natoms)
           & symm_index(:), &      ! symmetry op. to optain irred. atom (1:natoms)
           & irred_atom_index(:),& ! index list with irreduzible atoms (1:nirred_atoms)
           & selatom(:)            ! atom-numbers which special isotopes are selected for
      real(double),allocatable :: &
           & distortion(:)         ! generalized distortion in the cartesian basis
      integer,pointer          :: &
           & dist_index(:), &    ! index of irreducible dist.
           & dsymm_index(:), &   ! symmetry op. to optain irred. dist.
           & irred_dist_index(:) ! index list with irreducible dist.
      integer :: nirred_dist,orinum
      integer iiat,iidum,iixyz,iidiraus,iidir
      integer jiat,jidum,jixyz,jidiraus,jidir
      integer :: nirred_atoms,i,status,pgstat,nselatom,ipre,ipreo,j
      real(double),pointer :: gradients(:,:,:),dipols(:,:,:),pols(:,:,:,:),massatom(:)
      real(double),pointer :: gtenlao(:,:,:,:),gtenao(:,:,:,:),aten(:,:,:,:)
      real(double),pointer :: ngradients(:,:,:),ndipols(:,:,:),npols(:,:,:,:)
      real(double),pointer :: ngtenlao(:,:,:,:),ngtenao(:,:,:,:),naten(:,:,:,:)
      real(double) :: cthres       ! lowest acceptable value for cstep
      character(len=8),pointer :: &
           & nvmach(:),           & ! list of all used machines
           & onvmach(:)             !
      logical :: valid,found_control,fconhere,wrestart,goback,crap,rflag,vroaflag,nocheck
      character(len=132) :: zeile,errbuff
      character(len=255) :: eligdir
      character(len=80) :: jobfile
      character(len=3) :: pointgrp,oripg
      character :: c
      parameter (cthres=1.0d-3)
!CH Definitions for intensity-only mode (C. Herrmann, 02/2005)
      integer  :: natoms
      integer  :: allmodes ! number of modes read in from external file
                           ! (ctrl%nummodes: number of selected modes)
      logical :: ready     ! true if normal modes have been read from file
!CH End of definitions for intensity-only mode

!----------------------------------------------------------------
! print header; initialize basic variables
!----------------------------------------------------------------
      call pr_snfdefhead
      call init_ctrl(ctrl)
      call rkeystr('DCINPUT','ELIGPATH',ctrl%eligible) 
      found_control=.false.
      crap=.false.
      ready = .false.

!----------------------------------------------------------------
! read file snf_control, if this exists
!----------------------------------------------------------------
      inquire(file='snf_control',exist=fconhere)
      if(fconhere) call old_snfcont(ctrl,selatom,massatom,molecule,c,pointgrp)
      if(c.eq.'n') goto 200

!----------------------------------------------------------------
! start interactive part
!----------------------------------------------------------------

110   call alt_cont(ctrl%control,found_control)
!----------------------------------------------------------------
! read default data if ctrl%control is present.
! if .not.found_control, but ./snf_control exists,
! then read default data from ./snf_control.
! coordinates are  read in next block !
!----------------------------------------------------------------
      if(found_control) then
        call read_control(ctrl,selatom,massatom)
      else if(fconhere) then
        ctrl%control='snf_control'
        call read_control(ctrl,selatom,massatom)
      endif
      orinum=ctrl%numderiv

!----------------------------------------------------------------
! start interactive input:
! select program for single point calculations
!----------------------------------------------------------------
120   if(associated(molecule)) deallocate(molecule)
      call selprog(ctrl,molecule,goback)
      if(goback) goto 110

130 if(associated(ctrl%dal_ctrl)) then
    call dalopt(ctrl,goback)
    if(goback) goto 120
  end if
!----------------------------------------------------------------
! set masses for isotopes found in snf_control
!----------------------------------------------------------------
  molecule(:)%changem=.false.
  if(associated(selatom)) call setisomass(molecule,selatom,massatom)
!----------------------------------------------------------------
! test linearity of molecule
!----------------------------------------------------------------
  if(testlin(molecule)) then
    if(ctrl%symmetry.eq.'c6v') then
      call serror('snf_define',-1,'Found linear molecule. Symmetry ##&
       & is set to C4v to use the full symmetry.')
      ctrl%symmetry='c4v'
    else if(ctrl%symmetry.eq.'d6h') then
      call serror('snf_define',-1,'Found linear molecule. Symmetry ##&
       & is set to D4h to use the full symmetry.')
      ctrl%symmetry='d4h'
    else
      call serror('snf_define',-1,'Found linear molecule. Set symmetry ##&
      & to C4v or D4h, resp., to use the full symmetry.')
    end if
  end if
!----------------------------------------------------------------
! check point-group
!----------------------------------------------------------------
  pgstat=-1
  call checkpg(ctrl%symmetry,pgstat)
  if (pgstat.lt.1) then
    if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
         ctrl%symmetry,'" is not implemented'
    if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
         ctrl%symmetry
    call serror('snf_define',2,errbuff)
  end if
  if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%symmetry)) then
     call print_molecule(molecule)
     write(*,*)
     write(*,*)'Point-group ',Ctrl%symmetry,' does not fit to coordinates.'
     write(*,*)'(Turbomole standard orientation is assumed !)'
     nocheck=.false.
!... check whether symmetry is C1 and should be maintained for DALTON single points
     if (ctrl%progno.eq.2) call test_dal_sym(nocheck)
     if (.not.nocheck) then
       call get_point_grp(molecule,1.D-3,atom_equal,oripg)
       call get_rpoint_grp(molecule,1.D-3,atom_equal,pointgrp)
       write(*,*)
       write(*,*)'Automatic point-group check yields: ',pointgrp
     end if
     if(pointgrp.ne.oripg) then
       write(*,*)
       if(ctrl%progno.eq.1) then
         call bfile(ctrl%tm_ctrl%coord)
         call write_tm_coord(ctrl%tm_ctrl%coord,molecule)
       else if(ctrl%progno.eq.2) then
         call bfile(ctrl%dal_ctrl%coord)
       else if(ctrl%progno.eq.3) then
         call bfile(ctrl%gauss_ctrl%coord)
       else if(ctrl%progno.eq.4) then
         call bfile(ctrl%adf_ctrl%coord)
       end if
       call serror('snf_define',0,'Note that a coordinate transformation was '//&
       &'necessary to verify##this point group. A define run is mandatory to get new MOs !')
     end if
     Ctrl%symmetry=pointgrp
  endif
  pgstat=-1
  if(ctrl%isosym .ne. 'def') then
     call checkpg(ctrl%isosym,pgstat)
     if (pgstat.lt.1) then
       if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
            ctrl%symmetry,'" is not implemented'
       if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
            ctrl%symmetry
        call serror('snf_define',2,errbuff)
     end if
     if(.not.is_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)) then
        call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
        write(*,*)
        write(*,*)'Point-group ',Ctrl%isosym,' does not fit to coordinates'
        write(*,*)'if different isotopes are taken into account.'
        write(*,*)'Automatic point-group check yields: ',pointgrp
        Ctrl%isosym='def'
     endif
  endif
  !----------------------------------------------------------------
  ! small output to validate right inputfile
  !----------------------------------------------------------------
  write(*,*)
  write(*,'(''  Symmetry     : '',a3)')ctrl%symmetry
  if(ctrl%isosym.ne.'def') then
     write(*,'(''  Symmetry with selected isotopes    : '',a3)')ctrl%isosym
  end if
  write(*,'(''  No. of atoms : '',i3)')ubound(molecule,1)
  write(*,*)
  call print_molecule(molecule)

140 valid=.false.
    goback=.false.
    do while(.not.valid)
     write(*,*)
     write(*,*)
     write(*,*) ' If you want to enter one of the following menus'
     write(*,*) ' then type the corresponding command.'
     write(*,*)
     write(*,*) '   iso      : isotope menu'
     write(*,*) '   num      : numerical derivative menu'
     write(*,*) '   spe      : spectrum settings menu'
     write(*,*) '   the      : thermodynamics menu'
     if(ctrl%progno.eq.1.and.ctrl%calcno.eq.1) then
       write(*,*) '   exs      : excited state calculation menu'
     end if
     write(*,*) 
     write(*,*) '   desy     : determine molecular symmetry'
     write(*,*) '   p        : print molecule information'
     write(*,*) '   &        : go back to program selection'
     write(*,*) '   >return< : quit this menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(len_trim(zeile).eq.0) then
       valid=.true.
     else if(len_trim(zeile).eq.1.and.zeile(1:1).eq.'&') then
       if(associated(ctrl%dal_ctrl)) then
         goto 130
       else
         goto 120
       end if
     else if(len_trim(zeile).eq.3) then
       select case(zeile(1:3))
        case ('iso')
          call isomenu(ctrl,molecule,selatom,massatom,pointgrp,pgstat)
        case ('num')
          call nummenu(ctrl)
        case ('spe')
          call specmenu(ctrl)
        case ('the')
          call thermmenu(ctrl)
        case ('exs')
          if(ctrl%progno.eq.1.and.ctrl%calcno.eq.1) then
            call exmenu(ctrl)
          end if
        case default
       end select
     else if(zeile.eq.'desy') then
       call get_point_grp(molecule,1.D-3,atom_equal,oripg)
       call get_rpoint_grp(molecule,1.D-3,atom_equal,pointgrp)
       write(*,*)
       write(*,*)'Automatic point-group check yields: ',pointgrp
       if(pointgrp.ne.oripg) then
        write(*,*)
        if(ctrl%progno.eq.1) then
          call bfile(ctrl%tm_ctrl%coord)
          call write_tm_coord(ctrl%tm_ctrl%coord,molecule)
        else if(ctrl%progno.eq.2) then
          call bfile(ctrl%dal_ctrl%coord)
        else if(ctrl%progno.eq.3) then
          call bfile(ctrl%gauss_ctrl%coord)
        else if(ctrl%progno.eq.4) then
          call bfile(ctrl%adf_ctrl%coord)
        end if
        call serror('snf_define',0,'Note that a coordinate transformation was '//&
        &'necessary to verify##this point group. A define run is mandatory to get new MOs !')
       end if
       ctrl%symmetry=pointgrp
     else if(zeile.eq.'p') then
       call print_molecule(molecule)
     end if
   end do
   if(associated(ctrl%tm_ctrl)) then
     if(ctrl%tm_ctrl%rpast.ne.0) then
     call serror('snf_define',4,'Excited state calculation specified !##Make sure that TURBOMOLE V >= 5.6##is available in your PATH')
     end if
   end if
  !----------------------------------------------------------------
  ! Intensities for selected normal modes menu (C.Herrmann 02/2005)
  !----------------------------------------------------------------
    natoms= ubound(molecule,1)
    if(.not.associated(ctrl%freqin)) allocate(ctrl%freqin(natoms*3))
    if(.not.associated(ctrl%nmodein)) allocate(ctrl%nmodein(natoms*3,natoms*3))
    if(.not.associated(ctrl%selectmode)) allocate(ctrl%selectmode(natoms*3))
    if(.not.ready.and.fconhere) then  
       ctrl%control = './snf_control'
       call read_control2(ctrl,natoms)
       ready=.true.
    end if
!CH150 call intonlymenu(natoms,ctrl,goback,allmodes,ready)
!
!      If intensities are calculated for selected normal modes only,
!      C1 symmetry assumed, since a sensible choice of input normal
!      modes will not contain any symmetry-redundant modes.
!
!CH       if(ctrl%symmetry.ne.'c1 '.and.ctrl%nummodes.gt.0) then
!CH       call serror('snf_define',-1,'Symmetry is changed to C1 for intensities &
!CH                   &only mode')
!CH       ctrl%symmetry='c1 '
!CH       ctrl%maxsym=.false.
!CH       end if
!CH  if(goback) goto 140
  !----------------------------------------------------------------
  ! Raman intensities menu
  !----------------------------------------------------------------
160 if(ramanpos(ctrl%progno,ctrl%calcno)) then
      if(ctrl%progno.eq.1) then
        if(ctrl%tm_ctrl%rpast.eq.0) then
          call ramanmenu(ctrl,goback)
        end if
      else
        call ramanmenu(ctrl,goback)
      end if
    else
      write(*,*) ' Skipping Raman menu since Raman calculations'
      write(*,*) ' cannot be performed in this program mode'
    end if
!CH  if(goback) goto 150
  if(goback) goto 140
  !----------------------------------------------------------------
  ! VROA menu
  !----------------------------------------------------------------
!165 if(vroapos(ctrl%progno,ctrl%calcno)) then
165 if (.false.) then
       call vroamenu(ctrl,goback)
!
!      for VROA C1 symmetry is standard (-> chiral molecules!) and required 
!      (A tensors are differentiated in C1 sym.)
!
       if(ctrl%symmetry.ne.'c1 '.and.ctrl%vroaew.ge.1) then
       call serror('snf_define',-1,'Do you really want to run a VROA calculation for ##&
               &a symmetric molecule? ## Symmetry is changed to C1 for VROA !')
       ctrl%symmetry='c1 '
       ctrl%maxsym=.false.
       end if
    else
      write(*,*) ' Skipping VROA menu since VROA calculations'
      write(*,*) ' cannot be performed in this program mode!'
      write(*,*) ' (DALTON must be used with SCF, MCSCF or DFT!)'
      write(*,*) ' [Note: Raman intensities will automatically be calculated,'
      write(*,*) ' [      if raman is switched off and vroa is switched on !]'
      ctrl%vroaew=0
    end if
  if(goback) goto 160
   if(goback) then
     if(ramanpos(ctrl%progno,ctrl%calcno)) then
       goto 160
     else
!CH       goto 150
       goto 140
     end if
   end if
  !----------------------------------------------------------------
  ! Output options menu
  !----------------------------------------------------------------
170 call outmenu(ctrl,goback)
   if(goback) then
!      goto 165
      goto 160
   end if
  !----------------------------------------------------------------
  ! check file containing total electronic energy 
  ! and try to read total electronic energy
  !----------------------------------------------------------------
  if(abs(ctrl%elec_energy).le.1.0d-10) then
    call clean(jobfile)
    jobfile = './job.last'
!CH  ... this is the only point where progno 3 is referred to as MP2.
!CH  probably this is not up-to-date any more.
!CH    if (ctrl%progno.eq.3) then
!CH      call check_jobfile(jobfile)
!CH      call get_energy_mp2(jobfile,ctrl%elec_energy,6)
!CH!MR can rimp2 handle open-shell cases? => total spin is not yet read
!CH!MR      call get_spin(jobfile,ctrl%spin,6)
!CH    else if(ctrl%progno.eq.2) then
    if(ctrl%progno.eq.2) then
      call clean(jobfile)
      jobfile='./molecule.out'
      call check_jobfile(jobfile)
      if(ctrl%calcno.eq.1.or.ctrl%calcno.eq.6) then
        call get_dalescf(ctrl%elec_energy,jobfile,ctrl%calcno)
      else
        call get_dalenergy(ctrl%elec_energy,jobfile,ctrl%calcno)
      end if
    else
      call get_energy(jobfile,ctrl%elec_energy,6)
      call get_spin(jobfile,ctrl%spin,6)
    endif
  end if
  !----------------------------------------------------------------
  ! all input work done
  !----------------------------------------------------------------
  !----------------------------------------------------------------
  ! build up symmetry informations
  !----------------------------------------------------------------
  call create_char_table(char_table,ctrl%symmetry)
  call create_irred_atoms(molecule,char_table,atom_index,symm_index, &
       &                  irred_atom_index,nirred_atoms)
!MAXSYM>
  !----------------------------------------------------------------
  ! one distortion value for all cartesian coordinates
  ! note: generalized distortions may be necessary in order to
  !       perform distortions along the normal coordinates
  !----------------------------------------------------------------
  allocate(distortion(ubound(molecule,1)*3))
  distortion=ctrl%cstep
  if(ctrl%maxsym) then
     call create_irred_dist(molecule,char_table,dist_index, &
       &  dsymm_index,irred_dist_index,nirred_dist,nirred_atoms,&
       &  irred_atom_index,2*ctrl%numderiv,distortion,.true.)
  end if
!MAXSYM<
  !----------------------------------------------------------------
  ! warnings, consistency checks
  !----------------------------------------------------------------
  call def_warnings(ctrl,crap)
  !----------------------------------------------------------------
  ! write modified input files for turbomole, dalton, molpro ...
  ! Note: turbomole input files are always necessary
  !----------------------------------------------------------------
  if(ctrl%progno.eq.1) then
     call copy_ctrl_to_tm(ctrl)
     ctrl%tm_ctrl%symmetry='c1'
     call bfile(ctrl%tm_ctrl%control)
     call modify_tm_ctrl(ctrl%tm_ctrl)
  else if(ctrl%progno.eq.2) then
    call copy_ctrl_to_dal(ctrl)
    call write_dal_ctrl(ctrl%dal_ctrl,molecule)
  else if(ctrl%progno.eq.3) then
    call copy_ctrl_to_gauss(ctrl)
  else if(ctrl%progno.eq.4) then
    call copy_ctrl_to_adf(ctrl)
  else
     call serror('snf_define',2,'Input-file generation failed')
  endif
  !----------------------------------------------------------------
  ! write control, restart and snf_control
  !----------------------------------------------------------------
  rflag=(ctrl%ramanew.ge.1)
  vroaflag=(ctrl%vroaew.ge.1)
  if (rflag.and.vroaflag) stop " MIRACLE "
  if(associated(ctrl%dynfreq)) &
   &allocate(ctrl_restart%dynfreq(ubound(ctrl%dynfreq,1)))
  call read_restart(ctrl%restart,ctrl_restart,&
       & gradients,dipols,pols,gtenlao,gtenao,aten,&
       & istepfl,nvmach,status,rflag,vroaflag,6)
  if(status.ne.-9.and.ctrl_restart%numderiv.ne.orinum) then
    call serror('snf_define',0, &
  & 'Numderiv values different in control and restart')
    orinum=ctrl_restart%numderiv
  end if
  call destroy_control(ctrl_restart)
  wrestart=.true.
  if (status.eq.1 .or.status.eq.-1.or.status.eq.-2) then
     ! nonemtpy restart exists !!!
     valid= .false.
     wrestart=.false.
     do while(.not. valid)
        write(*,*)
        if(status.eq.1) then
           write(*,*)' Found restart file of a finished calculation'
        else if (status.eq.-1) then
           write(*,*)' Found restart file of a running or aborted calculation'
        else if (status.eq.-2) then
           call serror('snf_define',1,' Old restart file seems to be corrupt. ##&
            &Maybe you tried to calculate Raman intensities ##&
            &but restart file is not prepared for Raman.')
        endif
        write(*,*)
        write(*,*)' Would you like to write a new restart file? (y/n)'
        write(*,*)' (A backup of the existing restart file will be created.)'    
        write(*,*)
        write(*,*)' Note: it is always necessary to write a new restart'
        write(*,*)'       file in order to prepare a SNFDC run with'
        write(*,*)'       new settings !'
        write(*,*)
        write(*,*)' Default: n'
        write(*,*)
        read(*,'(a)') zeile
        if(len_trim(zeile).eq.0) then 
           valid = .true.
        elseif(len_trim(zeile) .eq. 1) then
           if(index('*yn',trim(zeile)).ne.0) then
              valid=.true.
           endif
        endif
     end do
     if(zeile.eq.'y') then
        call backup_restart(ctrl%restart,unit=6)
        wrestart=.true.
     endif
  endif
  if(wrestart) then
     ctrl%nvar=3*ubound(molecule,1)
     if(status.ne.0.and.status.ne.-9) then 
       allocate(oistepfl(ctrl%nvar*2*orinum))
       allocate(onvmach(ctrl%nvar*2*orinum))
       oistepfl(:)=istepfl(:)
       onvmach(:)=nvmach(:)
     end if
     if(associated(nvmach)) deallocate(nvmach)
     if(associated(istepfl)) deallocate(istepfl)
     call prepare_dummy_restart(irred_atom_index,nirred_atoms,ctrl,istepfl)
     if(ctrl%dtype.eq.'fward'.or.ctrl%dtype.eq.'bward') then
       call serror('snf_define',-1,'Note that $maxsym will be turned off##'//&
      & 'since special type of derivatives has been chosen',6)
       ctrl%maxsym=.false.
       call prepare_restartfb(istepfl,ctrl)
     else if(ctrl%maxsym) then
       call prepare_restart2(2*ctrl%numderiv,istepfl,irred_dist_index,&
      &  ctrl%nvar/3,nirred_dist)
     end if
     if(orinum.ne.ctrl%numderiv.and.status.ne.0.and.status.ne.-9) then
     !-------------------------------------------------------------------------
     ! should old data be conserved ?
     !-------------------------------------------------------------------------
      valid=.false.
      do while(.not. valid)
        write(*,*)
        write(*,*)
        write(*,*)' Would you like to use data from old restart file ? (y/n)'
        write(*,*)
        write(*,*)' Default: y'
        write(*,*)
        read(*,'(a)') zeile
        if(len_trim(zeile).eq.0) then 
           valid = .true.
        elseif(len_trim(zeile) .eq. 1) then
           if(index('*yn',trim(zeile)).ne.0) then
              valid=.true.
           endif
        end if
      end do
      if(zeile(1:1).ne.'n') then
        allocate(nvmach(ctrl%nvar*2*ctrl%numderiv))
        allocate(ndipols(3,ctrl%nvar,2*ctrl%numderiv),&
            &   ngradients(ctrl%nvar,ctrl%nvar,2*ctrl%numderiv))
        ndipols=0.0d0
        ngradients=0.0d0
        if(ctrl%ramanew.ge.1) then
          allocate(npols(8,ctrl%nvar,2*ctrl%numderiv,ctrl%ramanew))
          npols=0.0d0
        end if
        if(ctrl%vroaew.ge.1) then
          allocate(npols(8,ctrl%nvar,2*ctrl%numderiv,ctrl%ramanew))
          allocate(ngtenlao(9,ctrl%nvar,2*ctrl%numderiv,ctrl%ramanew))
          allocate(ngtenao(9,ctrl%nvar,2*ctrl%numderiv,ctrl%ramanew))
          allocate(naten(27,ctrl%nvar,2*ctrl%numderiv,ctrl%ramanew))
          npols=0.0d0
          ngtenlao=0.0d0
          ngtenao=0.0d0
          naten=0.0d0
        end if
        !-------------------------------------------------------------------------
        ! copy corresponding entries in dipols, gradients, and pols
        !-------------------------------------------------------------------------
        if(orinum.le.ctrl%numderiv) then
          do j=1,orinum*2
             i=ctrl%numderiv-orinum+j
             ndipols(:,:,i)=dipols(:,:,j)
             ngradients(:,:,i)=gradients(:,:,j)
             if(ctrl%ramanew.ge.1) npols(:,:,i,:)=pols(:,:,j,:)
             if(ctrl%vroaew.ge.1) npols(:,:,i,:)=pols(:,:,j,:)
             if(ctrl%vroaew.ge.1) ngtenlao(:,:,i,:)=gtenlao(:,:,j,:)
             if(ctrl%vroaew.ge.1) ngtenao(:,:,i,:)=gtenao(:,:,j,:)
             if(ctrl%vroaew.ge.1) naten(:,:,i,:)=aten(:,:,j,:)
          end do
        else
          do i=1,ctrl%numderiv*2
             j=orinum-ctrl%numderiv+i
             ndipols(:,:,i)=dipols(:,:,j)
             ngradients(:,:,i)=gradients(:,:,j)
             if(ctrl%ramanew.ge.1) npols(:,:,i,:)=pols(:,:,j,:)
             if(ctrl%vroaew.ge.1) npols(:,:,i,:)=pols(:,:,j,:)
             if(ctrl%vroaew.ge.1) ngtenlao(:,:,i,:)=gtenlao(:,:,j,:)
             if(ctrl%vroaew.ge.1) ngtenao(:,:,i,:)=gtenao(:,:,j,:)
             if(ctrl%vroaew.ge.1) naten(:,:,i,:)=aten(:,:,j,:)
          end do 
        end if
!        do i=1,2*ctrl%numderiv
!          ipre=2*ctrl%numderiv-(i-1)
!          if(ipre.le.0) ipre=ipre-1
!          do j=1,2*orinum
!            ipreo=2*orinum-(j-1)
!            if(ipreo.le.0) ipreo=ipreo-1
!            if(ipre.eq.ipreo) then
!              ndipols(:,:,i)=dipols(:,:,j)
!              ngradients(:,:,i)=gradients(:,:,j)
!              if(ctrl%ramanew.ge.1) npols(:,:,i,:)=pols(:,:,j,:)
!            end if
!          end do
!        end do
        if(status.ne.0) then
         do i=1,ctrl%nvar*2*ctrl%numderiv
          iiat=(i+(3*2*ctrl%numderiv-1))/(3*2*ctrl%numderiv)
          iidum=(i-3*2*ctrl%numderiv*(iiat-1))
          iixyz=(iidum+(2*ctrl%numderiv-1))/(2*ctrl%numderiv)
          iidiraus=mod(iidum,2*ctrl%numderiv)
          if(iidiraus.eq.0) iidiraus=2*ctrl%numderiv
          iidir=ctrl%numderiv-iidiraus+1
          if(iidir.le.0) iidir=iidir-1
          nvmach(i)='------'
          do j=1,ctrl%nvar*2*orinum
            jiat=(j+(3*2*orinum-1))/(3*2*orinum)
            jidum=(j-3*2*orinum*(jiat-1))
            jixyz=(jidum+(2*orinum-1))/(2*orinum)
            jidiraus=mod(jidum,2*orinum)
            if(jidiraus.eq.0) jidiraus=2*orinum
            jidir=orinum-jidiraus+1
            if(jidir.le.0) jidir=jidir-1
            if((iiat.eq.jiat).and.(iixyz.eq.jixyz).and.(iidir.eq.jidir)) then
              istepfl(i)=oistepfl(j)
              nvmach(i)=onvmach(j)
            end if
          end do
         end do
        end if
        !-------------------------------------------------------------------------
        ! deallocate old pointers and set new ones
        !-------------------------------------------------------------------------
        if(associated(dipols)) deallocate(dipols)
        if(associated(pols)) deallocate(pols)
        if(associated(gradients)) deallocate(gradients)
        if(associated(gtenlao)) deallocate(gtenlao)
        if(associated(gtenao)) deallocate(gtenao)
        if(associated(aten)) deallocate(aten)
        dipols => ndipols
        gradients => ngradients
        if(ctrl%ramanew.ge.1) pols => npols
        if(ctrl%vroaew.ge.1) pols => npols
        if(ctrl%vroaew.ge.1) gtenlao => ngtenlao
        if(ctrl%vroaew.ge.1) gtenao => ngtenao
        if(ctrl%vroaew.ge.1) aten => aten
      else
        if(associated(dipols)) deallocate(dipols)
        if(associated(pols)) deallocate(pols)
        if(associated(gradients)) deallocate(gradients)
        if(associated(gtenlao)) deallocate(gtenlao)
        if(associated(gtenao)) deallocate(gtenao)
        if(associated(aten)) deallocate(aten)
      endif
     else 
      if(associated(dipols)) deallocate(dipols)
      if(associated(pols)) deallocate(pols)
      if(associated(gradients)) deallocate(gradients)
      if(associated(gtenlao)) deallocate(gtenlao)
      if(associated(gtenao)) deallocate(gtenao)
      if(associated(aten)) deallocate(aten)
     end if
     call write_restart(gradients,dipols,pols,gtenlao,gtenao,aten,&
                       &istepfl,nvmach,ctrl,unit=6)
  endif
  if(associated(istepfl)) deallocate(istepfl)
  if(associated(oistepfl)) deallocate(oistepfl)
  if(associated(nvmach)) deallocate(nvmach)
  if(associated(onvmach)) deallocate(onvmach)
  if(associated(gradients)) deallocate(gradients)
  if(associated(dipols)) deallocate(dipols)
  if(associated(pols)) deallocate(pols)
  !--------------------------------------------------------------------
  ! name of snf control file should always be "./snf_control", although
  ! input information may be read from other files.
  !--------------------------------------------------------------------
  ctrl%control='./snf_control'
  call write_control(ctrl,molecule,6,allmodes)
200 continue
  call prepdc(crap,ctrl%eligible)
  !----------------------------------------------------------------
  ! write reminder
  !----------------------------------------------------------------
  if(ctrl%symmetry .ne. 'c1'.and. associated(ctrl%tm_ctrl)) then
     write(*,*)
     write(*,*)' ********************************************************'
     write(*,*)'                     R E M I N D E R                     '
     write(*,*)' ********************************************************'
     write(*,*)'  Don''t forget to run define to get mos in c1-symmetry!'
     write(*,*)'    Check afterwards whether scfconv has been changed ! '
     write(*,*)' ********************************************************'
     write(*,*)
  else
     write(*,*)' good luck...'
  endif

  !----------------------------------------------------------------
  ! clean up memory
  !----------------------------------------------------------------
  call destroy_char_table(char_table)
  call destroy_molecule(molecule)
  call destroy_irred_atoms(atom_index,symm_index,irred_atom_index)
  if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
  if(associated(ctrl%freqin)) deallocate(ctrl%freqin)
  if(associated(ctrl%nmodein)) deallocate(ctrl%nmodein)
  if(associated(ctrl%selectmode)) deallocate(ctrl%selectmode)

contains
!************************************************************************

subroutine switchbox(u,val,key,descr)

  !----------------------------------------------------------------
  ! Utility function to print toggle-option
  !
  ! written by Carsten Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  integer,intent(in) :: u
  logical ,intent(in) :: val
  character(len=6),intent(in) :: key
  character(len=*),intent(in) :: descr
  if(val) then
     write(u,'(4x,a6,a18,(a))')key,' off : switch off ',trim(adjustl(descr))
  else
     write(u,'(4x,a6,a17,(a))')key,' on  : switch on ',trim(adjustl(descr))
  endif
end subroutine switchbox

!************************************************************************

subroutine getswitch(answer,val)
  !----------------------------------------------------------------
  ! Utility function to get toggle-option
  !
  ! written by Carsten Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  logical ,intent(inout) :: val
  character(len=*),intent(in) :: answer
  if(index(answer,'off').ne.0) val=.false.
  if(index(answer,'on' ).ne.0) val=.true.
end subroutine getswitch

!************************************************************************

  subroutine dalopt(ctrl,goback)
  !----------------------------------------------------------------
  ! Get basic information for DALTON control file
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  ! slightly modified by M. Reiher, Uni Erlangen, June 2003
  ! (very) slightly modified by C. Herrmann, Uni Jena, June 2005
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl) :: ctrl
  logical         :: goback,valid,ex
  character(len=132) :: zeile,buf
  character(len=10) :: calctype
  valid=.false.
  goback=.false.
  do while(.not.valid)
    write(*,*)
    write(*,*)
    write(*,*) ' DALTON options: '
    write(*,*)
    if(ctrl%progno.ne.2) &
     & call serror('dalopt',2,'Wrong mode value for DALTON calculation')
    call calc_type(ctrl%progno,ctrl%calcno,calctype)
    write(*,'(2a)') '    type of DALTON calculation: ',calctype
    if(ctrl%dal_ctrl%dalmw.gt.0) &
   &  write(*,'(a,i18)') '    DALTON scratch memory size: ',ctrl%dal_ctrl%dalmw
    if(ctrl%calcno.eq.6) ctrl%dal_ctrl%direct=.false.
    call switchbox(6,ctrl%dal_ctrl%direct,'direct',&
     &'integral direct dalton calculation')
    write(*,*)'   dalmw <wm> : set non-default DALTON work memory <wm>'
    write(*,*)'   <ctype>    : select type of calculation: '
    write(*,*)'                HF, DFT, TDHF, MP2, SOPPA, MCSCF, CC2, CCS or CCSD'
    write(*,*)'                (Notice: For VROA you may currently only use '
    write(*,*)'                 the DALTON versions released after Dec. 2003)'
    write(*,*)
    write(*,*)'   &          : go back to last menu'
    write(*,*)
    read(*,'(a)') zeile
    zeile=trim(adjustl(zeile))
    call lower(zeile)
!CH130605    if((zeile(1:3).eq.'scf')) then
    if((zeile(1:3).eq.'hf')) then
      ctrl%calcno=1
    else if((zeile(1:3).eq.'mp2')) then
      write(*,*)
      write(*,*)'   Note: we will use the DALTON''s CC module'
      write(*,*)'         for MP2 calculations'
      write(*,*)
      ctrl%calcno=2
    else if((zeile(1:5).eq.'mcscf')) then
      ctrl%calcno=6
    else if((zeile(1:5).eq.'soppa')) then
      write(*,*)
      write(*,*)'   Note: we will use the DALTON''s CC module'
      write(*,*)'         for MP2 gradients and dipole moments;'
      write(*,*)'         afterwards, polarizabilities will be'
      write(*,*)'         computed using the SOPPA method'
      write(*,*)
      ctrl%calcno=8
    else if((zeile(1:3).eq.'cc2')) then
      ctrl%calcno=3
    else if((zeile(1:4).eq.'ccsd')) then
      ctrl%calcno=5
    else if((zeile(1:3).eq.'ccs')) then
      ctrl%calcno=4
    else if((zeile(1:4).eq.'tdhf')) then
      write(*,*)
      write(*,*)'   Note: we will use DALTON''s RESPONSE'
      write(*,*)'         for Raman calculations'
      write(*,*)
      ctrl%calcno=7
    else if((zeile(1:3).eq.'dft')) then
      write(*,*)
      write(*,*)'   Note: DFT is only available in DALTON versions'
      write(*,*)'         which have appeared after Dec. 2003 !'
666   write(*,*)
      write(*,*)'   Please provide the name of the functional: '
      write(*,*)'   available are: B3LYP, BLYP, WAH, LDA '
      write(*,*)
      read(*,'(a)') zeile
      zeile=trim(adjustl(zeile))
      call lower(zeile)
      if((zeile(1:5).eq.'b3lyp')) then	
	ctrl%dal_func='B3LYP  '
      else if((zeile(1:4).eq.'blyp')) then	
	ctrl%dal_func='BLYP   '
      else if((zeile(1:3).eq.'wah')) then	
	ctrl%dal_func='WAH    '
      else if((zeile(1:3).eq.'lda')) then	
	ctrl%dal_func='LDA    '
      else
	write(*,*)'   THIS FUNCTIONAL IS NOT AVAILABLE !!! '
	goto 666
      end if  
      ctrl%calcno=666
    else if(len_trim(zeile).eq.0) then
      valid=.true.
    else if(zeile(1:5).eq.'dalmw') then
      ctrl%dal_ctrl%dalmw=string2integer(zeile(6:))
      if(ctrl%dal_ctrl%dalmw.lt.0) ctrl%dal_ctrl%dalmw=0
    else if(zeile(1:6).eq.'direct') then
      call getswitch(zeile,ctrl%dal_ctrl%direct)
    else if(zeile(1:1) .eq. '&') then
      goback=.true.
      return
    else
      write(*,*) 'Unknown command'
    end if
  end do
  
  end subroutine dalopt

!************************************************************************

  subroutine isomenu(ctrl,molecule,selatom,massatom,pointgrp,pgstat)
  !----------------------------------------------------------------
  ! select isotopes masses
  !
  ! written by J. Neugebauer and C. Kind, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  integer,pointer         :: selatom(:)
  integer                 :: pgstat
  real(double),pointer    :: massatom(:)
  character(len=3)        :: pointgrp
  character(len=132)      :: zeile,zeile2,errbuff
  logical                 :: valid

  valid = .false.
11 do while(.not. valid)
     write(*,*)
     write(*,*)
     write(*,*) ' Isotope menu'
     write(*,*)
     write(*,*) ' If no isotopes are specified, masses of the most'
     write(*,*) ' abundant (for naturally occuring elements) or most'
     write(*,*) ' stable (for synthetical elements) isotopes will '
     write(*,*) ' be used to calculate the spectra.'
     write(*,*)
     if(any(molecule(:)%changem).or.associated(selatom)) then
        if(ctrl%isosym.eq.'def') then 
           call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
        else
           pointgrp=ctrl%isosym
        endif
        write(*,*) '***************************************************'
        write(*,*) ' WARNING! Specification of isotopes may change the'
        write(*,*) ' molecular symmetry. SNF will therefore perform'
        write(*,'(a,a5,a)') '  the calculations in ',pointgrp, &
             & ' symmetry. '
        if(ctrl%isosym.ne.'def') &
             write(*,*) ' THIS POINT-GROUP HAS BEEN SELECTED BY THE USER.'
        write(*,*) ' If you want to use another symmetry, then choose '
        write(*,*) ' command >sy<'
        write(*,*) '***************************************************'
        write(*,*)
     end if
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   >atom No.<       : specify an isotope for the selected atom'
     write(*,*)'   p                : print molecule information'
     write(*,*)'   d                : default settings'
     if(any(molecule(:)%changem).or.associated(selatom)) then
        write(*,*)'   sy >sym<         : set molecular symmetry to sym'
     end if
     write(*,*)'   >return<         : quit this menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(zeile(1:3).eq.'sy ') then
        if(any(molecule(:)%changem).or.associated(selatom)) then
           if(len_trim(zeile).gt.3) then
              pgstat=-1
              call lower(zeile(4:6))
              call checkpg(zeile(4:6),pgstat)
              if (pgstat.lt.1) then
                errbuff=''
                if(pgstat.eq.0) write(errbuff,'(a,a3,a)') '  Point-group "', &
                     zeile(4:6),'" is not implemented'
                if(pgstat.lt.0) write (errbuff,'(a,a3)') '  Unknown point-group: ',&
                     zeile(4:6)
                call serror('snf_define',0,errbuff)
              else
                 if(.not.is_point_grp(molecule,1.D-3,isotope_equal,zeile(4:6))) then
                    call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
                    write(*,*)
                    write(*,*)'Point-group ',zeile(4:6),' does not fit to coordinates.'
                    write(*,*)'Automatic point-group check yields: ',pointgrp
                    ctrl%isosym='def'
                 else
                    ctrl%isosym=zeile(4:6)
                 endif
              end if
           end if
        else
           write(*,*)' Unknown command'
        end if
     else if(zeile(1:1).eq.'p') then
        call print_molecule(molecule)
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for atomic masses.'
        write(*,*)
        do i=1,ubound(molecule,1)
           molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end do
        molecule(:)%changem=.false.
        ctrl%isosym='def'
        if(associated(selatom)) deallocate(selatom)
     else if(string2integer(zeile).gt.0.and.string2integer(zeile) &
          .le.ubound(molecule,1).and.(zeile(1:2).ne.'sy')) then
        nselatom=string2integer(zeile)
        if(molecule(nselatom)%element.eq."Q") then
          write(*,*) 
          write(*,*) ' You selected a dummy atom. I don''t think '
          write(*,*) ' you really want to change its mass ...  '
          goto 11
        end if
126     write(*,*) 
        write(*,'(a,i5)') ' Select mass number = number of nucleons for atom No. ',nselatom
        write(*,*)
        write(*,*) ' The following isotopes are available:'
        write(*,*)
        write(*,*) ' isotope:             atomic mass [amu]:'
        do i=1,3*(molecule(nselatom)%nucl_charge)
           if(fiso_mass(molecule(nselatom)%nucl_charge,i).gt.1.0d0) then
              write(*,'(2x,a,i3,a,i3,a,1x,a2,5x,f16.8)') &
                   '(',i,',',molecule(nselatom)%nucl_charge,')', &
                   molecule(nselatom)%element, &
                   fiso_mass(molecule(nselatom)%nucl_charge,i)
           end if
        end do
        write(*,*)
        write(*,*)' You may choose one of the following commands:'
        write(*,*)'   >atomic mass No.< : select isotope'
        write(*,*)'   m <mass>          : select another atomic mass [amu]'
        write(*,*)'   >return<          : go back to the main menu'
        write(*,*)
        read(*,'(a)') zeile2
        zeile2=trim(adjustl(zeile2))
        if(fiso_mass(molecule(nselatom)%nucl_charge,string2integer(zeile2)) &
             .gt.1.0d0) then
           molecule(nselatom)%mass= &
                fiso_mass(molecule(nselatom)%nucl_charge,string2integer(zeile2))
           molecule(nselatom)%changem=.true.
        else if (zeile2(1:1).eq.'m') then
           if(len_trim(zeile2).gt.1) then
              if(string2real(zeile2(2:)).ge.1.0d0)then
                 molecule(nselatom)%mass=string2real(zeile2(2:))
                 molecule(nselatom)%changem=.true.
              else 
                  call serror('snf_define',0,'Value for atomic mass &
                   &is critically small!')
              end if
           else
              write(*,*)
              write(*,*)'No <mass> given for atomic mass! Try again!'
              write(*,*)
              goto 126
           end if
        else if (len_trim(zeile2).ne.0) then
           if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
        end if
     else if(string2integer(zeile).gt.0.and.string2integer(zeile) &
          .gt.ubound(molecule,1)) then
        write(*,*)
        write(*,*)' No such atom in this molecule.'
        write(*,*)
     else if(len_trim(zeile).eq.0) then
        if(ctrl%isosym.ne.'def') then
           if(.not.is_point_grp(molecule,1.D-3,isotope_equal,ctrl%isosym)) then
              call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
              write(*,*)
              write(*,*)'Point-group ',Ctrl%isosym,' does not fit to coordinates.'
              write(*,*)'Automatic point-group check yields: ',pointgrp
              Ctrl%isosym='def'
           else
              valid=.true.
           endif
        else
           valid=.true.
        end if
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     end if
  end do
  if(ctrl%isosym.eq.'def' .and.any(molecule(:)%changem)) then
     call get_point_grp(molecule,1.D-3,isotope_equal,pointgrp)
     write(*,*)
     write(*,*)' Calcultion will be evaluated in ',pointgrp,' symmetry'
     write(*,*)' due to isotope settings.'
  endif
  if(associated(selatom)) then
     deallocate(selatom)
     deallocate(massatom)
  end if
  end subroutine isomenu

!************************************************************************

  subroutine nummenu(ctrl)
  !----------------------------------------------------------------
  ! Numerical derivative menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile,errbuff
  logical            :: valid
  valid = .false.
  do while(.not. valid)
     write(*,*)
     write(*,*) ' Numerical derivative menu'
     write(*,*)
     write(*,'(''  Current settings: num='',i1,''  cstep='',f6.3)') &
      &  ctrl%numderiv*2+1,ctrl%cstep
     if(ctrl%dtype.ne.'centr') then
       errbuff=''
       write(errbuff,'(a)') 'Type of derivatives used for Hessian: ' &
   & //ctrl%dtype//'##DO NOT USE THIS TYPE OF DERIVATIVES IN##PRODUCTION CALCULATIONS'
       call serror('snf',0,errbuff,6)
       write(*,*)
     end if
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   num <i>     : set <i>-point central diff. formula (i=3,5,7)'
     write(*,*)'   cstep <val> : set step parameter on <val> [bohr] '
     write(*,*)'   d           : default settings'
     write(*,*)'   >return<    : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if((zeile(1:3)).eq.'num') then
        if(nint(string2real(zeile(4:))).eq.3.or.nint(string2real(zeile(4:))).eq.5 &
         & .or.nint(string2real(zeile(4:))).eq.7) then
           ctrl%numderiv=nint(string2real(zeile(4:)))
           ctrl%numderiv=(ctrl%numderiv-1)/2
        else
           if(len_trim(zeile).le.3) then 
              write(*,*)
              write(*,*)' No <val> given for num! Try again!'
              write(*,*)
           else
              write(*,*)
              write(*,*)' Only 3, 5 and 7 are acceptable values for num! Try again!'
              write(*,*)
           end if
        end if
     else if(zeile(1:5).eq.'cstep') then
        if(len_trim(zeile).gt.5) then
           if(string2real(zeile(6:)).ge.cthres)then
              ctrl%cstep=string2real(zeile(6:))
           else 
              call serror('snf_define',0,'Value for cstep is critically &
               &small! Try again!')
           end if
        else
           write(*,*)
           write(*,*)'No <val> given for cstep! Try again!'
           write(*,*)
        end if
     else if((zeile(1:6)).eq.'dtype ') then
       ctrl%dtype=zeile(7:11)
       if(ctrl%dtype.ne.'fward'.and.ctrl%dtype.ne.'bward') then
         if(ctrl%dtype.ne.'centr') then
           write(*,*)
           write(*,*) 'dtype value not valid'
           write(*,*)
           ctrl%dtype='centr'
         end if
       end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for num and cstep.'
        write(*,*)
        ctrl%cstep=0.01
        ctrl%numderiv=1
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  end do
  end subroutine nummenu

!************************************************************************

  subroutine specmenu(ctrl)
  !----------------------------------------------------------------
  ! Spectrum settings menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid
  integer, parameter :: mxitem=10
  character(len=132) :: zeile,item(mxitem)

  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' Spectrum settings menu'
     write(*,*)
     write(*,'(''  Current settings: minf='',f6.1,''   maxf='',f6.1,''   w='',f5.1,''   np='',i6)') &
          &    ctrl%minfreq,ctrl%maxfreq,ctrl%width,ctrl%npoint
     write(*,'(A)')'  Type of peak profile chosen: '//ctrl%peaktype
     write(*,'(''  Scaling factor for wavenumbers ='',f12.6)') ctrl%freqscale
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   minf <val> : set lower frequency bound in cm^(-1)' 
     write(*,*)'   maxf <val> : set upper frequency bound in cm^(-1)'
     write(*,*)'   w    <val> : set peak width in gaussian represenation in cm^(-1)'
     write(*,*)'   np   <num> : set number of grid points for gaussian representation '
     write(*,*)'   pt   <g/l> : set peak profile to Gauss (g) or Lorentz (l)'
     write(*,*)'   sf   <val> : set scaling factor for frequency output'
     write(*,*)'                (e.g. 0.8929 for HF/6-31G*)'
     write(*,*)'   d          : default settings'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if(zeile(1:4).eq.'minf') then
        if(string2real(zeile(5:)).lt.0.) then
           write(*,*)
           write(*,*)' minf must be greater than or equal zero!'
           write(*,*)
        else if(string2real(zeile(5:)).ge.ctrl%maxfreq) then
           write(*,*)
           write(*,*)' minf must be less than maxf!'
           write(*,*)
        else
           ctrl%minfreq=string2real(zeile(5:))
        end if
     else if(zeile(1:4).eq.'maxf') then
        if(string2real(zeile(5:)).le.ctrl%minfreq) then
           write(*,*)
           write(*,*)' maxf must be greater than minf!'
           write(*,*)
        else
           ctrl%maxfreq=string2real(zeile(5:))
        end if
     else if(zeile(1:1).eq.'w') then
        if(string2real(zeile(2:)).le.0.0d0) then
           write(*,*)
           write(*,*)' width must be greater than zero!'
           write(*,*)
        else
           ctrl%width=string2real(zeile(2:))
        end if
     else if(zeile(1:2).eq.'pt') then
        call split(zeile,item,mxitem)
        if(item(2).eq.'l') then
          ctrl%peaktype='lorentz'
        else if(item(2).eq.'g') then
          ctrl%peaktype='gauss  '
        else 
          write(*,*) 'no such peak type, using gauss profile'
          ctrl%peaktype='gauss  '
        end if
     else if(zeile(1:2).eq.'np') then
        if(nint(string2real(zeile(3:))).le.0.) then
           write(*,*)
           write(*,*)' np must be greater than zero!'
           write(*,*)
        else
           ctrl%npoint=nint(string2real(zeile(3:)))
        end if
     else if(zeile(1:2).eq.'sf') then
        if(string2real(zeile(3:)).le.0.0d0) then
           write(*,*)
           write(*,*)' scaling factor must be greater than zero!'
           write(*,*)
        else
           ctrl%freqscale=string2real(zeile(3:))
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for minf, maxf, width and np.'
        write(*,*)
        ctrl%minfreq=200
        ctrl%maxfreq=3500
        ctrl%width=20
        ctrl%npoint=2000
        ctrl%freqscale=1.0d0
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine specmenu

!************************************************************************

  subroutine thermmenu(ctrl)
  !----------------------------------------------------------------
  ! thermodynamics menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid
  character(len=132) :: zeile

  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)'Thermodynamics menu'
     write(*,*)
     write(*,'(''  Current settings:  t='',f5.1,'' k    p='',f10.4,&
       &'' bar'')')ctrl%temp,ctrl%press/100000.0d0
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   t <val>   : temperature in k'
     write(*,*)'   p <val>   : pressure in bar'
     write(*,*)'   d         : default settings'
     write(*,*)'   >return<  : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if(zeile(1:1).eq.'t') then
        if(string2real(zeile(2:)).lt.0.) then
           write(*,*)
           write(*,*)' Temperature must be greater than or equal zero!'
           write(*,*)
        else
           ctrl%temp=string2real(zeile(2:))
        end if
     else if(zeile(1:1).eq.'p') then
        if(string2real(zeile(2:))*100000.0d0.lt.0) then
           write(*,*)
           write(*,*)' Pressure must be greater than or equal zero!'
           write(*,*)
        else
           ! conversion from bar to pa
           ctrl%press=string2real(zeile(2:))*100000.0d0
           ! test on pressure dimension
           if (ctrl%press.gt.1.0d8) then
              write(*,*)
              write(*,*) "******************************************"
              write(*,*) " WARNING: pressure must be given in [bar]!"
              write(*,*) "******************************************"
              write(*,*)
           end if
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Setting default values for t and p.'
        write(*,*)
        ctrl%temp=273.15d0+25.0d0
        ctrl%press=101325.0d0
     else if(len_trim(zeile).eq.0) then   
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine thermmenu

!************************************************************************

  subroutine exmenu(ctrl)
  !----------------------------------------------------------------
  ! excited state calculation menu (for TURBOMOLE only)
  !
  ! written by J. Neugebauer, Uni Erlangen, 2003
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid,dft
  integer            :: exopt
  character(len=132) :: zeile,z2
  character(len=10)  :: spinstate
  character(len=1)   :: sps

  valid=.false.
  call rkeywd(ctrl%tm_ctrl%control,'$dft',dft)
  if(dft) ctrl%tm_ctrl%extype='rpa'
  if(ctrl%tm_ctrl%scftol.gt.1.0d-8) then
    write(*,*) '** WARNING: $scftol too large'
    write(*,*) '**          setting scftol = 1.0d-10'
    ctrl%tm_ctrl%scftol=1.0d-10
  end if
  do while(.not. valid)
     if(ctrl%tm_ctrl%smult.eq.1) then
       spinstate='singlet'
     else if(ctrl%tm_ctrl%smult.eq.3) then
       spinstate='triplet'
     else 
       spinstate='[ERROR]'
     end if
     exopt=ctrl%tm_ctrl%exopt
     if(exopt.eq.0) exopt=ctrl%tm_ctrl%rpast
     write(*,*)
     write(*,*)'Excited state calculation menu'
     write(*,*)
     write(*,*)' Current settings: '
     write(*,'(1x,a,i7)')' No. of excited states to be optimized : ',ctrl%tm_ctrl%rpast
     write(*,'(1x,a,i7)')' State no. to be calculated in SNF     : ',exopt
     write(*,'(1x,2a)')' Spin state of excited states          : ',spinstate
     if(.not.dft) &
   & write(*,'(1x,2a)')' Type of excited state calculation     : ',ctrl%tm_ctrl%extype
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     write(*,*)'   sps <s/t> : set spin state to <singlet/triplet>'
     write(*,*)'   eno <int> : set no. of state to be calculated in SNF'
     write(*,*)'   nex <int> : no. of excited states to be optimized'
     if(.not.dft) &
   & write(*,*)'   ext <type>: type of calculation (rpa/cis)'
     write(*,*)'   d         : default settings (ground state calculation)'
     write(*,*)'   >return<  : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if(zeile(1:3).eq.'sps') then
       z2=adjustl(zeile(4:))
       sps=z2(1:1)
       if(index(sps(1:1),'s').ne.0) then
         ctrl%tm_ctrl%smult=1
       else if(index(sps(1:1),'t').ne.0) then
         ctrl%tm_ctrl%smult=3
       else 
         write(*,*) ' No such spinstate !'
       end if
     else if(zeile(1:3).eq.'ext'.and.(.not.dft)) then
       read(zeile(5:),*) ctrl%tm_ctrl%extype
       if(ctrl%tm_ctrl%extype.ne.'rpa'.and.ctrl%tm_ctrl%extype.ne.'cis') then
         write(*,*)
         write(*,*) ' No such exited state calculation type,'
         write(*,*) ' using RPA instead !'
         write(*,*)
         ctrl%tm_ctrl%extype='rpa'
       end if
     else if(zeile(1:3).eq.'nex') then
        if(string2integer(zeile(4:)).lt.1) then
           write(*,*)
           write(*,*)' No excited states will be calculated !'
           write(*,*)
           ctrl%tm_ctrl%rpast=0
           ctrl%tm_ctrl%exopt=0
        else
           ctrl%tm_ctrl%rpast=string2integer(zeile(4:))
        end if
     else if(zeile(1:3).eq.'eno') then
        if(string2integer(zeile(4:)).lt.1) then
           ctrl%tm_ctrl%exopt=0
        else if(string2integer(zeile(4:)).gt.ctrl%tm_ctrl%rpast) then
           write(*,*)
           write(*,*)' Not so many excited states in this calculation !'
           write(*,*)
           ctrl%tm_ctrl%exopt=0
        else
           ctrl%tm_ctrl%exopt=string2integer(zeile(4:))
        end if
     else if(zeile(1:1).eq.'d') then
        write(*,*)
        write(*,*)' Ground state calculation chosen'
        write(*,*)
        ctrl%tm_ctrl%rpast=0
        ctrl%tm_ctrl%exopt=0
     else if(len_trim(zeile).eq.0) then   
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  if(ctrl%tm_ctrl%exopt.eq.0) ctrl%tm_ctrl%exopt=ctrl%tm_ctrl%rpast
  end subroutine exmenu

!************************************************************************

  subroutine ramanmenu(ctrl,goback)
  !----------------------------------------------------------------
  ! Raman intensities menu
  !
  ! written J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile,buf
  character(len=9)   :: tmptype
  logical            :: valid,goback,tmval,raman
  raman = (ctrl%ramanew.gt.0)
  valid = .false.
  goback= .false.
1 do while(.not. valid)
     if(associated(ctrl%tm_ctrl)) then
       if(ctrl%tm_ctrl%scftol.gt.1.0d-8) then
         write(*,*) '** WARNING: $scftol too large'
         write(*,*) '**          setting scftol = 1.0d-10'
         ctrl%tm_ctrl%scftol=1.0d-10
       end if
     end if
     write(*,*)
     write(*,*)' Raman intensities menu'
     write(*,*)
     write(*,*)'  CAUTION: Calculation of Raman intensities'
     write(*,*)'           is very time-consuming !'
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     call switchbox(6,raman,'raman ','Raman intensity calculation')
     if(ctrl%ramanew.le.0) then
       ctrl%ramanew=0
       write(*,*)  '                (Current setting: no Raman calculation)'
     else
       write(*,'(1x,a,i2,a)')  &
   & '                (Current setting: Raman spectra for ',ctrl%ramanew,' exc. wl.)'
     end if
     if(ctrl%ramanew.gt.0) then
       write(*,*)'   theta <val>: enter scattering angle for cross sections'
       write(*,'(1x,a,f8.2)') '                current setting:',ctrl%theta
       write(*,*)'   prptype    : print possible data types for rplot option'
       write(*,*)'   rplot <str>: enter type of data to be plotted in Raman spectrum'
       write(*,'(1x,2a)') '                current setting: ',ctrl%ramanptype
       tmval=(ctrl%progno.eq.1)
       if(tmval) then
         write(*,*)'   rpa <val>  : enter rpa residuum parameter for escf run'
         write(*,'(a,i2)') '                 current setting: ', &
     &    ctrl%tm_ctrl%rpaconv
         write(*,*)'   tol <val>  : enter integral neglect threshold for Turbomole'
         write(*,'(a,d20.10)') '                 current setting: ', &
     &    ctrl%tm_ctrl%scftol
       end if
     end if
     if(ctrl%ramanew.gt.0.and..not.(ctrl%progno.eq.3.and.ctrl%calcno.eq.1)) then
!CH     if(ctrl%ramanew.gt.0.and.(ctrl%progno.ne.3.and.ctrl%calcno.ne.1)) then
       write(*,*)'   wl <val>   : enter up to 10 different wavelengths [nm]'
       write(*,*)'                for calculation of polarizabilities'
       write(*,*)'   fr <val>   : enter up to 10 different frequencies [a.u.]'
       write(*,*)'                for calculation of polarizabilities'
       write(*,*)'   p          : print chosen frequencies '
       write(*,*)'                (default = static limit; evaluation for 514.5 nm)'
     end if
     write(*,*)'   d          : default settings'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if((zeile(1:5).eq.'raman')) then
        call getswitch(zeile,raman)
        if(raman) then
          if(ctrl%ramanew.le.0) ctrl%ramanew=1
        else
          ctrl%ramanew=0
        end if
     else if((zeile(1:2).eq.'fr').and.raman) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl%dynfreq)
     else if((zeile(1:2).eq.'wl').and.raman) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl%dynfreq)
        if(associated(ctrl%dynfreq)) then
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              call serror('ramanmenu',0,'Infinite frequency detected!##&
               &Frequency is set to 0.0d0 instead.')
              ctrl%dynfreq(i)=0.0d0
            else
              ctrl%dynfreq(i)=nm2freq/ctrl%dynfreq(i)
            end if
          end do
          ctrl%ramanew=ubound(ctrl%dynfreq,1)
        else
          ctrl%ramanew=1
        end if
     else if((zeile(1:5).eq.'theta').and.raman) then
       read(zeile(6:),*,err=1,end=1) ctrl%theta
     else if((zeile(1:7).eq.'prptype').and.raman) then
       write(*,*) 
       write(*,*) ' Possible data types to be plotted in Raman spectrum: '
       write(*,*) 
       write(*,*) '  (ractivi) : Raman scattering activity'
       write(*,*) 
       write(*,*) '   Raman scattering cross sections:'
       write(*,*) 
       write(*,*)  '    data type | pol. incident beam   | analyzer'
       write(*,*)  '   -----------|----------------------|---------------------'
       write(*,*)  '    (per,u)   | perp. to scat. plane | none    '
       write(*,*)  '    (per,per) | perp.                | perp. to scat. plane'
       write(*,*)  '    (per,par) | perp.                | parallel'
       write(*,*)  '    (par,par) | parallel             | parallel'
       write(*,*)  '    (par,u)   | parallel             | none'
       write(*,*)  '    (nat,u)   | none (''natural'')     | none'
       write(*,*)
       write(*,*)  '    not calculated (no valid data types):'
       write(*,*)  '    (par,per) | parallel             | perp.    | = (per,par)'
       write(*,*)  '    (nat,per) | none                 | perp.    | = (per,u)/2'
       write(*,*)  '    (nat,par) | none                 | parallel | = (par,u)/2'
       write(*,*)
     else if((zeile(1:5).eq.'rplot').and.raman) then
       ctrl%ramanptype='         '
       read(zeile(6:),'(A)',err=1,end=1) buf
       tmptype=trim(adjustl(buf))
       ctrl%ramanptype(1:9)=tmptype(1:9)
       if(ctrl%ramanptype.ne.'(ractivi)'.and.ctrl%ramanptype.ne.'(per,u)  '.and. &
      &   ctrl%ramanptype.ne.'(per,per)'.and.ctrl%ramanptype.ne.'(per,par)'.and. &
      &   ctrl%ramanptype.ne.'(par,par)'.and.ctrl%ramanptype.ne.'(par,u)  '.and. &
      &   ctrl%ramanptype.ne.'(nat,u)  ') then
         write(*,*) ' no valid data type for Raman plot|'//ctrl%ramanptype(1:9)//'|'
         ctrl%ramanptype='(ractivi)'
       else
         ctrl%print_rcs=.true.
       end if
     else if((zeile(1:3).eq.'rpa').and.raman.and.tmval) then
       read(zeile(4:),*,err=1,end=1) ctrl%tm_ctrl%rpaconv
     else if((zeile(1:3).eq.'tol').and.tmval) then
       read(zeile(4:),*,err=1,end=1) ctrl%tm_ctrl%scftol
     else if(len_trim(zeile).eq.0) then 
        valid = .true.
     else if(zeile(1:1) .eq. 'p'.and.len_trim(zeile).eq.1.and.raman) then
        write (*,*) 
        write (*,*) ' selected wavelengths/frequencies for dynamical polarizabilities:'
        write (*,*) 
        if(associated(ctrl%dynfreq)) then
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              write(*,'(a,a,f18.10,a)') '    wavelength(1) =           infinity',&
               &' nm ; angular frequency = ',0.0d0, ' a.u.'
            else
             write(*,'(a,i1,a,f18.10,a,f18.10,a)') '    wavelength(',i,') = ',&
              &1.d0/ctrl%dynfreq(i)*nm2freq, ' nm ; angular frequency = ',&
              &ctrl%dynfreq(i),' a.u.'
            end if
          end do
        else
          write(*,'(a,a,f18.10,a)')     '    wavelength(1) =           infinity',&
           &' nm ; angular frequency =  ',0.0d0, ' a.u.'
        end if
        write (*,*) 
     else if(zeile(1:1) .eq. 'd'.and.len_trim(zeile).eq.1) then
        raman=.false.
        if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
        ctrl%ramanew=0
     else if(zeile(1:1) .eq. '&') then
       goback=.true.
       return
     else 
        write(*,*)
        write(*,*)' Unknown command'
     endif
  end do
  end subroutine ramanmenu

!************************************************************************

  subroutine vroamenu(ctrl,goback)
  !----------------------------------------------------------------
  ! VROA menu
  !
  ! written M. Reiher, Uni Erlangen, June 2003
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile,buf
  character(len=9)   :: tmptype
  logical            :: valid,goback,tmval,vroa
  vroa = (ctrl%vroaew.gt.0)
  valid = .false.
  goback= .false.
1 do while(.not. valid)
     if(associated(ctrl%tm_ctrl)) then
       if(ctrl%tm_ctrl%scftol.gt.1.0d-8) then
         write(*,*) '** WARNING: $scftol too large'
         write(*,*) '**          setting scftol = 1.0d-10'
         ctrl%tm_ctrl%scftol=1.0d-10
       end if
     end if
     write(*,*)
     write(*,*)' Vibrational Raman Optical Activity (VROA) menu'
     write(*,*)
     write(*,*)'      NOTE: VROA is currently only possible using the'
     write(*,*)'            DALTON version of Dec. 2003 or later !'
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     call switchbox(6,vroa,'vroa  ','VROA calculation')
!
!	check for 'raman on'
!
     if (vroa.and.(ctrl%ramanew.gt.0)) then
       call serror('vroamenu',0,'Switch raman off before you switch vroa on!')
       ctrl%vroaew=0
       vroa=.false.                     
     end if
!
     if(ctrl%vroaew.le.0) then
       ctrl%vroaew=0
       write(*,*)  '                (Current setting: no VROA calculation)'
     else
       write(*,'(1x,a,i2,a)')  &
   & '                (Current setting: VROA spectra for ',ctrl%vroaew,' exc. wl.)'
     end if
     if(ctrl%vroaew.gt.0) then
       write(*,*)'   theta <val>: enter scattering angle for cross sections'
       write(*,'(1x,a,f8.2)') '                current setting:',ctrl%theta
       write(*,*)'   prptype    : print possible data types for rplot option'
!MR       write(*,*)'   rplot <str>: enter type of data to be plotted in Raman spectrum'
!MR       write(*,'(1x,2a)') '                current setting: ',ctrl%ramanptype
!MR       tmval=(ctrl%progno.eq.1)
!MR       if(tmval) then
!MR         write(*,*)'   rpa <val>  : enter rpa residuum parameter for escf run'
!MR         write(*,'(a,i2)') '                 current setting: ', &
!MR     &    ctrl%tm_ctrl%rpaconv
!MR         write(*,*)'   tol <val>  : enter integral neglect threshold for Turbomole'
!MR         write(*,'(a,d20.10)') '                 current setting: ', &
!MR     &    ctrl%tm_ctrl%scftol
!MR       end if
     end if
     if(ctrl%vroaew.gt.0) then
       write(*,*)'   wl <val1,val2,...>   : enter up to 10 different wavelengths [nm]'
       write(*,*)'                          for calculation of polarizabilities'
       write(*,*)'   fr <val1,val2,...>   : enter up to 10 different frequencies [a.u.]'
       write(*,*)'                          for calculation of polarizabilities'
       write(*,*)'   p                    : print chosen frequencies '
       write(*,*)'                          (default = static limit; evaluation for 514.5 nm)'
     end if
     write(*,*)'   d          : default settings'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
!
! read in chosen option:
!
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
!
     if((zeile(1:5).eq.'vroa')) then
        call getswitch(zeile,vroa)
        if(vroa) then
!	  check for 'raman on'
          if (ctrl%ramanew.gt.0) then
            call serror('vroamenu',0,'Switch raman off before you switch vroa on! ##&
                       &Till then, VROA is switched off!')
            ctrl%vroaew=0
            vroa=.false.                     
	  else 
            if(ctrl%vroaew.le.0) ctrl%vroaew=1
            if (.not.associated(ctrl%dynfreq)) then
              allocate (ctrl%dynfreq(1))
              ctrl%dynfreq(1)=0.0d0
            end if
            call serror('vroamenu',-1,'NOTE: C1 symmetry is used for VROA! ##&
                       &(Since point group symmetry is not exploited for A tensors!)')
          end if
        else
          ctrl%vroaew=0
        end if
     else if((zeile(1:2).eq.'fr').and.vroa) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl%dynfreq)
        if (associated(ctrl%dynfreq))ctrl%vroaew=ubound(ctrl%dynfreq,1)
     else if((zeile(1:2).eq.'wl').and.vroa) then
        if(len_trim(zeile).gt.2) call getdfreq(zeile(3:),ctrl%dynfreq)
        if(associated(ctrl%dynfreq)) then
          ctrl%vroaew=ubound(ctrl%dynfreq,1)
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              call serror('vroamenu',0,'Infinite frequency detected!##&
               &Frequency is set to 0.0d0 instead.')
              ctrl%dynfreq(i)=0.0d0
            else
              ctrl%dynfreq(i)=nm2freq/ctrl%dynfreq(i)
            end if
          end do
          ctrl%vroaew=ubound(ctrl%dynfreq,1)
        else
          ctrl%vroaew=1
        end if
     else if((zeile(1:5).eq.'theta').and.vroa) then
       read(zeile(6:),*,err=1,end=1) ctrl%theta
     else if((zeile(1:7).eq.'prptype').and.vroa) then
       write(*,*) 
       write(*,*) ' In the VROA mode, you will also get the Raman spectrum! '
       write(*,*) ' Please, enter the experimental set up for calc. of Raman cross sections:'
       write(*,*) 
       write(*,*) ' (For VROA there are currently no options and you will get '
       write(*,*) ' all data, which is of importance for an experimental '
       write(*,*) ' VROA set up [for four standard scattering angles] !)'
       write(*,*) 
       write(*,*) '  (ractivi) : Raman scattering activity'
       write(*,*) 
       write(*,*) '   Raman scattering cross sections:'
       write(*,*) 
       write(*,*)  '    data type | pol. incident beam   | analyzer'
       write(*,*)  '   -----------|----------------------|---------------------'
       write(*,*)  '    (per,u)   | perp. to scat. plane | none    '
       write(*,*)  '    (per,per) | perp.                | perp. to scat. plane'
       write(*,*)  '    (per,par) | perp.                | parallel'
       write(*,*)  '    (par,par) | parallel             | parallel'
       write(*,*)  '    (par,u)   | parallel             | none'
       write(*,*)  '    (nat,u)   | none (''natural'')     | none'
       write(*,*)
       write(*,*)  '    not calculated (no valid data types):'
       write(*,*)  '    (par,per) | parallel             | perp.    | = (per,par)'
       write(*,*)  '    (nat,per) | none                 | perp.    | = (per,u)/2'
       write(*,*)  '    (nat,par) | none                 | parallel | = (par,u)/2'
       write(*,*)
     else if((zeile(1:5).eq.'rplot').and.vroa) then
       ctrl%ramanptype='         '
       read(zeile(6:),'(A)',err=1,end=1) buf
       tmptype=trim(adjustl(buf))
       ctrl%ramanptype(1:9)=tmptype(1:9)
       if(ctrl%ramanptype.ne.'(ractivi)'.and.ctrl%ramanptype.ne.'(per,u)  '.and. &
      &   ctrl%ramanptype.ne.'(per,per)'.and.ctrl%ramanptype.ne.'(per,par)'.and. &
      &   ctrl%ramanptype.ne.'(par,par)'.and.ctrl%ramanptype.ne.'(par,u)  '.and. &
      &   ctrl%ramanptype.ne.'(nat,u)  ') then
         write(*,*) ' no valid data type for Raman plot|'//ctrl%vroaptype(1:9)//'|'
         ctrl%ramanptype='(ractivi)'
       else
         ctrl%print_rcs=.true.
       end if
!MR     else if((zeile(1:3).eq.'rpa').and.vroa.and.tmval) then
!MR       read(zeile(4:),*,err=1,end=1) ctrl%tm_ctrl%rpaconv
!MR     else if((zeile(1:3).eq.'tol').and.tmval) then
!MR       read(zeile(4:),*,err=1,end=1) ctrl%tm_ctrl%scftol
     else if(len_trim(zeile).eq.0) then 
        valid = .true.
     else if(zeile(1:1) .eq. 'p'.and.len_trim(zeile).eq.1.and.vroa) then
        write (*,*) 
        write (*,*) ' selected wavelengths/frequencies for dynamical polarizabilities:'
        write (*,*) 
        if(associated(ctrl%dynfreq)) then
          do i=1,ubound(ctrl%dynfreq,1)
            if(abs(ctrl%dynfreq(i)).le.1.0d-10) then
              write(*,'(a,a,f18.10,a)') '    wavelength(1) =           infinity',&
               &' nm ; angular frequency = ',0.0d0, ' a.u.'
            else
             write(*,'(a,i1,a,f18.10,a,f18.10,a)') '    wavelength(',i,') = ',&
              &1.d0/ctrl%dynfreq(i)*nm2freq, ' nm ; angular frequency = ',&
              &ctrl%dynfreq(i),' a.u.'
            end if
          end do
        else
          write(*,'(a,a,f18.10,a)')     '    wavelength(1) =           infinity',&
           &' nm ; angular frequency =  ',0.0d0, ' a.u.'
        end if
        write (*,*) 
     else if(zeile(1:1) .eq. 'd'.and.len_trim(zeile).eq.1) then
        vroa=.false.
        if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
        ctrl%vroaew=0
     else if(zeile(1:1) .eq. '&') then
       goback=.true.
       return
     else 
        write(*,*)
        write(*,*)' Unknown command'
     endif
  end do
  end subroutine vroamenu

!************************************************************************
  subroutine intonlymenu(natoms,ctrl,goback,allmodes,ready)
  !----------------------------------------------------------------
  ! intensities for selected normal modes menu
  !
  ! written by C. Herrmann, Uni Bonn, 2005 
  !----------------------------------------------------------------
  use snf_spectrum, only : type_peak
  implicit none
  type(type_ctrl)    :: ctrl
!  type(type_peak)    :: spectrum(:)
  character(len=132) :: zeile
  logical            :: valid,goback,all,ex,ready,intonly
  integer            :: i,modenr,lfile,natoms,allmodes,dumsum
  character(len=10)   :: bla
  character(len=30)  :: filename
  intonly = (ctrl%nummodes.gt.0)
  valid = .false.
  goback= .false.
  if(allmodes.lt.ctrl%nummodes) allmodes = ctrl%nummodes
1 do while(.not. valid)
!     if(associated(ctrl%tm_ctrl)) then
!       if(ctrl%tm_ctrl%scftol.gt.1.0d-8) then
!         write(*,*) '** WARNING: $scftol too large'
!         write(*,*) '**          setting scftol = 1.0d-10'
!         ctrl%tm_ctrl%scftol=1.0d-10
!       end if
!     end if
     write(*,*)
     write(*,*)
     write(*,*)
     write(*,*)' Intensities only menu'
     write(*,*)' Read in selected normal modes and calculate IR-intensities'
     call switchbox(6,intonly,'int   ','Intensites only calculation')
     if(ctrl%nummodes.eq.0) then
       write(*,*)  '                (Current setting: no intensities only calculation)'
     else
       write(*,*)  &
   & ' (Current setting: intensities only calculation)'
     end if
     if(intonly) then
     if(ready) then 
       write(*,*)
       write(*,*)' The following normal modes have been read in ' 
       write(*,*)  &
   & ' (intensities will be calculated for the modes marked by 1):'
       write(*,*)
       do i = 1, allmodes
           write(*,'(2x,a,i4,a,f10.2,i4)') ' mode no. ',i,', wavenumber:',&
   &            ctrl%freqin(i), ctrl%selectmode(i)
       end do
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     end if
     write(*,*)'   re  <file> : read normal modes from file <file>'
     write(*,*)'   mode <no.> : select mode <no.> for intensity calculation'
     write(*,*)' - mode <no.> : de-select mode <no.> for intensity calculation'
     write(*,*)'   all        : select all modes'
     write(*,*)' - all        : de-select all modes'
     write(*,*)'   dtype <int>: choose type of displacement'
     write(*,*)'                (1: length, 2: expected energy change)'
     end if
     write(*,*)'   d          : default settings' 
     write(*,*)'                (read and select all modes from g98.out; dtype 1)'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if((zeile(1:3).eq.'int')) then
        call getswitch(zeile,intonly)
        if(.not.intonly) then
          ctrl%nummodes = 0
          allmodes=0
        end if
     else if(zeile(1:4) .eq.'mode') then
        read(zeile,'(a4,i4)') bla, modenr
        if(modenr.lt.1.or.modenr.gt.allmodes) then
          write(*,*)
          write(*,*)' Mode nr. ',modenr,' does not exist.'
          write(*,*)
        else if(ctrl%selectmode(modenr).eq.1) then
          write(*,*)
          write(*,*)' Mode nr. ',modenr,' has already been selected'
          write(*,*)
        else
          ctrl%selectmode(modenr) = 1
        end if
     else if(zeile(1:6) .eq.'- mode') then
        read(zeile,'(a6,i4)') bla, modenr
        if(modenr.lt.1.or.modenr.gt.allmodes) then
          write(*,*)
          write(*,*)' This mode does not exist.'
          write(*,*)
        else
          ctrl%selectmode(modenr) = 0
        end if
     else if(zeile(1:3) .eq.'all') then
         do i = 1,allmodes
           ctrl%selectmode(i) = 1
         end do
     else if(zeile(1:5) .eq.'- all') then
         do i = 1,allmodes
           ctrl%selectmode(i) = 0
         end do
     else if(zeile(1:2) .eq.'re') then
         read(zeile,*) bla, filename
         call llen(filename,lfile)
         write(*,*)
         write(*,*)' Normal modes will be read from file '//filename(1:lfile)
         write(*,*)
         call remod(ctrl,natoms,molecule,filename,lfile,allmodes)
         do i = 1,allmodes
            ctrl%selectmode(i) = 1
         end do
         ready = .true.
     else if(len_trim(zeile).eq.0) then 
        valid = .true.
     else if(zeile(1:5).eq.'dtype') then
         read (zeile(6:132),*)  ctrl%disptype
     else if(zeile(1:1) .eq. 'd'.and.len_trim(zeile).eq.1) then
         lfile = 7
         filename(1:lfile) = 'g98.out'
         write(*,*)
         write(*,*)' Normal modes will be read from file '//filename(1:lfile)
         write(*,*)
         call remod(ctrl,natoms,molecule,filename,lfile,allmodes)
         do i = 1,allmodes
            ctrl%selectmode(i) = 1
         end do
         ready = .true.
         ctrl%disptype=1
         intonly=.true.
     else if(zeile(1:1) .eq. '&') then
       goback=.true.
       return
     else 
        write(*,*)
        write(*,*)' Unknown command'
     endif
  dumsum = 0
  do i = 1,allmodes
    dumsum = dumsum + ctrl%selectmode(i) 
  end do
  ctrl%nummodes = dumsum
  end do
  end subroutine intonlymenu


!************************************************************************
  subroutine remod(ctrl,natoms,molecule,filename,lfile,allmodes)
!
!    read frequencies and normal coordinates from file filename
!
!    written by C. Herrmann, University of Bonn, 02/2005
!
    use snf_spectrum, only : type_peak
    use snf_control, only : type_ctrl
    implicit none
    type(type_atom),intent(in)    :: molecule(:)
    type(type_ctrl)    :: ctrl
    integer       :: mxitem,allmodes
    parameter(mxitem=10)
    character(len=30)             :: filename,item(mxitem)
    character(len=132)            :: row
    character                     :: bla
    integer                       :: lfile,unit,i,j,natoms,info,system
    double precision              :: freq(natoms*3),normmode(natoms*3,natoms*3)
    logical                       :: ex
    unit=10
!-------------------------------------------------------------------
!  check existence of file
!-------------------------------------------------------------------
   inquire(file=filename(1:lfile),exist=ex) 
   if(.not.ex) then
     write(*,*) ' The file '//filename(1:lfile)//' does not exist.'
     write(*,*)
     return
   end if
!-------------------------------------------------------------------
!  if modes are going to be read from 'g98.out', backup this file 
!   - otherwise, it might be overwritten in a subsequent snfint run 
!-------------------------------------------------------------------
    if(filename(1:lfile).eq.'g98.out') then
      info=system('cp g98.out g98.out.orig')
    end if 
!-------------------------------------------------------------------
!  initialize some variables
!-------------------------------------------------------------------
    natoms= ubound(molecule,1)
    do i = 1,natoms*3
      freq(i) = 0.d0
    end do
    j = 1
    open(unit,file=filename(1:lfile),status='unknown')
!-------------------------------------------------------------------
!  search for frequencies, read frequencies and normal coordinates
!-------------------------------------------------------------------
65    continue
      read(unit,'(a)',end=100) row
       call split(row,item,mxitem)
       if(item(1)(1:11).eq.'Frequencies') then
        read(row(17:132),*,end = 90) freq(j),freq(j+1),freq(j+2)  
90      continue      
        do i = 1,6
          read(unit,'(a)',end=100) row
        end do
        do i = 1,natoms
          read(unit,'(a)',end=100) row
          read(row(10:132),*,end=95) normmode(j,i*3-2),  &
  &                                  normmode(j,i*3-1),  &
  &                                  normmode(j,i*3),    &
  &                                  normmode(j+1,i*3-2),&
  &                                  normmode(j+1,i*3-1),&
  &                                  normmode(j+1,i*3),  &
  &                                  normmode(j+2,i*3-2),&
  &                                  normmode(j+2,i*3-1),&
  &                                  normmode(j+2,i*3)
95      continue
        end do
        if(freq(j+1).eq.0.d0) then
          j = j+1
        else if(freq(j+2).eq.0.d0) then
          j = j+2
        else
          j = j+3
        end if
        end if  
      goto 65
100   continue
    close(unit)
  write(*,*) 
  allmodes = j-1
  ctrl%freqin = freq
  ctrl%nmodein = normmode
  end subroutine remod

!************************************************************************
  subroutine outmenu(ctrl,goback)
  !----------------------------------------------------------------
  ! output menu
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  ! modified by M. Reiher, Uni Erlangen, June 2003
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  character(len=132) :: zeile
  logical            :: valid,goback,all,ex
  valid=.false.
  goback=.false.
  all=.false.
  if(ctrl%ramanew.ge.1) ctrl%print_rcs = .true.
  if(ctrl%vroaew.ge.1) ctrl%print_vroa = .true.
  do while(.not. valid)
     write(*,*)
     write(*,*)' Output options and DCINPUT menu'
     write(*,*)
     write(*,*)' You may choose one of the following commands:'
     call switchbox(6,ctrl%print_irred_at,'iat   ','information about symmetry redundant atoms')
     call switchbox(6,ctrl%print_irred_dist,'idt   ','information about symmetry redundant distortions')
     call switchbox(6,ctrl%print_hessian,'hess  ','output of cartesian hessian')
     call switchbox(6,ctrl%sumdeg,'sumdeg','sum intensities for degenerate modes')
     if(ctrl%ramanew.ge.1) call switchbox(6,ctrl%print_rcs,'rcs   ','output of raman cross sections')
     if(ctrl%vroaew.ge.1) call switchbox(6,ctrl%print_vroa,'vroa  ','output of VROA data')
     call switchbox(6,ctrl%print_gaussian,'gau   ','gaussian fake output')
     call switchbox(6,ctrl%print_mopac,'mopac ','mopac fake output')
     if(ctrl%print_mopac) call switchbox(6,ctrl%print_mopxmol,'moxmol',&
       &'mopac fake output readable by xmol')
     call switchbox(6,ctrl%print_xmol,'xmol  ','output of xmol XYZ input file')
     if(all) then
       call switchbox(6,ctrl%print_symmetry,'sym   ','general symmetry informations')
       call switchbox(6,ctrl%print_cioslowski,'cio   ',&
        &'cioslowski charge decomposition')
       call switchbox(6,ctrl%print_zpe,'zpe   ','zeropoint energy output')
       call switchbox(6,ctrl%print_thermo,'thermo','detailed thermodynamic output')
       call switchbox(6,ctrl%print_transrot,'trm   ',&
        & 'translational and rotational contributions output')
       call switchbox(6,ctrl%print_force,'force ','output of force constant matrix')
       call switchbox(6,ctrl%print_dip,'dip   ','output of cartesian dipol derivates')
       call switchbox(6,ctrl%tmpdirclean,'tmpcl ','removal of temporary directories for single point calculations')
       call switchbox(6,ctrl%logdirclean,'logcl ','removal of logfiles for slave processes')
       if(ctrl%ramanew.ge.1) call switchbox(6,ctrl%print_pol,'pol   ', &
         & 'output of cartesian polarizability derivates')
       if(ctrl%vroaew.ge.1) call switchbox(6,ctrl%print_pol,'pol   ', &
         & 'output of cartesian polarizability derivates')
       if(ctrl%vroaew.ge.1) call switchbox(6,ctrl%print_gten_lao,'gten/lao ', &
         & 'output of cartesian G tensor (LAO) derivatives')
       if(ctrl%vroaew.ge.1) call switchbox(6,ctrl%print_gten_ao,'gten/ao ', &
         & 'output of cartesian G tensor (AO) derivatives')
       if(ctrl%vroaew.ge.1) call switchbox(6,ctrl%print_aten,'Aten  ', &
         & 'output of cartesian A tensor derivates')
       write(*,*)'   elig <PATH>: set path to ELIGIBLE file;'
       write(*,*)'                current setting: '//trim(adjustl(ctrl%eligible))
     end if
     write(*,*)'   bak        : create backup copy of original mos file'
     if(.not.all) write(*,*)'   all        : show all options'
     if(all) write(*,*)'   all off    : show reduced set of options'
     write(*,*)'   &          : go back to last menu'
     write(*,*)'   >return<   : quit menu'
     write(*,*)
     read(*,'(a)') zeile
     if((zeile(1:3).eq.'sym')) then
        call getswitch(zeile,ctrl%print_symmetry)
     else if((zeile(1:3)).eq.'iat') then
        call getswitch(zeile,ctrl%print_irred_at)
     else if((zeile(1:3)).eq.'idt') then
        call getswitch(zeile,ctrl%print_irred_dist)
     else if((zeile(1:3)).eq.'cio') then
        call getswitch(zeile,ctrl%print_cioslowski)
     else if((zeile(1:5)).eq.'force') then
        call getswitch(zeile,ctrl%print_force)
     else if((zeile(1:4)).eq.'hess') then
        call getswitch(zeile,ctrl%print_hessian)
     else if((zeile(1:3)).eq.'dip') then
        call getswitch(zeile,ctrl%print_dip)
     else if((zeile(1:6)).eq.'sumdeg') then
        call getswitch(zeile,ctrl%sumdeg)
     else if((zeile(1:3)).eq.'pol') then
        call getswitch(zeile,ctrl%print_pol)
     else if((zeile(1:3)).eq.'rcs') then
        call getswitch(zeile,ctrl%print_rcs)
     else if((zeile(1:3)).eq.'trm') then
        call getswitch(zeile,ctrl%print_transrot)
     else if((zeile(1:3)).eq.'zpe') then
        call getswitch(zeile,ctrl%print_zpe)
     else if((zeile(1:6)).eq.'thermo') then
        call getswitch(zeile,ctrl%print_thermo)
     else if((zeile(1:3)).eq.'gau') then
        call getswitch(zeile,ctrl%print_gaussian)
     else if((zeile(1:4)).eq.'xmol') then
        call getswitch(zeile,ctrl%print_xmol)
     else if((zeile(1:5)).eq.'tmpcl') then
        call getswitch(zeile,ctrl%tmpdirclean)
     else if((zeile(1:5)).eq.'logcl') then
        call getswitch(zeile,ctrl%logdirclean)
     else if((zeile(1:5)).eq.'mopac') then
        call getswitch(zeile,ctrl%print_mopac)
     else if((zeile(1:6)).eq.'moxmol') then
        call getswitch(zeile,ctrl%print_mopxmol)
     else if((zeile(1:3)).eq.'bak') then
        call bfile('mos')
     else if((zeile(1:4)).eq.'elig') then
        inquire(file=zeile(5:),exist=ex)
        if(ex) then
          ctrl%eligible=trim(adjustl(zeile(5:)))
        else
          write(*,*) 'No such ELIGIBLE file: ',zeile(5:)
        end if
     else if((zeile(1:14)).eq.'werhatsgemacht') then
        call pr_snfdefhead
     else if((zeile(1:7)).eq.'all off') then
        all=.false.
     else if((zeile(1:3)).eq.'all') then
        all=.true.
     else if((zeile(1:1)).eq.'&') then
        goback=.true.
        return
     else if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        if(len_trim(zeile).ne.0) write(*,*)' Unknown command'
     endif
  enddo
  end subroutine outmenu

!************************************************************************

  subroutine check_jobfile(jobfile)
  !----------------------------------------------------------------
  ! check file containing total electronic energy 
  !
  ! written by J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  character(len=*) :: jobfile
  character(len=132) :: zeile
  logical           :: ex
  inquire(file=jobfile, exist=ex)
  if(.not.ex) then
    write(*,*)
    write(*,'(3a)') '  Cannot find file: ',jobfile(1:len_trim(jobfile)),&
     &' to read electronic energy.'
    write(*,*) ' (This is only necessary to get the correct total energy.)'
    write(*,*) ' You may enter an alternative filename to read the &
     &electronic energy from'
    write(*,*) ' (otherwise it is set to zero).'
    write(*,*)
    read(*,'(a)') zeile
    if(.not.(len_trim(zeile).eq.0)) then
      jobfile=''
      write(jobfile,'(a)') zeile(1:len_trim(zeile))
      write(*,*)
      write(*,'(2a)') '  Trying to read total energy from file: ',&
       &trim(adjustl(jobfile))
    end if
  end if
  end subroutine check_jobfile

!************************************************************************

  subroutine def_warnings(ctrl,crap)
  !----------------------------------------------------------------
  ! warnings, consistency checks
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)    :: ctrl
  logical            :: valid,crap
  character(len=132) :: zeile

  if(associated(ctrl%tm_ctrl)) then
   if (ctrl%tm_ctrl%scfconv.lt.8) then
    valid=.false.
    do while(.not. valid)
      write(*,*)
      write(*,'(a,i1)')'  Warning: $scfconv = ',ctrl%tm_ctrl%scfconv
      write(*,*)' We strongly recommend $scfconv > 7 !'
      write(*,*)
      write(*,*)'   Enter $scfconv-value to be used in this calculation'
      write(*,*)'   or press return to set default value ($scfconv = 8).'
      write(*,*)
      read(*,'(a)') zeile
      zeile=trim(adjustl(zeile))
      if(string2integer(zeile).gt.0.and.string2integer(zeile).le.10) then
        ctrl%tm_ctrl%scfconv = string2integer(zeile)
        valid = .true.
      else if(string2integer(zeile).gt.10) then
        write(*,*)
        write(*,*)' $scfconv too large'
        write(*,*)
      else if(zeile.eq.'') then
        ctrl%tm_ctrl%scfconv = 8
        valid = .true.
      end if
    end do
   endif
  endif
  if(ctrl%cstep.lt.0.01) then
     call serror('snf_define',0,'ctrl%cstep is critically small !!!')
  end if
  crap=.false.
  if(ctrl%cstep.lt.0.01.or.ctrl%cstep.gt.0.05) crap=.true.
  if(associated(ctrl%tm_ctrl)) then 
    if(ctrl%tm_ctrl%scfconv.lt.8) crap=.true.
  end if
  if(crap) then
     write(*,*)
     write(*,*) ' *************************************************************'
     write(*,*)'   Note: <CRAP_OK yes> must be set in DCINPUT input,'
     if(ctrl%cstep.lt.0.01) then
        write(*,*)'         since cstep < 0.01 !!!'
     else if(ctrl%cstep.gt.0.05) then
        write(*,*)'         since cstep > 0.05 !!!'
     else
        write(*,*)'         since $scfconv < 8 !!!'
     end if
     write(*,*) ' *************************************************************'
     write(*,*)
  end if
  if(ctrl%ramanew.ge.1) then
    if(.not.ramanpos(ctrl%progno,ctrl%calcno)) then
      call serror('snf_define',0,'Raman calculations not available for this##&
       & program mode. Raman is switched off')
      ctrl%ramanew=0
      if(associated(ctrl%dynfreq)) deallocate(ctrl%dynfreq)
    end if
  end if
  end subroutine def_warnings

!************************************************************************
  subroutine prepdc(crap,eligdir)

!-----------------------------------------------------------------------
! prepare inputfiles USE_NODES and DCINPUT for SNFDC.
! furthermore, the files MACHINES and mpi.sub are
! created, which may be used for the MPI version of SNFDC.
!
! written by J. Neugebauer, Uni Erlangen, 2002
!-----------------------------------------------------------------------
  implicit none
  integer info,system,info2,li,ncpu,info3
  character(len=255) buf,eligdir
  logical crap
  parameter(li=1)
  write(*,*) 
  write(*,*) '===================================='
  write(*,*) '==== CREATING SNFDC INPUT FILES ===='
  write(*,*) '===================================='
  write(*,*) 
  write(*,*) '... creating file MACHINES for MPI-MACHINERY'
  if(eligdir.ne.'$CIPROC/ELIGIBLE') then
    write(buf,'(A)') 'choose_nodes -e '//trim(adjustl(eligdir))
    info=system(buf)
  else
    info=system('choose_nodes')
  end if
  if(info.ne.0) call serror('prepdc',1,'choose_nodes failed! Make &
  &sure that choose_nodes is available in your path!')
  ncpu=0
  open(unit=li,file='USE_NODES')
  do i=1,3
    read(li,'(A)',end=999) buf
  end do
10 read(li,'(A)',end=999) buf
  if(index(buf,'# chosen').ne.0) goto 20
  goto 10
999 call serror('prepdc',1,'Could not determine number of nodes !')
  goto 30
20 read(buf(index(buf,'with')+4:index(buf,'proc')-1),*) ncpu
30 close(li)
  if(eligdir.ne.'$CIPROC/ELIGIBLE') then
    write(buf,'(A)') 'choose_nodes -mpi -e '//trim(adjustl(eligdir))
    info=system(buf)
  else
    info=system('choose_nodes -mpi')
  end if
  info2=system('mv USE_NODES MACHINES')
  if(info.ne.0.or.info2.ne.0) call serror('prepdc',1,'choose_nodes &
  &failed! Make sure that choose_nodes is available in your path!')
  write(*,*) '... creating file mpi.sub for MPI-MACHINERY'
  open(unit=li,file='mpi.sub')
!---------------------------------------------------------------------------
!  write(li,'(A,i3,A)') '$MPICH/bin/mpirun -machinefile MACHINES -np ',ncpu,&
!  &' snfdcmpi'
  write(li,'(A,i3,A)') 'mpirun -machinefile MACHINES -np ',ncpu,&
  &' snfdc'
!---------------------------------------------------------------------------
  close(li)
  info=system('chmod u+x mpi.sub')
  if(info.ne.0) call serror('prepdc',1,'Error while preparing mpi.sub')
  write(*,*) '... creating file USE_NODES for PVM-MACHINERY'
  if(eligdir.ne.'$CIPROC/ELIGIBLE') then
    write(buf,'(A)') 'choose_nodes -e '//trim(adjustl(eligdir))
    info=system(buf)
  else
    info=system('choose_nodes')
  end if
  if(info.ne.0) call serror('prepdc',1,'choose_nodes failed! Make &
  &sure that choose_nodes is available in your path!')
  write(*,*) '... creating file DCINPUT (using script mkdcinput)'
  info=system('mkdcinput 2>/dev/null')
  info2=0
  if(crap) info2=system('echo CRAP_OK yes >> DCINPUT')
  if(eligdir.ne.'$CIPROC/ELIGIBLE') then
    write(buf,'(A)') 'echo ELIGPATH '//trim(adjustl(eligdir))//' >> DCINPUT'
    info3=system(trim(adjustl(buf)))
  end if
  if(info.ne.0.or.info2.ne.0.or.info.ne.0) &
  &call serror('prepdc',1,'mkdcinput failed! Make &
  &sure that mkdcinput is available in your path!')
  write(*,*) 
  write(*,*) 'File mpi.sub: '
  write(*,*) 
  info=system('cat mpi.sub')
  write(*,*) 
  write(*,*) 'File DCINPUT: '
  write(*,*) 
  info=system('cat DCINPUT')
  write(*,*) 
  write(*,*) '===================================================='
  write(*,*) '=== change the parameters in DCINPUT and mpi.sub ===' 
  write(*,*) '=== according to your needs if necessary         ==='
  write(*,*) '=== (press <return> to continue)                 ==='
  write(*,*) '===================================================='
  write(*,*) 
  read(*,*) 
  end subroutine prepdc

!************************************************************************

subroutine pr_snfdefhead
  implicit none
  include '../snfdc/VERSION.h'
  write(*,*)
  write(*,*)' *************************************************************'
  write(*,*)' *                                                           *'
  write(*,*)' *                         SNFDEFINE                         *'
  write(*,*)' *                                                           *'
  write(*,'(A,a21,a)') &
   &       '  *                   ',version,          '                   *'
  write(*,*)' *                                                           *'
  write(*,*)' *      J. Neugebauer, C. Kind, M. Reiher, and B.A. Hess     *'
  write(*,*)' *                                                           *'
  write(*,*)' *                University of Bonn 2004                    *'
  write(*,*)' *                                                           *'
  write(*,*)' *************************************************************'
  write(*,*)
  write(*,'(A)')
  write(*,'(2A)') ' Last modification: ',lastmod
  write(*,'(A)')
end subroutine pr_snfdefhead


!************************************************************************

      subroutine old_snfcont(ctrl,selatom,massatom,molecule,c,pointgrp)
      !----------------------------------------------------------------
      ! check, whether the user wants to use an old snf_control-file
      !
      ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
      !----------------------------------------------------------------

      implicit none
      type(type_ctrl),intent(inout) :: ctrl
      type(type_atom),pointer       :: molecule(:)
      logical                       :: valid,nocheck
      character(len=132)            :: zeile
      character(len=3)              :: pointgrp,oripg
      character                     :: c
      integer,pointer               :: selatom(:)
      real(double),pointer          :: massatom(:)

      valid=.false.
      do while(.not. valid)
         write(*,*)
         write(*,*)' File snf_control already exists!'
         write(*,*)' Would you like to change the parameters (y/n)?'
         write(*,*)' (>return< = default = n)'
         write(*,*)
         read(*,'(a)')zeile
         zeile=trim(adjustl(zeile))
         if(len_trim(zeile).eq.0) then
            valid = .true.
            c='n'
         elseif(len_trim(zeile) .le. 1) then
            if(index('yn',trim(zeile)).ne.0) then
               valid=.true.
               c=zeile(1:1)
            endif
         else
            write(*,*)
            write(*,*)' Unknown command'
            write(*,*)
         endif
      end do
     if(c.eq.'n') then
     !----------------------------------------------------------------
     ! take snf_control data and go to the end of interactive part
     !----------------------------------------------------------------
     call read_control(ctrl,selatom,massatom)
     if(associated(ctrl%tm_ctrl)) then
        call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
     else if(associated(ctrl%dal_ctrl)) then
        call read_dal_coord(ctrl,molecule)
     else if(associated(ctrl%adf_ctrl)) then
        call read_adf_coord(ctrl,molecule)
     else
       call serror('snf_define',2,'Only TURBOMOLE coord-files can be&
          & read so far !')
     endif
     !----------------------------------------------------------------
     ! test isosym, if specified
     !----------------------------------------------------------------
     if(ctrl%isosym.ne.'def') then
        write (*,*)
        write (*,*) ' Checking molecular symmetry considering different isotopes'
        if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%isosym)) then
           call get_point_grp(molecule,1.D-3,atom_equal,pointgrp)
           write(*,*)
           write(*,*)'Point-group ',ctrl%isosym,' does not fit to coordinates.'
          write(*,*)'Automatic point-group check yields: ',pointgrp
           ctrl%isosym=pointgrp
        else
           write(*,*)' Symmetry: ',ctrl%isosym, ' is validated.'
        endif
     endif
     write (*,*)
     write (*,*) ' Checking molecular symmetry. Ignoring different isotopes. '
     if(.not.is_point_grp(molecule,1.D-3,atom_equal,ctrl%symmetry)) then
       call print_molecule(molecule)
       write(*,*)
       write(*,*)'Point-group ',Ctrl%symmetry,' does not fit to coordinates.'
       write(*,*)'(Turbomole standard orientation is assumed !)'
       nocheck=.false.
       if (ctrl%progno.eq.2) call test_dal_sym(nocheck)
       if (.not.nocheck) then
         call get_point_grp(molecule,1.D-3,atom_equal,oripg)
         call get_rpoint_grp(molecule,1.D-3,atom_equal,pointgrp)
         write(*,*)
         write(*,*)'Automatic point-group check yields: ',pointgrp
       end if
       if(pointgrp.ne.oripg) then
        write(*,*)
        if(ctrl%progno.eq.1) then
          call bfile(ctrl%tm_ctrl%coord)
          call write_tm_coord(ctrl%tm_ctrl%coord,molecule)
        else if(ctrl%progno.eq.2) then
          call bfile(ctrl%dal_ctrl%coord)
        end if
        call serror('snf_define',0,'Note that a coordinate transformation was '//&
        &'necessary to verify##this point group. A define run is mandatory to get new MOs !')
       end if
       ctrl%symmetry=pointgrp
     else
        write(*,*)' Symmetry: ',ctrl%symmetry, ' is confirmed.'
     endif
     molecule(:)%changem=.false.
     if(associated(selatom)) call setisomass(molecule,selatom,massatom)
  endif
  end subroutine old_snfcont

!************************************************************************


  subroutine alt_cont(control,found_control)
  !----------------------------------------------------------------
  ! the user may enter the name of an alternative snf_control file
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  !----------------------------------------------------------------
  implicit none
  logical :: valid, found_control
  character(len=132) :: zeile,errbuff
  character(len=*)   :: control
  valid=.false.
  do while(.not. valid)
     write(*,*)
     write(*,*)' If you want to read default-data from another snf_control file,'
     write(*,*)' then enter its location/name or otherwise hit >return< '
     if(fconhere) write(*,*) ' to take data from snf_control as default values.'
     write(*,*)
     read(*,'(a)') zeile
     if(len_trim(zeile).eq.0) then
        valid=.true.
     else
        control=trim(adjustl(zeile));
        found_control=.false.
        do i=1,len_trim(control)
           if(index('./',control(i:i)).eq.0) then
              inquire(file=control,exist=found_control)
              exit
           end if
        end do
        if(found_control) then
           valid=.true.
        else
           write(errbuff,'(2a)') 'Cannot find snf_control file:##',&
            &trim(adjustl(control))
           call serror('snf_define',0,errbuff)
           errbuff=''
        endif
     endif
  end do
  end subroutine alt_cont

!************************************************************************

  subroutine selprog(ctrl,molecule,goback)
  !----------------------------------------------------------------
  ! Select program to be used for the single point calculations
  !
  ! written by C. Kind and J. Neugebauer, Uni Erlangen, 2001
  ! modified by M. Reiher, Uni Erlangen, June 2003
  !----------------------------------------------------------------
  implicit none
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  integer                 :: lw
  character(len=132)      :: zeile
  character(len=10)       :: defz
  logical valid,goback,ex,nocheck

  goback=.false.
  valid=.false.
  inquire(file='adf.in',exist=ex)
  if(ex) then
    defz='adf       '
    ctrl%progno=4
  end if
  inquire(file='gaussian.com',exist=ex)
  if(ex) then
    defz='g03       '
    ctrl%progno=3
    ctrl%calcno=2
  end if
  inquire(file='molecule.mol',exist=ex)
  if(ex) then
    defz='dalton    '
    ctrl%progno=2
  end if
  inquire(file='control',exist=ex)
  if(ex) then
    defz='tm control'
    ctrl%progno=1
  end if

  do while(.not. valid)
     write(*,*)
     write(*,*)' Which program would you like to use:'
     write(*,*)'   tm [<control>]        : TURBOMOLE [control-file with input settings]'
     write(*,*)'   dalton                : DALTON'
     write(*,*)'   g98                   : GAUSSIAN98'
     write(*,*)'   g03                   : GAUSSIAN03'
     write(*,*)'   adf                   : ADF'
     write(*,*)'   &                     : go back to control file selection'
     write(*,*)'   >return<              : quit menu and accept default settings'
     write(*,*)'   Default: '//defz
     write(*,*)
     read(*,'(a)') zeile
     zeile=trim(adjustl(zeile))
     if(associated(ctrl%adf_ctrl)) deallocate(ctrl%adf_ctrl)
     if(associated(ctrl%gauss_ctrl)) deallocate(ctrl%gauss_ctrl)
     if(associated(ctrl%dal_ctrl)) deallocate(ctrl%dal_ctrl)
     if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
10   continue
     if(len_trim(zeile).eq.0) then
       zeile(1:10)=defz(1:10)
       goto 10
     else if(zeile(1:2).eq.'tm') then
        lw=2
        zeile='tm control'
        call gettmin(ctrl,molecule,valid,zeile,lw)
        ctrl%progno=1
        ctrl%calcno=ctrl%tm_ctrl%calcno
        valid=.true.
     else if(zeile(1:6).eq.'dalton') then
        allocate(ctrl%dal_ctrl)
        write(*,*)
        write(*,*)' Select type of input files for DALTON calculation: '
        write(*,*)
        write(*,*)'   tm <path/control> : DALTON input created from TURBOMOLE files'
        write(*,*)'   dal               : original DALTON input files:'
        write(*,*)'                       (molecule.dal and molecule.mol)'
        write(*,*)'   >return<          : quit menu and accept default settings'
        write(*,*)
        write(*,*)'  (Default: dal)'
        write(*,*)
        read(*,'(a)') zeile
        zeile=trim(adjustl(zeile))
        valid=.true.
        ctrl%progno = 2
        call init_dal_ctrl(ctrl%dal_ctrl)
        if(len_trim(zeile).eq.0.or.zeile(1:3).eq.'dal') then
          call testfile('molecule.mol')
          call testfile('molecule.dal')
          call read_dal_coord(ctrl,molecule)
          call read_dal_ctrl(ctrl)
          ctrl%calcno=ctrl%dal_ctrl%calcno
!... test whether symmetry tests should be omitted for DALTON run
          call test_dal_sym(nocheck)
          if (.not.nocheck) call checkdalgeo(molecule,ctrl)
        else if(zeile(1:2).eq.'tm') then
          ctrl%dal_ctrl%udalinp=.false.
          lw = 2
          call gettmin(ctrl,molecule,valid,zeile,lw)
        else
          valid=.false.
          cycle
        end if
     else if(zeile(1:3).eq.'g98') then
        allocate(ctrl%gauss_ctrl)
        ctrl%progno=3
        ctrl%calcno=1
        call init_gauss_ctrl(ctrl%gauss_ctrl)
        call testfile('gaussian.com')
        call read_gauss_coord(ctrl,molecule)
        valid=.true. 
     else if(zeile(1:3).eq.'g03') then
        allocate(ctrl%gauss_ctrl)
        ctrl%progno=3
        ctrl%calcno=2
        call init_gauss_ctrl(ctrl%gauss_ctrl)
        call testfile('gaussian.com')
        call read_gauss_coord(ctrl,molecule)
        valid=.true. 
     else if(zeile(1:3).eq.'adf') then
        allocate(ctrl%adf_ctrl)
        ctrl%progno=4
        call init_adf_ctrl(ctrl%adf_ctrl)
        call testfile('adf.in')
        call read_adf_coord(ctrl,molecule)
        call read_adf_ctrl(ctrl)
        ctrl%calcno=ctrl%adf_ctrl%calcno
        call check_adf_settings
        valid=.true. 
     else if(zeile(1:6).eq.'molpro') then
     call serror('snf_define',0,'MOLPRO is still not implemented!##&
		     &TURBOMOLE will be used instead.')
        lw = 6
        call gettmin(ctrl,molecule,valid,zeile,lw)
        ctrl%progno=1
        ctrl%calcno=ctrl%tm_ctrl%calcno
        valid=.true.
     else if(zeile(1:1).eq.'&') then
        goback=.true.
        return
     end if
     if(valid) then
       if(ctrl%progno.eq.1) then
         ctrl%tm_ctrl%calcno=ctrl%calcno
       else if(ctrl%progno.eq.2) then
         ctrl%dal_ctrl%calcno=ctrl%calcno
       else if(ctrl%progno.eq.3) then
         ctrl%gauss_ctrl%calcno=ctrl%calcno
       else if(ctrl%progno.eq.4) then
         ctrl%adf_ctrl%calcno=ctrl%calcno
       end if
     end if
  end do
  end subroutine selprog

!************************************************************************

  subroutine gettmin(ctrl,molecule,valid,zeile,lw)
 !----------------------------------------------------------------
 ! Get basic information from Turbomole control file
 !
 !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 ! re-written by J. Neugebauer, Uni Erlangen, 2003
 !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
 ! written by J. Neugebauer, Uni Erlangen, 2001
 !----------------------------------------------------------------
  implicit none
  integer                 :: lw
  type(type_ctrl)         :: ctrl
  type(type_atom),pointer :: molecule(:)
  character(len=132)      :: zeile,errbuff
  logical valid,extmc
  if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
  allocate(ctrl%tm_ctrl)
  if(len_trim(zeile).le.lw) then
     write(*,*)
     write(*,*) ' No input filename given, so default settings will be used'
     write(*,*) ' (Default: control)'
     write(*,*)
     if(associated(ctrl%tm_ctrl)) deallocate(ctrl%tm_ctrl)
     allocate(ctrl%tm_ctrl)
     call init_tm_ctrl(ctrl%tm_ctrl)
     call testfile(ctrl%tm_ctrl%control)
     call read_tm_ctrl(ctrl%tm_ctrl)
     call copy_tm_to_ctrl(ctrl)
     call testfile(ctrl%tm_ctrl%coord)
     call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
     valid=.true.
  else
     extmc=.false.
     do i=3,len_trim(zeile((lw+1):))
        if(index('./',trim(adjustl(zeile(lw+i:lw+i)))).eq.0) then
           inquire(file=trim(adjustl(zeile((lw+1):))), exist=extmc)
           exit
        end if
     end do
     if (extmc) then
        call init_tm_ctrl(ctrl%tm_ctrl)
        ctrl%tm_ctrl%control=trim(adjustl(zeile((lw+1):)))
        call testfile(ctrl%tm_ctrl%control)
        call read_tm_ctrl(ctrl%tm_ctrl)
        call copy_tm_to_ctrl(ctrl)
        call testfile(ctrl%tm_ctrl%coord)
        call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
        valid=.true.
     else
        write(errbuff,'(2a)') 'Cannot find TURBOMOLE control file:##',&
         &trim(adjustl(zeile((lw+1):)))
        call serror('snf_define',0,errbuff)
        errbuff=''
     end if
  end if
  end subroutine gettmin

!************************************************************************
      end program snfdefine
