module snf_gaussmol
  !------------------------------------------------------------------------
  ! written by C. Herrmann, Uni Jena, April-June 2005
  !------------------------------------------------------------------------
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param

  private
  public ::  rwgaussmol, read_gauss_coord 

  !*********************************************************************

contains


      subroutine rwgaussmol(natoms,ic,co,mxat)
!
! read/write basic molecular input information from gaussian.com file
!
! written by C. Herrmann, Uni Jena 2005.
!
      use snf_molecule, only : find_nucl_charge
      implicit none
      integer mxat,mxsh,mxitem,gunit,lrow,nempty,j
      parameter(mxsh=50,mxitem=10)
      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      character(len=132) :: row,item(mxitem)
      character(len=2) el(mxat)
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos

! coordinates are given in Angstroem (Gaussian default), 
! so convert them to Bohr (SNF internal standard)
      open(gunit,file='gaussian.com',status='unknown')
1     read(gunit,'(a132)',end=2) row
      call llen(row,lrow)
      if(row(1:10).eq.'          ') nempty = nempty +1
      if(nempty.eq.2) then
      i=1
      read(gunit,'(a132)',end=2) row
3     read(gunit,'(a132)',end=2) row
      if(row(1:10).ne.'          ') then
        read(row,*) el(i), co(1,i), co(2,i), co(3,i)
        ic(i)=find_nucl_charge(el(i))
        do j=1,3
           co(j,i) =  co(j,i)*(1/bohr2ang)
        end do
        i=i+1
        goto 3
      end if
      end if
      goto 1
      close(gunit)
2     continue
      natoms = i-1

      end subroutine rwgaussmol

!************************************************************************
      subroutine read_gauss_coord(ctrl,molecule)
!
! read molecule information from the Gaussian input file gaussian.com
!
! written by C. Herrmann, University of Jena, 2005
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_atom,find_element,find_mass,fiso_mass
      implicit none
      type(type_ctrl) :: ctrl
      type(type_atom),pointer :: molecule(:)
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,calcno,nfreq
      parameter(mxat=1000)
      real(double) :: co(3,mxat)
      character(len=80) form,form2
      character(len=10) :: progtype,calctype
      integer :: ic(mxat)
      li=1
      !--------------------------------------------------------
      ! read geometry
      !--------------------------------------------------------
      call bfile('gaussian.com')
      call rwgaussmol(natoms,ic,co,mxat)
      allocate(molecule(natoms))
      molecule(:)%changem=.false.
      do i=1,natoms
        molecule(i)%cartcoord(:)=co(:,i)
        molecule(i)%nucl_charge=ic(i)
        molecule(i)%element=find_element(ic(i))
        if(molecule(i)%element.eq.'D') then
          molecule(i)%mass=fiso_mass(1,2)
        else
          molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end if
      end do

      end subroutine read_gauss_coord

!************************************************************************

end module snf_gaussmol
