
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
program snfint
  use f90_kind
  !----------------------------------------------------------------------
  ! this program needs the files "coord" and "control". and what is more
  ! important, it needs the file "restart" with all raw data.
  ! from that point the derivatives of dipole moments, polarizabilities
  ! and VROA-raw data along normal modes
  ! are calculated as avaible from an c1 calculation. 
  ! finally the intensities 
  ! are collected and sorted in one global spectrum. 
  !
  ! written by C. Herrmann, Uni Bonn, 02/2005
  ! strongly relying on snf
  !   written by Carsten Kind, Markus Reiher and Johannes Neugebauer, 
  !              Uni Erlangen / Uni Bonn 1999-2004
  !----------------------------------------------------------------------
  use snf_spectrum,             only: type_peak, print_ir, destroy_spectrum, &
       & calc_frequencies, calc_ir_intensities,calc_raman_intensities,sort_spectrum, print_raman, print_ir_nosym,&
       & print_norm_modes,get_translation_and_rotation, &
       & zeropoint_energy,calc_mode_properties, &
       & get_relcross,sum_degen,print_rcs,get_relir,calc_ir_intensities_intonly
  use snf_molecule,       only: type_atom,type_molecule,print_molecule,&
    & destroy_molecule,hauptax_trans,print_isomass,isotope_equal,atom_equal,&
    & setisomass,testlin
  use snf_turbomole,      only: read_tm_coord
  use snf_char_table,     only: type_char_table,create_char_table,&
    & destroy_char_table,print_char_table,create_product_table,&
    & print_product_table
  use snf_representation, only: create_representation_for_vib,destroy_representation, &
       print_representation
  use snf_projection,     only: create_proj_op,destroy_proj_op,&
       print_proj_op
  use snf_restart,        only: read_restart,write_restart,backup_restart
  use snf_error,          only: serror
  use snf_xmol,           only: print_xmol_norm
  use snf_rdalmol,        only: read_dal_coord
  use snf_control,        only: type_ctrl,read_control,print_ctrl,&
       &                        init_ctrl,destroy_control,check_ctrl,&
                                read_control2
  use snf_dipol,          only: create_dipol_dxyz,print_dipol_dxyz, &
       &                        destroy_dipol_dxyz, expand_dipol, &
       &                        create_dipol_dq, symm_dipol,create_dipol_dq_intonly
  use snf_polar,          only: create_polar_dxyz,print_polar_dxyz, &
       &                        destroy_polar_dxyz, expand_polar, &
       &                        create_polar_dq,create_polar_dq_intonly,symm_polar
  use snf_vroa,           only: create_aten_dxyz,print_aten_dxyz, &
       &                        create_aten_dq, create_aten_dq_intonly,&
       &                        calcvroa,symm_gten,expand_gten
  use snf_irred_dist,     only: create_irred_dist,create_red_grad,&
                                create_red_dip,create_red_pol,create_red_gten
  use snf_irred_atoms,    only: create_irred_atoms,destroy_irred_atoms,&
       &                        print_irred_atoms,check_geometry,get_point_grp,is_point_grp
  use snf_thermo,         only: get_tdvalues
  use snf_tools
  use snf_norm_modes
  use snf_gaussian  , only: print_gaussian_frequencies
  use snf_mopac     , only: print_mopac_frequencies
  use snf_cioslowski, only: create_cioslowski,print_cioslowski
  use snf_gnuplot,    only: generate_plots
  use snf_param
  use snf_symmat,     only: symel_mat
  use snf_dummy,      only: rm_dummies
  implicit none

  type(type_ctrl) :: ctrl,ctrl_restart  ! essential control flags and parameters
  type(type_atom), pointer :: &
       & molecule(:)     ! geometry atoms and atomic masses of the molecule  
  type(type_atom),allocatable :: mol(:)
  type(type_molecule)      :: molekel  ! information about the whole molecule
  real(double),pointer :: &
       & gradients(:,:,:)      ! energy gradients of the negative distortion (nvar,nvar)
  real(double),pointer :: &
       & dipols(:,:,:)         ! dipol moments of the negative distortion (nvar,3)
  real(double),pointer :: &
       & pols(:,:,:,:)           ! polarizabilities pols(1..8,nvar=3*n_atoms,&
       !&no_steps=no_distortions,no_frequencies(for dynamic polarizabilities)
  real(double),pointer :: gtenlao(:,:,:,:),gtenao(:,:,:,:),aten(:,:,:,:) ! VROA tensors
  real(double),allocatable :: &
       & distortion(:)         ! generalized distortion in the cartesian basis

  real(double),allocatable :: &
       & norm_modes(:,:)       ! transformation matrix from symmetryadapted
  !                              basis to normal coordinates (irrep,irrep)
  !                              transformation matrix from cartesian basis 
  !                              to normal coordinates  (nvar,irrep)
  real(double),pointer :: &
       & dipol_dxyz(:,:), &    ! derivatives of dipol moments in cartesian basis (nvar,3)
       & dipol_dq(:,:),   &    ! derivatives of dipol moments in normal coordinates (irrep,3)
       & polar_dxyz(:,:,:), &  ! derivatives of polarizability tensor (first dimension: 6) in cartesian basis (nvar,?)
       & polar_dq(:,:,:), &    ! derivatives of polarizability tensor in normal coordinates (irrep,?)
       & polar9_dq(:,:,:), &   ! derivatives of polarizability tensor (first dimension: 9) in normal coordinate basis (nvar,?)
       & gtenlao_dxyz(:,:,:), &! derivatives of G/LAO tensor in cartesian basis (nvar,?)
       & gtenlao_dq(:,:,:), &  ! derivatives of G/LAO tensor in normal coordinates (irrep,?)
       & gtenao_dxyz(:,:,:), & ! derivatives of G/AO tensor in cartesian basis (nvar,?)
       & gtenao_dq(:,:,:), &   ! derivatives of G/AO tensor in normal coordinates (irrep,?)
       & aten_dxyz(:,:,:), &   ! derivatives of A tensor in cartesian basis (nvar,?)
       & aten_dq(:,:,:)        ! derivatives of A tensor in normal coordinates (irrep,?)

  real(double),pointer :: &
       & representation(:,:,:),&! representations of symmetry operations s -->(s,:,:)
       & proj_op(:,:,:), &     ! projectors on irreps i-->(i,:,:)
       & character(:)          ! characters of the representations

  integer,pointer :: &
       & irreps(:)             ! vector with number of vectors per irrep
  type(type_char_table) :: &
       & char_table            ! character table
  type(type_peak), pointer :: &
       & spectrum(:), &       ! spectral information  with frequencies,modes and intensities 
       & sum_spectrum(:)      ! spectral information, summed over degenerate modes
  integer,pointer          :: &
       &  istepfl(:), &      ! points to calculate(=6*nirred_atoms)
       & atom_index(:), &    ! index of irreducible atom (1:natoms)
       & symm_index(:), &    ! symmetry op. to optain irred. atom (1:natoms)
       & selatom(:), &       ! atom-numbers which special isotopes are selected for
       & irred_atom_index(:) ! index list with irreducible atoms (1:nirred_atoms)

  integer,pointer          :: &
       & dist_index(:), &    ! index of irreducible dist. 
       & dsymm_index(:), &   ! symmetry op. to optain irred. dist. 
       & irred_dist_index(:) ! index list with irreducible dist. 

  character(len=8),pointer :: &
       & nvmach(:)           ! list of all used machines 
  character(len=132) :: errbuff ! buffer for error messages
  character(len=132) :: tbuf ! buffer for time messages

  real(double),allocatable :: eig_proj_op(:)
  real(double) :: zeropoint ! zero point energy
  real(double),pointer :: massatom(:) ! masses for special isotopes
  integer zeros,t1,t2,ltbuf

  real(double) dumlaserf(1),XFAMU,ddum
!MR NB: XFAMU is the conversion factor for u to a.u., i.e. 1822.88848; it is NOT identical
!MR     with the proton(electron mass ratio, which is 1836. ...
  parameter (XFAMU=atomic_mass_unit/el_mass)
 
  real(double)         :: mol_mass
  real(double),pointer :: freqs(:)
  
  character(len=7),pointer :: prod_table(:,:)


  real(double),pointer :: cioslowski(:) ! cioslowski charge decomposition
  ! less important
  integer :: i,ii,j,k,nirred_atoms,nvar,offset,nfreq,nirred_dist
  integer :: number_projected_modes,out
  logical :: new_restart,fdum,static,haflag,rflag,vroaflag
  ! diagonalization
  integer :: lwork,info,ilaenv,status
  real(double),allocatable :: dip_contr(:,:,:),pol_contr(:,:,:)
  integer :: csv,natoms

  real(double),allocatable :: normmodes(:,:)

  call secs(t1)
  csv=77
!JN  out=6
  out=44
  open(unit=out,file='snfint.out',status='unknown')
  call print_header(out)
  call last_mod(out)
  call snf_cite_as(out)
  nullify(molecule,gradients,dipols,pols,gtenlao,gtenao,aten,&
       &  polar_dq,           &
       &  gtenlao_dxyz,gtenlao_dq,gtenao_dxyz,gtenao_dq,     &
       &  aten_dxyz,aten_dq,polar9_dq,             &
       &  representation,proj_op,character,irreps,           &
       &  nvmach,                                 &
       &  irred_atom_index,symm_index,atom_index,istepfl,    &
       &  spectrum,sum_spectrum)
  !----------------------------------------------------------------
  ! read coord file, write data to "molecule"
  ! determine symmetry and distortion from control file
  ! read gradients, dipol moments, polarizabilities and
  ! VROA raw data from restart file
  !----------------------------------------------------------------
  call init_ctrl(ctrl)
  call testfile('snf_control')
  call testfile('restart')
  call read_control(ctrl,selatom,massatom)
  if(ctrl%progno.eq.1) then
!----------
! TURBOMOLE
!----------
    call testfile('control')
    call testfile('coord')
  else if(ctrl%progno.eq.2) then
!-------
! DALTON
!-------
    call testfile('molecule.dal')
    call testfile('molecule.mol')
  else if(ctrl%progno.eq.4) then
!-------
! ADF
!-------
     call testfile('adf.in')
  end if
  if(associated(ctrl%tm_ctrl)) then
     call read_tm_coord(ctrl%tm_ctrl%coord,molecule)
  else if(associated(ctrl%dal_ctrl)) then
     call read_dal_coord(ctrl,molecule)
  endif
  if(associated(selatom)) then
     call setisomass(molecule,selatom,massatom)
     deallocate(selatom,massatom)
  end if
  fdum=.false.
  do i=1,ubound(molecule,1)
    if(molecule(i)%element.eq.'Q') fdum=.true.
  end do
  ! calculate molecular mass and moments of inertia
  nvar=3*ubound(molecule,1)
  ctrl%nvar=nvar
!-------------------------------------------------
! allocate fields an read control for intonly mode
!-------------------------------------------------
  if(.not.associated(ctrl%nmodein)) allocate(ctrl%nmodein(nvar, &
 &nvar))
  allocate(ctrl%freqin(nvar*3))
  allocate(ctrl%selectmode(nvar*3))
  call read_control2(ctrl,ubound(molecule,1))

!-------------------------------------------------

  call print_ctrl(ctrl,unit=out)
  if(any(molecule(:)%changem)) call print_isomass(molecule,unit=out)
  call print_molecule(molecule,unit=out)

  if(associated(ctrl%dynfreq)) then
    nfreq=ubound(ctrl%dynfreq,1)
  else
    nfreq=1
  end if
  allocate(ctrl_restart%dynfreq(nfreq))
  rflag=ctrl%ramanew.ge.1
  vroaflag=ctrl%vroaew.ge.1
! raman and vroa modes should never be possible at the same time:
  if (rflag.and.vroaflag) stop " MIRACLE "
  call read_restart(ctrl%restart,ctrl_restart,&
       &   gradients,dipols,pols,gtenlao,gtenao,aten,&
       &    istepfl,nvmach,status,rflag,&
       &   vroaflag,unit=out)
  if(ctrl_restart%nummodes.eq.0) then
     write(*,*)
     write(*,*) ' ERROR: Restart file does not contain raw data'
     write(*,*) ' for distortion along normal modes,'
     write(*,*) ' thus evaluation of data with snfint not possible.'
     write(*,*) ' Maybe you should use snf for evaluation of your data.'
     stop
  end if 
  if(status.eq.-9) then
    call serror('snf',2,'Restart file does not exist.',out)
  else if(status.eq. -1) then
    call serror('snf',2,'Some calculations are not finished.',out)
  else if(status.eq. 0) then
    call serror('snf',2,'Not all calculations have been done yet.',out)
  else if(status.eq. -2) then
    call serror('snf',2,'Not all data necessary for frequency analysis##&
     &available in file restart.',out)
  endif
  call check_ctrl(ctrl_restart,ctrl,out)

  call destroy_control(ctrl_restart)
  if(fdum) then
    call serror('snf',-1,'Dummy atoms are neglected in all further &
     &evaluations',out)
    call rm_dummies(molecule,gradients,dipols,pols,gtenlao,gtenao,aten,nvmach,&
     & istepfl,2*ctrl%numderiv,rflag,vroaflag,nfreq)
    nvar=3*ubound(molecule,1)
    ctrl%nvar=nvar
  end if
  !----------------------------------------------------------------
  ! one distortion value for all cartesian coordinates
  ! note: generalized distortions may be necessary in order to 
  !       perform distortions along the normal coordinates
  !----------------------------------------------------------------
  allocate(distortion(nvar))
  distortion(:)=ctrl%cstep  

    ctrl%numderiv = ctrl_restart%numderiv
  if(ctrl%nummodes.eq.0) then
     write(*,*)
     write(*,*) ' ERROR: No normal modes present in snf_control,'
     write(*,*) ' thus evaluation of data with snfint not possible.'
     write(*,*) ' Maybe you should use snf instead.'
     stop
  end if

!------------------------------------------------------------------
! calculation of IR, Raman and VROA intensities
! Note: This is a very basic version - no print-out implemented
! so far, molecular symmetry not used.
!------------------------------------------------------------------

  !----------------------------------------------------------------
  ! calculate derivatives of dipole moments 
  ! along normal modes and calculate ir intensities
  !----------------------------------------------------------------
    natoms=ubound(molecule,1)
    allocate(dipol_dq(3,ctrl%nummodes))
! mass-weigthing of normal modes: ctrl%nmodein --> normalmodes
    allocate(normmodes(3*natoms,3*natoms))
    normmodes=ctrl%nmodein
    call nmod_c2m(normmodes,natoms,molecule,ctrl%nummodes)
    call create_dipol_dq_intonly(dipol_dq,dipols,ctrl%nummodes,normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
    allocate(spectrum(ctrl%nummodes))
    call calc_ir_intensities_intonly(dipol_dq,spectrum)
    if(associated(dipol_dq)) deallocate(dipol_dq)
    spectrum(:)%freq = ctrl%freqin(:)
    if(associated(ctrl%freqin))  deallocate(ctrl%freqin)
    if(associated(ctrl%selectmode))  deallocate(ctrl%selectmode)
!-------------------------------------------------------------
! everything is done in C1 symmetry --> set all irreps to 'a'
!-------------------------------------------------------------
  do i = 1,ctrl%nummodes
     spectrum(i)%irrep = 'a'
  end do

  !----------------------------------------------------------------
  ! calculate derivatives of polarizabilities
  ! along normal modes and calculate raman intensities
  !----------------------------------------------------------------
   if(ctrl%vroaew.ge.1) ctrl%cstep=ctrl%cstep*(1/bohr2ang)
  if ((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.(.not.associated(ctrl%dynfreq))) then
     allocate(polar_dq(6,ctrl%nummodes,1))
     call create_polar_dq_intonly(polar_dq(:,:,1),pols(:,:,:,1),ctrl%nummodes,normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
  else if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.associated(ctrl%dynfreq)) then
     allocate(polar_dq(6,ctrl%nummodes,ubound(ctrl%dynfreq,1)))
!   restore normal coordinates
     normmodes=ctrl%nmodein
     do i=1,ubound(ctrl%dynfreq,1)
       call create_polar_dq_intonly(polar_dq(:,:,i),pols(:,:,:,i),ctrl%nummodes,normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
     end do
  end if
  if ((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.(.not.associated(ctrl%dynfreq))) then
    do i=1,ctrl%nummodes
      allocate(spectrum(i)%raman(nfreq))
    end do
    call calc_raman_intensities(polar_dq(:,:,1),spectrum,ctrl%temp,&
     &ctrl%theta,1) 
  else if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.associated(ctrl%dynfreq)) then
    do i=1,ctrl%nummodes
      allocate(spectrum(i)%raman(nfreq))
    end do
    do i=1,ubound(ctrl%dynfreq,1)
      call calc_raman_intensities(polar_dq(:,:,i),spectrum,ctrl%temp,&
       &ctrl%theta,i,ctrl%dynfreq(i)) 
    end do
  else 
    !----------------------------------------------------------------
    ! This is done for compatibility with other parts of the program
    !----------------------------------------------------------------
    do i=1,ctrl%nummodes
      allocate(spectrum(i)%raman(1))
    end do
  endif
  !----------------------------------------------------------------
  ! calculate derivatives of generalized polarizabilities
  ! along normal modes and calculate vroa intensities
  !----------------------------------------------------------------
!#####################################################################################
! allocate arrays for VROA calculations and calculate derivatives
!
  if (ctrl%vroaew.ge.1.and.(.not.associated(ctrl%dynfreq))) then
! G/LAO tensor:
     allocate(gtenlao_dq(9,ctrl%nummodes,1))
     call create_aten_dq_intonly(gtenlao_dq(:,:,1),gtenlao(:,:,:,1),ctrl%nummodes, &
  & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
! G/AO tensor:
     allocate(gtenao_dq(9,ctrl%nummodes,1))
     call create_aten_dq_intonly(gtenao_dq(:,:,1),gtenao(:,:,:,1),ctrl%nummodes, &
 & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
! A tensor:
     allocate(aten_dq(27,ctrl%nummodes,1))
     call create_aten_dq_intonly(aten_dq(:,:,1),aten(:,:,:,1),ctrl%nummodes, &
  & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
  else if(ctrl%vroaew.ge.1.and.associated(ctrl%dynfreq)) then
! G/LAO tensor:
     allocate(gtenlao_dq(9,ctrl%nummodes,ubound(ctrl%dynfreq,1)))
!   restore normal coordinates
     normmodes=ctrl%nmodein
     do i=1,ubound(ctrl%dynfreq,1)
       call create_aten_dq_intonly(gtenlao_dq(:,:,i),gtenlao(:,:,:,i),ctrl%nummodes, &
 & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
     write(*,*) 'gtenlao_dq ',gtenlao_dq
     end do
! G/AO tensor:
     allocate(gtenao_dq(9,ctrl%nummodes,ubound(ctrl%dynfreq,1)))
     normmodes=ctrl%nmodein
     do i=1,ubound(ctrl%dynfreq,1)
       call create_aten_dq_intonly(gtenao_dq(:,:,i),gtenao(:,:,:,i),ctrl%nummodes, &
 & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
     write(*,*) 'gtenao_dq ',gtenao_dq
     end do
! A tensor:
     allocate(aten_dq(27,ctrl%nummodes,ubound(ctrl%dynfreq,1)))
     normmodes=ctrl%nmodein
     do i=1,ubound(ctrl%dynfreq,1)
       call create_aten_dq_intonly(aten_dq(:,:,i),aten(:,:,:,i),ctrl%nummodes, &
 & normmodes,ctrl%cstep,ctrl%numderiv,natoms,molecule)
     end do
  end if

! allocation of arrays and calculation of derivatives 
! for VROA calculations finished
!#####################################################################################
        if (ctrl%vroaew.ge.1.and.(.not.associated(ctrl%dynfreq))) then
          allocate(polar9_dq(9,ctrl%nummodes,1))
!... copy 6 non-redundant entries of polarizability derivatives to 9 redundant entries
!    and transfrom back to atomic units (mass AND length!)
          ddum=sqrt(XFAMU)*bohr2ang*bohr2ang
          polar9_dq(1,:,1)=polar_dq(1,:,1)/ddum
          polar9_dq(2,:,1)=polar_dq(2,:,1)/ddum
          polar9_dq(3,:,1)=polar_dq(3,:,1)/ddum
          polar9_dq(4,:,1)=polar_dq(2,:,1)/ddum
          polar9_dq(5,:,1)=polar_dq(4,:,1)/ddum
          polar9_dq(6,:,1)=polar_dq(5,:,1)/ddum
          polar9_dq(7,:,1)=polar_dq(3,:,1)/ddum
          polar9_dq(8,:,1)=polar_dq(5,:,1)/ddum
          polar9_dq(9,:,1)=polar_dq(6,:,1)/ddum
!MR type spectrum does not yet contain vroa as a possible mode
!MR          do i=1,ctrl%nummodes
!MR            allocate(spectrum(i)%vroa(nfreq))
!MR          end do
!... transfrom mass dimension to atomic units
          ddum=sqrt(XFAMU)
          gtenlao_dq(:,:,1)=gtenlao_dq(:,:,1)/ddum
          gtenao_dq(:,:,1)=gtenao_dq(:,:,1)/ddum
          aten_dq(:,:,1)=aten_dq(:,:,1)/ddum
          write(out,'(a)') ' '
          write(out,'(a)') ' Entering routines based on the DALTON program system (ABACUS) for VROA calcs:'
          write(out,'(a)') ' ... original version of the routines by K. Ruud et al. !'
          write(out,'(a)') ' ... see: http://www.kjemi.uio.no/software/dalton/dalton.html'
          write(out,'(a)') ' '
          allocate(freqs(ubound(spectrum,1)))
          do i=1,ubound(spectrum,1)
            freqs(i)=spectrum(i)%freq
          enddo
          dumlaserf(1)=0.0d0
          call calcvroa(freqs,polar9_dq,gtenao_dq,gtenlao_dq,aten_dq,&
                       &nvar,ctrl%nummodes,1,1,dumlaserf,out)
          deallocate(freqs)
        else if(ctrl%vroaew.ge.1.and.associated(ctrl%dynfreq)) then
          allocate(polar9_dq(9,nvar,nfreq))
!... copy 6 non-redundant entries of polarizability derivatives to 9 redundant entries
!    and transfrom back to atomic units (mass AND length!)
          ddum=sqrt(XFAMU)*bohr2ang*bohr2ang
          polar9_dq(1,:,:)=polar_dq(1,:,:)/ddum
          polar9_dq(2,:,:)=polar_dq(2,:,:)/ddum
          polar9_dq(3,:,:)=polar_dq(3,:,:)/ddum
          polar9_dq(4,:,:)=polar_dq(2,:,:)/ddum
          polar9_dq(5,:,:)=polar_dq(4,:,:)/ddum
          polar9_dq(6,:,:)=polar_dq(5,:,:)/ddum
          polar9_dq(7,:,:)=polar_dq(3,:,:)/ddum
          polar9_dq(8,:,:)=polar_dq(5,:,:)/ddum
          polar9_dq(9,:,:)=polar_dq(6,:,:)/ddum
!MR type spectrum does not yet contain vroa as a possible mode
!MR          do i=1,ctrl%nummodes
!MR            allocate(spectrum(i)%vroa(nfreq))
!MR          end do
          do i=1,ubound(ctrl%dynfreq,1)
!... transfrom mass dimension to atomic units
            ddum=sqrt(XFAMU)
            gtenlao_dq(:,:,i)=gtenlao_dq(:,:,i)/ddum
            gtenao_dq(:,:,i)=gtenao_dq(:,:,i)/ddum
            aten_dq(:,:,i)=aten_dq(:,:,i)/ddum
          end do
          write(out,'(a)') ' '
          write(out,'(a)') ' Entering routines based on the DALTON program system (ABACUS) for VROA calcs:'
          write(out,'(a)') ' ... original version of the routines by K. Ruud et al. !'
          write(out,'(a)') ' ... see: http://www.kjemi.uio.no/software/dalton/dalton.html'
          write(out,'(a)') ' '
          allocate(freqs(ubound(spectrum,1)))
	  do i=1,ubound(spectrum,1)
            freqs(i)=spectrum(i)%freq
          enddo
          call calcvroa(freqs,polar9_dq,gtenao_dq,gtenlao_dq,aten_dq,&
                       &nvar,ctrl%nummodes,nfreq,&
                       &nfreq,ctrl%dynfreq,out)
          deallocate(freqs)
!MR        else
!MR          do i=1,ctrl%nummodes
!MR            allocate(spectrum(i)%vroa(1))
!MR          end do
        endif
  ctrl%nmodein=normmodes
  deallocate(normmodes)
        !----------------------------------------------------------------
        ! assign several values concerning the spectrum
        !----------------------------------------------------------------
        do i=1,ctrl%nummodes
           allocate(spectrum(i)%norm_mode(nvar))
           spectrum(i)%norm_mode(:)=ctrl%nmodein(i,:)
           spectrum(i)%red_mass = 0.d0
           do j=1,nvar
              spectrum(i)%red_mass = spectrum(i)%red_mass + &
                   & (spectrum(i)%norm_mode(j)**2 &
                   & /(molecule((j+2)/3)%mass))
           enddo
           spectrum(i)%red_mass=1.d0/spectrum(i)%red_mass
           spectrum(i)%force = 0.d0
        enddo
        if (associated(polar_dq)) deallocate(polar_dq)
! set free all memory used for VROA tensors:
        if (associated(polar9_dq)) deallocate(polar9_dq)
        if (associated(gtenlao_dq)) deallocate(gtenlao_dq)
        if (associated(gtenao_dq)) deallocate(gtenao_dq)
        if (associated(aten_dq)) deallocate(aten_dq)

  ! In case of unexpected incorrect translational or rotational modes
  ! dummy modes are introduced
  !----------------------------------------------------------------
  ! clean up, and print out.
  !----------------------------------------------------------------
  call sort_spectrum(spectrum)
  call get_relir(spectrum)
  call print_ir_nosym(spectrum,unit=out)
  static=.not.associated(ctrl%dynfreq)
  if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.(.not.associated(ctrl%dynfreq))) then
    call get_relcross(spectrum,1)
    call print_raman(spectrum,1,out)
    if(ctrl%print_rcs) call print_rcs(spectrum,1,static,&
    &ctrl%theta,out)
  else if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.associated(ctrl%dynfreq)) then
    !----------------------------------------------------------------
    ! if dynamic polarizabilities with non-standard laser wavelengths
    ! have been selected, print raman intensities for these
    ! wavelengths.
    !----------------------------------------------------------------
    do i=1,nfreq
      if(abs(ctrl%dynfreq(i)).le.1.0d-6) then
        write(out,'(3(/),a)') ' Following Raman results obtained for &
         &laser wavelength:  infinity'
        write(out,'(80a)') ('=',j=1,80)
        static=.true.
      else
        write(out,'(3(/),a,f16.5,a)') ' Following Raman results obtained &
         &for laser wavelength: ',1.0d0/ctrl%dynfreq(i)*nm2freq,' nm'
         write(out,'(80a)') ('=',j=1,80)
        static=.false.
      end if
      write(out,'(a)') 
      call get_relcross(spectrum,i)
      call print_raman(spectrum,i,out)
      if(ctrl%print_rcs) call print_rcs(spectrum,i,static,&
       &ctrl%theta,out)
    end do
  end if
  !-------------------------------------------------------------
  ! additional output: intensities summed over degenerate modes
  !-------------------------------------------------------------
!  if(ctrl%sumdeg) then
!    write(out,'((/),152(''#''))') 
!    write(out,'(3(/),a)') ' Intensities summed over degenerate modes'
!    write(out,'(a,2(/))') ' ----------------------------------------'
!    call sum_degen(spectrum,sum_spectrum,rflag)
!    call get_relir(sum_spectrum)
!    call print_ir(sum_spectrum,unit=out)
!    if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.(.not.associated(ctrl%dynfreq))) then
!      call get_relcross(sum_spectrum,1)
!      call print_raman(sum_spectrum,1,out)
!      if(ctrl%print_rcs) call print_rcs(sum_spectrum,1,static,&
!       &ctrl%theta,out)
!CMR291004    else if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.associated(ctrl%dynfreq)) then
!CMR291004 It was necessary to deactivate this for VROA because parts of sum_spectrum
!CMR291004   have not been allocated up to this point (namely: sum_spectrum%raman(*),
!CMR291004   which will cause a segmentation fault in call get_relcross(sum_spectrum,i)
!CMR291004
!    else if(ctrl%vroaew.ge.1.and.associated(ctrl%dynfreq)) then
!          call serror('snf',1,'No intensity summation over degenerate modes&
!     & ##available for VROA at the moment!.',out) 
!!CMR291004
!    else if(ctrl%ramanew.ge.1.and.associated(ctrl%dynfreq)) then
!      do i=1,nfreq
!        if(abs(ctrl%dynfreq(i)).le. 1.0d-10) then
!          write(out,'(3(/),a)') ' Following Raman results obtained for &
!           &laser wavelength:  infinity'
!        else
!          write(out,'(3(/),a,f16.5)') ' Following Raman results obtained &
!           &for laser wavelength: ',1.0d0/ctrl%dynfreq(i)*nm2freq
!        end if
!        write(out,'(a)') 
!        call get_relcross(sum_spectrum,i)
!        call print_raman(sum_spectrum,i,out)
!        if(ctrl%print_rcs) call print_rcs(sum_spectrum,i,static,&
!         &ctrl%theta,out)
!      end do
!    end if
!    deallocate(sum_spectrum)
!    write(out,'(2(/),152(''#''),2(/))') 
!  end if
  !cmr
  write(out,*)
  write(out,'('' Frequencies and mass-weighted normal coordinates'')')
  write(out,'('' ------------------------------------------------'')')
  call preig(out,ubound(spectrum,1),spectrum,'eigenvalues  in  cm^-1')
  call print_norm_modes(spectrum,molecule,unit=out)
  !----------------------------------------------------------------
  ! write mopac and gaussian frequency output
  !----------------------------------------------------------------
  call calc_mode_properties(molecule,spectrum)
  if(ctrl%print_mopac.or.ctrl%print_gaussian.or.ctrl%print_xmol) then
     write(out,*)
     write(out,'('' Generate fake outputs for normal mode presentation'')')
     write(out,'('' --------------------------------------------------'')')
     if((ctrl%ramanew.ge.1.or.ctrl%vroaew.ge.1).and.associated(ctrl%dynfreq)) then
       if(ubound(ctrl%dynfreq,1).ge.2) call serror('snf',-1,'Files for&
         & normal mode presentation contain##Raman data only for the first&
         & selected frequency.',out)
     end if
     if(ctrl%print_mopac) &
          &  call print_mopac_frequencies(molecule,spectrum,&
          &            ctrl%print_mopxmol,'mopac.out',unit=out)
     if(ctrl%print_gaussian) &
          &  call print_gaussian_frequencies(molecule,spectrum,&
          &                            'g98.out',rflag,unit=out)       
     if(ctrl%print_xmol) &
      & call print_xmol_norm(molecule,spectrum,'xmol.XYZ.out',&
      &ctrl%xmolsc1,unit=out)
  endif
  !----------------------------------------------------------------
  ! generate data-, gnuplot-, and eps-files for spectrum plotting
  !----------------------------------------------------------------
  call generate_plots(ctrl,spectrum,1,unit=out)
  !----------------------------------------------------------------
  ! clean up memory 
  !----------------------------------------------------------------
  call destroy_spectrum(spectrum)
  deallocate(spectrum)
  call destroy_molecule(molecule)
  call destroy_control(ctrl)
  deallocate(distortion)
  if(associated(ctrl%nmodein))  deallocate(ctrl%nmodein)
!=============
! time control
!=============
  call secs(t2)
  call clean(tbuf)
  call ptime(t2-t1,tbuf)
  call llen(tbuf,ltbuf)
  write(out,*)  
  write(out,'(500a)')  ('*',j=1,ltbuf+3+21)
  write(out,'(3a)')'** time used in snf: ',tbuf(1:ltbuf),' **'
  write(out,'(500a)')  ('*',j=1,ltbuf+3+21)
!=============
  close(out)
end program snfint

subroutine print_header(out)
  implicit none
  include '../snfdc/VERSION.h'
  integer,intent(in) :: out
  write(out,'(A)')
!                   +1234567890123456789012345678+8765432109876543210987654321+'
  write(out,'(A)')' +---------------------------------------------------------+'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                          S N F                          |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A,a21,a)') &
   &              ' |                  ',version,          '                  |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |   A program package for numerical frequency analyses    |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                       written by                        |'
  write(out,'(A)')' |            C. Kind, M. Reiher, J. Neugebauer            |'
  write(out,'(A)')' |                      and B. A. Hess                     |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |      Theoretical Chemistry, University of Erlangen      |'
  write(out,'(A)')' |        Egerlandstr. 3, D-91058 Erlangen, Germany        |'
  write(out,'(A)')' |                        1999-2003                        |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                           and                           |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |        Theoretical Chemistry, University of Bonn        |'
  write(out,'(A)')' |          Wegelerstr. 12, D-53115 Bonn, Germany          |'
  write(out,'(A)')' |                           2004                          |'
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' |                           and                           |'
  write(out,'(A)')' |        Theoretical Chemistry, University of Jena        |'
  write(out,'(A)')' |         Helmholtzweg 4, D-07743 Jena, Germany           |'    
  write(out,'(A)')' |                                                         |'
  write(out,'(A)')' +---------------------------------------------------------+'
!                   +1234567890123456789012345678+8765432109876543210987654321+'
  write(out,'(A)')
end subroutine print_header

subroutine last_mod(out)
  implicit none
  include '../snfdc/VERSION.h'
  integer,intent(in) :: out
  write(out,'(A)')
  write(out,'(2A)') ' Last modification: ',lastmod
  write(out,'(A)')
end subroutine last_mod
