 
!********************************************************************************
!*   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
!*   Chair of Theoretical Chemistry, Friedrich-Alexander-University
!*   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
!*
!*
!*   This file is part of the package SNF.
!*
!*   SNF is free software; you can redistribute it and/or modify
!*   it under the terms of the GNU General Public License as published by
!*   the Free Software Foundation; either version 2 of the License, or
!*   (at your option) any later version.
!*
!*   This program is distributed in the hope that it will be useful,
!*   but WITHOUT ANY WARRANTY; without even the implied warranty of
!*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!*   GNU General Public License for more details.
!*
!*   You should have received a copy of the GNU General Public License
!*   along with this program; if not, write to the Free Software
!*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
!*
!********************************************************************************
module snf_rdalmol
  use f90_kind
  use snf_error
  use snf_tools
  use snf_param

  private 
  public :: rwdalmol,read_dal_coord,checkdalgeo,read_dal_ctrl,test_dal_sym
 
  !*********************************************************************
    
contains

  !*********************************************************************
 
      subroutine rwdalmol(natoms,ic,co,ir1,mxat,kcharg,rewri,lo)
!
! read/write basic molecular input information from molecule.mol file
!
! written by J. Neugebauer, Uni Erlangen, 2002.
! bug lr-1 fixed by M. Reiher, Uni Erlangen, June 2003
! NB by JN: this might be a f90 compiler problem (ifc seems to need lr-1)!
!
      implicit none
      integer mxat,mxsh
      parameter(mxsh=50)
      real(double) :: co(3,mxat)
      integer :: ic(mxat)
      integer :: ir1,lo,lr,lend,iat
      logical :: atomba,basis,intgrl
      character(len=132) :: buf,bbuf
      character(len=2) :: symtxt
      character(len=4) :: rewri
 
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      integer lam(mxat),nco(mxsh,mxat)
      character(len=1) :: charm
      character(len=72) ::  coform

!*****
! Card 1: INTGRL, BASIS, or ATOMBASIS
!*****
      read (ir1,'(a)',ERR=900,END=900) buf
      lend=2
      IF(buf(1:5).EQ.'BASIS') THEN
       basis=.true.
       lend=3
      ELSE IF (buf(1:6).eq.'ATOMBA') THEN
       atomba=.true.
      ELSE IF (buf(1:6).eq.'INTGRL') THEN
       intgrl=.true.
      ELSE
       call serror('rdalmol',2,'Error! Found wrong keyword: '//buf(1:6))
      END IF
      if(rewri.eq.'writ') then
        call llen(buf,lr)
!MR        write(lo,'(a)') buf(1:lr-1)
        write(lo,'(a)') buf(1:lr)
!        call flush(lo)
      end if
!*****
! Card 2,3: (basis,) title
!*****
      do i=1,lend
        read (ir1,'(a)',ERR=900,END=900) buf
        if(rewri.eq.'writ') then
          call llen(buf,lr)
!MR          write(lo,'(a)') buf(1:lr-1)
          write(lo,'(a)') buf(1:lr)
!          call flush(lo)
        end if
      end do
!*****
! Card 4:
!*****
      READ (ir1,'(a)',ERR=900,END=900) buf
      call clean(bbuf)
      READ (buf,'(BN,A1,I4,I3,A2,A)') &
     &    charm,nat,kcharg,symtxt,bbuf
      if(symtxt.ne.' 0'.and.symtxt.ne.'0 ') then
        if(symtxt.eq.'  ') then
          call serror('rdalmol',0,'Displaced geometries '// &
     &     'must be calculated in C1 !##Setting symmetry to C1 !')
          symtxt=' 0'
          call llen(bbuf,lr)
!MR          write(buf,8800) charm,nat,kcharg,symtxt,bbuf(1:lr-1)
          write(buf,8800) charm,nat,kcharg,symtxt,bbuf(1:lr)
8800      FORMAT(A1,I4,I3,A2,A)
        end if
      end if
      if(rewri.eq.'writ') then
        call llen(buf,lr)
!MR        write(lo,'(a)') buf(1:lr-1)
        write(lo,'(a)') buf(1:lr)
!        call flush(lo)                              
      end if

!*****
! geometry records
!*****
      natoms=0
      nsaos=0
      nsh=0
      DO iat=1,nat
        CALL rwdalgeo(lo,ir1,1+natoms,lam(iat),nco(1,iat),symtxt,nont, &
     &    mxsh,co,atomba,basis,mxat,ic,rewri)
        natoms=natoms+nont
        if(intgrl) then
          nnsaos=0
          nnsh=0
          CALL rwgtodal(lo,ir1,lam(iat),nco(1,iat),nnsaos,nnsh,mxsh,rewri)
          nsh=nsh+nnsh*nont
          nsaos=nsaos+nnsaos*nont
        end if
      END DO
 
 
900   return
      end subroutine rwdalmol

!************************************************************************

      SUBROUTINE rwdalgeo(lo,ir1,iat,lam,nco,stxt,nont,mxsh,co, &
     & atomba,basis,mxat,ic,rewri)
!
! read/write geometry record for one block
!
! written by J. Neugebauer 10/2001
! bug lr-1 fixed by M. Reiher, Uni Erlangen, June 2003
! NB by JN: this might be a f90 compiler problem (ifc seems to need lr-1)!
!
      use snf_molecule, only : find_nucl_charge,find_element
      implicit none
      INTEGER ir1,iat,lam,mxsh,nco(mxsh),nat,lo,mxat
      integer ic(mxat)
      integer idum,ist,ien
      real(double) co(3,mxat),zs
! local
      CHARACTER*1 zset
      CHARACTER*4 atn,rewri
      CHARACTER*2 stxt
      CHARACTER*40 atbstring
      CHARACTER*132 buf
      INTEGER irc,nont,i,j,k,nu
      LOGICAL zmattst,atomba,basis
 
      zmattst=.false.
      IF(stxt.ne.' 0'.and.stxt.ne.'  '.and.stxt.ne.'0 ') THEN
        call serror('rwdalgeo',2,'Symmetry adapted input not yet possible')
      END IF
 
! read data for all blocks
 
!*****
! Card 6: ATOMIC DATA:
!         zs(i) - nuclear charge
!         nont  - number of symmetry independent atoms
!         lam   - maximum angular quantum number (s=1,p=2,d=3 etc.)
!         nco   - number of AO-blocks for each l-value
!*****
      if(atomba) then
        read (ir1,12,ERR=999,END=999) zset,zs,nont,atbstring
        if(rewri.eq.'writ') then
          write (lo,12) zset,zs,nont,atbstring
!          call flush(lo)
        end if
      else if(basis) then
        read (ir1,13,ERR=999,END=999) zset,zs,nont
        if(rewri.eq.'writ') then
          write (lo,13) zset,zs,nont
!          call flush(lo)
        end if
      else
        read (ir1,11,ERR=999,END=999) zset,zs,nont,lam,(nco(j),j=1,lam)
        if(rewri.eq.'writ') then
          write (lo,11) zset,zs,nont,lam,(nco(j),j=1,lam)
!          call flush(lo)
        end if
      end if
11    FORMAT (A1,F9.0,12I5)
12    FORMAT (A1,F9.0,I5,A)
13    FORMAT (A1,F9.0,I5)
      IF(zset.ne.' ') THEN
        zmattst=.true.
        call serror('rwdalgeo',2,'Z-matrix input not yet implemented')
      END IF
 
!*****
! Card 7: atomic name and coordinates
!           atn - name of symmetry independent center
!           coo - coordinates of symmetry independent center
!*****
 
! this atom and the symmetry-equivalent atoms
      DO k=iat,iat+nont-1
        READ (ir1,'(A)',ERR=999,END=999) buf
        READ (buf,'(a4)',ERR=999,END=999) atn
!new: 2 lines
        atn(3:4)='  '
        atn(1:2)=find_element(nint(zs))
        if(rewri.eq.'read') then
          READ (buf(5:),*,ERR=999,END=999) (co(j,k),j=1,3)
!          ic(k)=find_nucl_charge(atn(1:2))
!new: 1 line
           ic(k)=zs
        end if
        if(rewri.eq.'writ') then
          write (lo,'(a4,3f20.12)') atn,(co(j,k),j=1,3)               
!          call flush(lo)
        end if
      END DO
      RETURN
999   call serror('rwdalgeo',2,'Error while reading molecular structure input')
      END subroutine rwdalgeo

!************************************************************************
 
      SUBROUTINE rwgtodal(lo,ir1,lam,nco,nsaos,nsh,mxsh,rewri)
! bug lr-1 fixed by M. Reiher, Uni Erlangen, June 2003
! NB by JN: this might be a f90 compiler problem (ifc seems to need lr-1)!
! modified by M. Reiher, Uni Bonn, Oct. 2004
!  (with respect to dalton's free input format)
 
      implicit none
      INTEGER ir1,psy,nuc,nrc,lam,nsaos,lr
      integer mxsh,lo
      CHARACTER*72 coform
      CHARACTER*132 buf
      CHARACTER*1 frmt
      character*4 rewri
      INTEGER i,j,k,ncol,l,m,nco(mxsh),nsh,pb
      LOGICAL free
 
      nsaos=0
      nsh=0
      DO 200 i = 1,lam
       DO 250 j = 1,nco(i)
!*****
! Card 8:
!
! nuc - number of uncontracted shells in AO-block
! nrc - number of contracted shells in AO-block
!*****
        READ (ir1,'(a)',ERR=900,END=900) buf
        if(rewri.eq.'writ') then
          call llen(buf,lr)
!MR          write (lo,'(a)') buf(1:lr-1)
          write (lo,'(a)') buf(1:lr)
!          call flush(lo)
        end if
        READ (buf,'(BN,A1,I4,I5)') frmt,nuc,nrc
! i=l - 1
        nsaos=nsaos+nrc*(2*i-1)
        nsh=nsh+nrc
        IF (frmt .EQ. 'F' .OR. frmt .EQ. 'f') THEN
           free = .TRUE.
        ELSE IF (frmt .EQ. 'H' .OR. frmt .EQ. 'h') THEN
           free = .FALSE.
           ncol = 4
        ELSE
           free = .FALSE.
           ncol = 8
        END IF
        IF(nrc.NE.0) THEN
          DO 100 l=1,nuc
!*****
! Card 9:
!*****
            READ(ir1,'(a)',ERR=900,END=900) buf
            if(rewri.eq.'writ') then
              call llen(buf,lr)
!MR              write (lo,'(a)') buf(1:lr-1)
              write (lo,'(a)') buf(1:lr)
!              call flush(lo)
            end if
            IF(free) THEN
!CMR291004
!CMR291004              call serror('rwgtodal',2,
!CMR291004     *        'free input format not supported',6)
!CMR              call serror('rwgtodal',1,
!CMR     *        'free input format not supported',6)
              call serror('rwgtodal',1,'Using free input format may yield WRONG results !!!',6)
!CMR291004
            ELSE
              DO 300 k=2,(nrc-1)/(ncol-1)+1
                READ(ir1,'(a)',ERR=900,END=900) buf
                if(rewri.eq.'writ') then
                  call llen(buf,lr)
!MR                  write (lo,'(a)') buf(1:lr-1)
                  write (lo,'(a)') buf(1:lr)
!                  call flush(lo)
                end if
300           CONTINUE
            END IF
100       CONTINUE
        ELSE
          call serror('rwgtodal',2,'No coefficients available')
        END IF                                      
250    CONTINUE
200   CONTINUE
 
900   RETURN
      END subroutine rwgtodal

!************************************************************************

      subroutine read_dal_coord(ctrl,molecule)
!
! read molecule information from the DALTON input file molecule.mol
!
! written by J. Neugebauer, University of Erlangen, 2002
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_atom,find_element,find_mass,fiso_mass
      implicit none
      type(type_ctrl) :: ctrl
      type(type_atom),pointer :: molecule(:)
      integer li,lo,mxat,natoms,mcharge,nel,ncs,nos,i,calcno,nfreq
      parameter(mxat=1000)
      integer,pointer :: ic(:)
      real(double),pointer :: co(:,:)
      character(len=80) form,form2
      character(len=10) :: progtype,calctype
      allocate(ic(mxat),co(3,mxat))
      li=1
      lo=2
      !--------------------------------------------------------
      ! read geometry
      !--------------------------------------------------------
      call bfile('molecule.mol')
      open(li, file='molecule.mol.bak')
      call rwdalmol(natoms,ic,co,li,mxat,mcharge,'read',lo)
      close(li)
      allocate(molecule(natoms))
      molecule(:)%changem=.false.
      do i=1,natoms
        molecule(i)%cartcoord(:)=co(:,i)
        molecule(i)%nucl_charge=ic(i)
        molecule(i)%element=find_element(ic(i))
        if(molecule(i)%element.eq.'D') then
          molecule(i)%mass=fiso_mass(1,2)
        else
          molecule(i)%mass=find_mass(molecule(i)%nucl_charge)
        end if
      end do
      deallocate(ic,co)
      
      end subroutine read_dal_coord
 
!************************************************************************
 
      subroutine wcoord(tempname,natoms,co,ic)
!
!     write TURBOMOLE coord file
!
!     written by J. Neugebauer, Uni Erlangen, 01/2002
!     declarations reordered by M. Reiher, Uni Bonn, 10/2004
 
      use snf_molecule,only: find_element
      implicit none
      integer natoms,ic(natoms),i,lu
      real(double) co(3,natoms)
      character*(*) tempname
      lu=44
      open (unit=lu,file=tempname,status='unknown',err=1001)
      write(44,'(''$coord'')')
      do i=1,natoms
        write(lu,300) co(1,i), co(2,i), co(3,i), find_element(ic(i))
      enddo
      write(44,'(''$end'')')
      close(lu)
300   format(3f18.10,5x,a2)
1001  return
      end subroutine wcoord
 
!************************************************************************

      subroutine checkdalgeo(molecule,ctrl)
!
! written by J. Neugebauer, Uni Erlangen, 2002
!
      use snf_control, only : type_ctrl
      use snf_molecule, only : type_molecule,type_atom,atom_equal
      use snf_irred_atoms, only : get_rpoint_grp,get_point_grp
      implicit none
      type(type_atom),pointer :: molecule(:)
      type(type_ctrl) :: ctrl
      type(type_molecule) :: molekel
      integer,pointer :: ic(:)
      integer :: natoms,i,j,li,nnat,mcharge,lo
      real(double), pointer :: co(:,:)
      character(len=3) pointgrp,oripg
 
      li=1
      lo=2
      call get_point_grp(molecule,1.D-3,atom_equal,oripg)
      call get_rpoint_grp(molecule,1.D-3,atom_equal,pointgrp)
      write(*,*)
      write(*,*)'Automatic point-group check yields: ',pointgrp
      if(pointgrp.ne.oripg) then
        write(*,*)
        if(associated(ctrl%tm_ctrl)) call bfile(ctrl%tm_ctrl%coord)
        call serror('snf_define',0,'Note that a coordinate transformation##was '//&
     & 'necessary to verify this point group.')
        ctrl%symmetry=pointgrp
        natoms=ubound(molecule,1)
        allocate(ic(natoms),co(3,natoms))
        do i=1,natoms
          ic(i)=molecule(i)%nucl_charge
          do j=1,3
            co(j,i)=molecule(i)%cartcoord(j)
          end do
        end do
!        call wcoord('coord',natoms,co,ic)
        nnat=natoms
        open(li, file='molecule.mol.bak')
        open(lo, file='molecule.mol')
        call rwdalmol(nnat,ic,co,li,natoms,mcharge,'writ',lo)
        close(lo)
        close(li)
        deallocate(ic,co)
      else
        ctrl%symmetry=oripg
      end if
      end subroutine checkdalgeo
 
!************************************************************************
 
      subroutine read_dal_ctrl(ctrl)
! 
! read type of calculation for DALTON calculation
!
! written by J. Neugebauer, Uni Erlangen, 2003
!
      use snf_control, only : type_ctrl
      implicit none
      type(type_ctrl) :: ctrl
      integer li,i,cno
      character(len=255) row
      li=1
      
      cno=0
      open(li, file=ctrl%dal_ctrl%control)
      do 
        read(li,'(a)',err=900,end=900) row
        if(index(row,'.DIRECT').ne.0) then
          ctrl%dal_ctrl%direct=.true.
        else if(index(row,'**WAVE').ne.0) then
          exit
        end if
      end do
     
      do 
        read(li,'(a)',err=900,end=900) row
        if(index(row,'.HF').ne.0.and.cno.eq.0) then
          cno=1
        else if(index(row,'.MP2').ne.0.and.cno.eq.0) then
          cno=2
        else if(index(row,'.CC2').ne.0) then
          cno=3
        else if(index(row,'.CCSD').ne.0) then
          cno=5
        else if(index(row,'.CCS').ne.0) then
          cno=4
        else if(index(row,'.MCSCF').ne.0) then
          cno=6
        end if
      end do
      
      if(cno.eq.1) then
        rewind(li)
        do 
          read(li,'(a)',err=900,end=900) row
          if(index(row,'.RUN RESPONSE').ne.0) cno=7
        end do
      end if

      if(cno.eq.2) then
        rewind(li)
        do 
          read(li,'(a)',err=900,end=900) row
          if(index(row,'.SOPPA').ne.0) cno=8
        end do
      end if

900   close(li)
      if(cno.ne.0) then
        ctrl%calcno=cno
        ctrl%dal_ctrl%calcno=cno
      end if
      end subroutine read_dal_ctrl

!************************************************************************

      subroutine test_dal_sym(nocheck)
!
! reads symmetry information from the DALTON input file molecule.mol
!
! written by M. Reiher, University of Erlangen, June 2003
!
      implicit none
      integer li,idum,idum1,idum2
      logical :: nocheck
      character(len=132) :: buf
      li=1
      nocheck=.false.
      call bfile('molecule.mol')
      open(li, file='molecule.mol.bak')
      read (li,'(a)',ERR=900,END=900) buf
      read (li,'(a)',ERR=900,END=900) buf
      read (li,'(a)',ERR=900,END=900) buf
      read (li,'(a)',ERR=900,END=900) buf
      idum=0
      idum1=0
      idum2=0
      read(li,*,ERR=900,END=900) idum,idum1,idum2 
      IF(idum2.eq.0) nocheck=.true.
900   close(li)
      return

      end subroutine test_dal_sym

!************************************************************************

end module snf_rdalmol
