CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      subroutine rwdalmol(natoms,ic,co,ir1,mxat,kcharg,rewri,lo)
C
C read/write molecular input information from molecule.mol file
C
C written by J. Neugebauer, Uni Erlangen, 2002.
C
      implicit none
      integer mxat,mxsh
      parameter(mxsh=50)
      real*8 co(3,mxat)
      integer ic(mxat)
      integer ir1,lo,lr,lend,iat
      logical atomba,basis,intgrl
      character(len=132) buf,bbuf
      character(len=2) symtxt
      character(len=4) rewri
 
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      integer lam(mxat),nco(mxsh,mxat)
      character(len=1) charm
 
      basis=.false.
      atomba=.false.
      intgrl=.false.
C*****
C Card 1: INTGRL, BASIS, or ATOMBASIS
C*****
      read (ir1,'(a)',ERR=900,END=900) buf
      lend=2
      IF(buf(1:5).EQ.'BASIS') THEN
       basis=.true.
       lend=3
      ELSE IF (buf(1:6).eq.'ATOMBA') THEN
       atomba=.true.
      ELSE IF (buf(1:6).eq.'INTGRL') THEN
       intgrl=.true.
      ELSE
       call serror('rdalmol',2,
     * 'Error! Found wrong keyword: '//buf(1:6),6)
      END IF
      if(rewri.eq.'writ') then
        call llen(buf,lr)
        write(lo,'(a)') buf(1:lr)
        call flush(lo)
      end if
C*****
C Card 2,3: (basis,) title
C*****
      do i=1,lend
        read (ir1,'(a)',ERR=900,END=900) buf
        if(rewri.eq.'writ') then
          call llen(buf,lr)
          write(lo,'(a)') buf(1:lr)
          call flush(lo)
        end if
      end do
C*****
C Card 4:
C*****
      READ (ir1,'(a)',ERR=900,END=900) buf
      call clean(bbuf)
      READ (buf,'(BN,A1,I4,I3,A2,A)')
     &    charm,nat,kcharg,symtxt,bbuf
      if(symtxt.ne.' 0'.and.symtxt.ne.'0 ') then
C        if(symtxt.eq.'  ') then                          
C          call serror('rdalmol',0,'Displaced geometries '// &
C     &     'must be calculated in C1 !##Setting symmetry to C1 !',6)
C          symtxt=' 0'
          call llen(bbuf,lr)
          write(buf,8800) charm,nat,kcharg,symtxt,bbuf(1:lr)
8800      FORMAT(A1,I4,I3,A2,A)
C        end if
      end if
      if(rewri.eq.'writ') then
        call llen(buf,lr)
        write(lo,'(a)') buf(1:lr)
        call flush(lo)
      end if
 
C*****
C geometry records
C*****
      natoms=0
      nsaos=0
      nsh=0
      DO iat=1,nat
        CALL rwdalgeo(lo,ir1,1+natoms,lam(iat),nco(1,iat),symtxt,nont,
     &    mxsh,co,atomba,basis,mxat,ic,rewri)
        natoms=natoms+nont
        if(intgrl) then
          nnsaos=0
          nnsh=0
          CALL rwgtodal(lo,ir1,lam(iat),nco(1,iat),nnsaos,nnsh,mxsh,
     &      rewri)
          nsh=nsh+nnsh*nont
          nsaos=nsaos+nnsaos*nont
        end if
      END DO
 
 
900   return
      end subroutine rwdalmol
 
*=================================================================
 
      SUBROUTINE rwdalgeo(lo,ir1,iat,lam,nco,stxt,nont,mxsh,co,
     & atomba,basis,mxat,ic,rewri)
C
C read/write geometry record for one block
C
C written by J. Neugebauer 10/2001
C
      implicit none
      integer ir1,iat,lam,mxsh,nco(mxsh),lo,mxat
      integer ic(mxat),find_nucl_charge
      real*8 co(3,mxat),zs
C local
      character*1 zset
      character*4 atn,rewri
      character*2 stxt
      character*40 atbstring
      character*132 buf
      integer nont,j,k
      logical zmattst,atomba,basis
 
      zmattst=.false.
      if(stxt.ne.' 0'.and.stxt.ne.'  '.and.stxt.ne.'0 ') then
        call serror('rwdalgeo',2,
     *    'Symmetry adapted input not yet possible',6)
      end if
 
C read data for all blocks
 
C*****
C Card 6: ATOMIC DATA:
C         zs(i) - nuclear charge
C         nont  - number of symmetry independent atoms
C         lam   - maximum angular quantum number (s=1,p=2,d=3 etc.)
C         nco   - number of AO-blocks for each l-value
C*****
      call clean(zset)
      if(atomba.or.basis) then
        read (ir1,12,ERR=999,END=999) zset,zs,nont,atbstring
        if(rewri.eq.'writ') then
          write (lo,12) zset,zs,nont,atbstring
          call flush(lo)
        end if
      else
        read (ir1,11,ERR=999,END=999) zset,zs,nont,lam,(nco(j),j=1,lam)
        if(rewri.eq.'writ') then
          write (lo,11) zset,zs,nont,lam,(nco(j),j=1,lam)
          call flush(lo)
        end if
      end if
11    format (A1,F9.0,12I5)
12    format (A1,F9.0,I5,A)
      if(zset.ne.' ') then
        write(*,'(3a)') 'zset =|',zset(1:1),'|'
        zmattst=.true.
        call serror('rwdalgeo',2,'Z-matrix input not yet implemented',6)
      end if
 
C*****
C Card 7: atomic name and coordinates
C           atn - name of symmetry independent center
C           coo - coordinates of symmetry independent center
C*****                                                      
 
C this atom and the symmetry-equivalent atoms
      do k=iat,iat+nont-1
        read (ir1,'(A)',ERR=999,END=999) buf
        read (buf,'(a4)',ERR=999,END=999) atn
        if(rewri.eq.'read') then
          read (buf(5:),*,ERR=999,END=999) (co(j,k),j=1,3)
          ic(k)=find_nucl_charge(atn(1:2))
        end if
        if(rewri.eq.'writ') then
          write (lo,'(a4,3f20.12)') atn,(co(j,k),j=1,3)
          call flush(lo)
        end if
      end do
      return
999   call serror('rwdalgeo',2,
     *  'Error while reading molecular structure input',6)
      end
 
*=================================================================
 
      SUBROUTINE rwgtodal(lo,ir1,lam,nco,nsaos,nsh,mxsh,rewri)
c
c
c     modified by M. Reiher, Uni Bonn, Oct. 2004
c	(with respect to dalton's free input format
c
 
      implicit none
      INTEGER ir1,nuc,nrc,lam,nsaos,lr
      integer mxsh,lo
      CHARACTER*132 buf
      CHARACTER*1 frmt
      INTEGER i,j,k,ncol,l,nco(mxsh),nsh
      LOGICAL free
      character*4 rewri
 
      nsaos=0
      nsh=0
      DO 200 i = 1,lam
       DO 250 j = 1,nco(i)
!*****
! Card 8:
!
! nuc - number of uncontracted shells in AO-block
! nrc - number of contracted shells in AO-block
!*****
        READ (ir1,'(a)',ERR=900,END=900) buf
        if(rewri.eq.'writ') then
          call llen(buf,lr)
          write (lo,'(a)') buf(1:lr)
          call flush(lo)
        end if
        READ (buf,'(BN,A1,I4,I5)') frmt,nuc,nrc
! i=l - 1
        nsaos=nsaos+nrc*(2*i-1)
        nsh=nsh+nrc
        IF (frmt .EQ. 'F' .OR. frmt .EQ. 'f') THEN
           free = .TRUE.
        ELSE IF (frmt .EQ. 'H' .OR. frmt .EQ. 'h') THEN
           free = .FALSE.
           ncol = 4
        ELSE
           free = .FALSE.
           ncol = 8
        END IF
        IF(nrc.NE.0) THEN
          DO 100 l=1,nuc
!*****
! Card 9:
!*****
            READ(ir1,'(a)',ERR=900,END=900) buf
            if(rewri.eq.'writ') then
              call llen(buf,lr)
              write (lo,'(a)') buf(1:lr)
              call flush(lo)
            end if
            IF(free) THEN
CMR291004
CMR291004              call serror('rwgtodal',2,
CMR291004     *        'free input format not supported',6)
CMR              call serror('rwgtodal',1,
CMR     *        'free input format not supported',6)
              call serror('rwgtodal',1,
     *        'Using free input format may yield WRONG results !!!',6)
CMR291004
            ELSE
              DO 300 k=2,(nrc-1)/(ncol-1)+1
                READ(ir1,'(a)',ERR=900,END=900) buf
                if(rewri.eq.'writ') then
                  call llen(buf,lr)
                  write (lo,'(a)') buf(1:lr)
                  call flush(lo)
                end if
300           CONTINUE
            END IF
100       CONTINUE
        ELSE
          call serror('rwgtodal',2,'No coefficients available',6)
        END IF
250    CONTINUE
200   CONTINUE
 
900   RETURN
      END           

C=========================================================================

      subroutine rbdalmol(natoms,ir1,kcharg)
C
C read/write basic molecular input information from molecule.mol file
C
C written by J. Neugebauer, Uni Erlangen, 2002.
C
      implicit none
      integer ir1,lr,lend,iat,dumic,mxsh
      parameter(mxsh=50)
      real*8 dumco(3)
      logical atomba,basis,intgrl
      character(len=132) buf,bbuf
      character(len=2) symtxt
 
      integer kcharg,nat,i,natoms,nsh,nsaos,nnsh,nont,nnsaos
      integer dumlam,dumnco(mxsh)
      character(len=1) charm
 
      basis=.false.
      atomba=.false.
      intgrl=.false.
C*****
C Card 1: INTGRL, BASIS, or ATOMBASIS
C*****
      read (ir1,'(a)',ERR=900,END=900) buf
      lend=2
      IF(buf(1:5).EQ.'BASIS') THEN
       basis=.true.
       lend=3
      ELSE IF (buf(1:6).eq.'ATOMBA') THEN
       atomba=.true.
      ELSE IF (buf(1:6).eq.'INTGRL') THEN
       intgrl=.true.
      ELSE
       call serror('rbdalmol',2,
     * 'Error! Found wrong keyword: '//buf(1:6),6)
      END IF
C*****
C Card 2,3: (basis,) title
C*****
      do i=1,lend
        read (ir1,'(a)',ERR=900,END=900) buf
      end do
C*****
C Card 4:
C*****
      READ (ir1,'(a)',ERR=900,END=900) buf
      call clean(bbuf)
      READ (buf,'(BN,A1,I4,I3,A2,A)')
     &    charm,nat,kcharg,symtxt,bbuf
 
C*****
C geometry records
C*****
      natoms=0
      nsaos=0
      nsh=0
      DO iat=1,nat
        CALL rbdalgeo(ir1,1+natoms,dumlam,dumnco,symtxt,nont,
     &    atomba,basis,dumic,mxsh)
        natoms=natoms+nont
        if(intgrl) then
          nnsaos=0
          nnsh=0
          CALL rbgtodal(ir1,dumlam,dumnco,nnsaos,nnsh,mxsh)
          nsh=nsh+nnsh*nont
          nsaos=nsaos+nnsaos*nont
        end if
      END DO
 
 
900   return
      end 
 
*=================================================================
 
      SUBROUTINE rbdalgeo(ir1,iat,dumlam,dumnco,stxt,nont,
     & atomba,basis,dumic,mxsh)
C
C read/write geometry record for one block
C
C written by J. Neugebauer 10/2001
C
      implicit none
      integer ir1,iat,dumlam,mxsh,dumnco(mxsh)
      integer dumic,iosl
      real*8 dumco(3),zs
C local
      character*1 zset
      character*4 atn
      character*2 stxt
      character*40 atbstring
      character*132 buf
      integer nont,j,k
      logical zmattst,atomba,basis
 
      iosl=41
      zmattst=.false.
      if(stxt.ne.' 0'.and.stxt.ne.'  '.and.stxt.ne.'0 ') then
        call serror('rbdalgeo',2,
     *    'Symmetry adapted input not yet possible',6)
      end if
 
C read data for all blocks
 
C*****
C Card 6: ATOMIC DATA:
C         zs(i) - nuclear charge
C         nont  - number of symmetry independent atoms
C         lam   - maximum angular quantum number (s=1,p=2,d=3 etc.)
C         nco   - number of AO-blocks for each l-value
C*****
      call clean(zset)
      if(atomba.or.basis) then
        read (ir1,12,ERR=999,END=999) zset,zs,nont,atbstring
      else
        read (ir1,11,ERR=999,END=999) zset,zs,nont,dumlam,
     *   (dumnco(j),j=1,dumlam)
      end if
11    format (A1,F9.0,12I5)
12    format (A1,F9.0,I5,A)
      if(zset.ne.' ') then
        write(iosl,'(3a)') 'zset =|',zset(1:1),'|'
        zmattst=.true.
        call serror('rbdalgeo',2,'Z-matrix input not yet implemented',
     *    iosl)
      end if
 
C*****
C Card 7: atomic name and coordinates
C           atn - name of symmetry independent center
C           coo - coordinates of symmetry independent center
C*****                                                      
 
C this atom and the symmetry-equivalent atoms
      do k=iat,iat+nont-1
        read (ir1,'(A)',ERR=999,END=999) buf
        read (buf,'(a4)',ERR=999,END=999) atn
        read (buf(5:),*,ERR=999,END=999) (dumco(j),j=1,3)
      end do
      return
999   call serror('rbdalgeo',2,
     *  'Error while reading molecular structure input',iosl)
      end
 
*=================================================================
 
      SUBROUTINE rbgtodal(ir1,lam,nco,nsaos,nsh,mxsh)
c
c
c     modified by M. Reiher, Uni Bonn, Oct. 2004
c	(with respect to dalton's free input format
c
 
      implicit none
      INTEGER ir1,nuc,nrc,lam,nsaos,lr
      integer mxsh,iosl
      CHARACTER*132 buf
      CHARACTER*1 frmt
      INTEGER i,j,k,ncol,l,nco(mxsh),nsh
      LOGICAL free
 
      iosl=41
      nsaos=0
      nsh=0
      DO 200 i = 1,lam
       DO 250 j = 1,nco(i)
C*****
C Card 8:
C
C nuc - number of uncontracted shells in AO-block
C nrc - number of contracted shells in AO-block
C*****
        READ (ir1,'(a)',ERR=900,END=900) buf
        READ (buf,'(BN,A1,I4,I5)') frmt,nuc,nrc
C i=l - 1
        nsaos=nsaos+nrc*(2*i-1)
        nsh=nsh+nrc
        IF (frmt .EQ. 'F' .OR. frmt .EQ. 'f') THEN
           free = .TRUE.
        ELSE IF (frmt .EQ. 'H' .OR. frmt .EQ. 'h') THEN
           free = .FALSE.
           ncol = 4
        ELSE
           free = .FALSE.
           ncol = 8
        END IF
        IF(nrc.NE.0) THEN
          DO 100 l=1,nuc
C*****
C Card 9:
C*****
            READ(ir1,'(a)',ERR=900,END=900) buf
            IF(free) THEN
CMR291004
CMR291004              call serror('rbgtodal',2,
CMR291004     *        'free input format not supported',6)
CMR              call serror('rbgtodal',1,
CMR     *        'free input format not supported',6)
              call serror('rbgtodal',1,
     *        'Using free input format may yield WRONG results !!!',6)
CMR291004 try to read in basis in free format
            ELSE
              DO 300 k=2,(nrc-1)/(ncol-1)+1
                READ(ir1,'(a)',ERR=900,END=900) buf
300           CONTINUE
            END IF
100       CONTINUE
        ELSE
          call serror('rbgtodal',2,'No coefficients available',iosl)
        END IF
250    CONTINUE
200   CONTINUE
 
900   RETURN
      END           
