      subroutine displacecoord(normmod,co,ic,nmno,
     * ipre)
*----------------------------------------------------------------------|
* generation of coordinates for structures displaced along
* the normal coordinates
* 
* written by J. Neugebauer, Uni Erlangen, 2002
* modified by C. Herrmann, Uni Bonn, 02/2005
*----------------------------------------------------------------------|
      include 'defs.h'
      include 'param.h'
      integer nmno,lcoo,ltcoo
      integer ic(natoms),ipre,i,nummodes,j
      real*8 normmod(maxmode,3*maxatoms),co(3,natoms)
      real*8 cstepq
      character*255 tcoord
      cstepq=cstep*dble(ipre)
      write(iosl,*) 'NORMMODE in displacecoord'
      do i = 1,3
       do j = 1,natoms*3
        write(iosl,*) normmod(i,j)
       end do
      end do
      call flush(iosl)
      call modcoord(co,normmod,cstepq,nummodes,nmno)
      
      return
      end

************************************************************************

      subroutine modcoord(co,dmode,cstepq,nummodes,imode)
*----------------------------------------------------------------------|
* yields new coordinates co = co + cstep*dmode
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------|
      include 'defs.h'
      include 'param.h'
      integer i,nummodes,imode
      real*8 co(3*natoms),dmode(maxmode,3*maxatoms)
      real*8 cstepq
      
      do i=1,3*natoms
        co(i)=co(i)+cstepq*dmode(imode,i)
      end do
      return
      end 

************************************************************************

      subroutine getcstepq(nmode,cstep,cstepq,ic,natoms)
*----------------------------------------------------------------------|
* calculates the step size cstepq for the displacement along the
* normal coordinates, such that the corresponding magnitude of the
* cartesian displacement vector is equal to cstep
* (assuming that the displacement is performed along non-mass-weighted
* normal modes which are normalized)
*
* => calculates cstepq if cstep has been selected
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------|
      implicit none
      integer natoms,i
      integer ic(natoms)
      real*8 nmode(3*natoms),cstep,cstepq,redmass
      call gredmass(nmode,natoms,ic,redmass)
      redmass=sqrt(redmass)
      cstepq=cstep*redmass
      return
      end

************************************************************************
************************************************************************

      subroutine getcstepq2(estep,cstepq,vibf)
*----------------------------------------------------------------------|
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------|
      implicit none
      integer natoms,i
      real*8 estep,cstepq,vibf,qfc,sqrt
      intrinsic sqrt
      call calcqfc(qfc,vibf)
      cstepq=sqrt(estep*2.0d0/qfc)
      end

************************************************************************

      subroutine getcstep(nmode,cstep,cstepq,ic,natoms)
*----------------------------------------------------------------------|
*
* => calculates cstep if cstepq has been selected
*
* written by J. Neugebauer, Uni Erlangen, 2002
*----------------------------------------------------------------------|
      implicit none
      integer natoms,i
      integer ic(natoms)
      real*8 nmode(3*natoms),cstep,cstepq,redmass
      call gredmass(nmode,natoms,ic,redmass)
      redmass=sqrt(redmass)
      cstep=cstepq/redmass
      return
      end

************************************************************************

      subroutine displacecoord2(normmod,natoms,co,ic,nmno,
     * estep,ipre,vibf)
*----------------------------------------------------------------------|
* generation of coordinates for structures displaced along
* the normal coordinates, such that the expected difference in
* energies from the original to the displaced structure is 
* <estep> hartree
* 
* written by J. Neugebauer, Uni Erlangen, 2002
* modified by C. Herrmann, Uni Bonn, 2005
*----------------------------------------------------------------------|
      include 'defs.h'
      integer natoms,nmno,lcoo,ltcoo
      integer ic(natoms),ipre
      real*8 normmod(3*natoms*3*natoms),co(3,natoms)
      real*8 cstep,cstepq,estep,vibf,qfc
      character*(*) coord
      character*255 tcoord
      call calcqfc(qfc,vibf)
      write(iosl,*) '** displacecoord2: vibf,qfc = ',vibf,qfc

      call getcstepq2(estep,cstepq,vibf)
      write(iosl,*) '** displacecoord2: cstepq = ',cstepq

** mass-weighted normal modes are needed to calculate cstep

      call nmod_c2m(normmod(3*natoms*(nmno-1)+1),natoms,ic)
      call getcstep(normmod(3*natoms*(nmno-1)+1),cstep,cstepq,ic,natoms)
      cstep=cstep*dble(ipre)
      write(iosl,*) '** displacecoord2: cstep = ',cstep

** back to non-mass-weighted normal modes

      call nmod_m2c(normmod(3*natoms*(nmno-1)+1),natoms,ic)
CCH  Attention: This call has to be checked!
CCH      call modcoord(co,dco,natoms,normmod(3*natoms*(nmno-1)+1),cstep)

      return
      end

************************************************************************

      subroutine displacetype(disptype,type)
      implicit none
      integer type,i,mxdtype
      parameter(mxdtype=2)
      character*(*) disptype
      character*40 dtype(mxdtype)
      data dtype / 'displacement in units of length',
     *             'displacement in units of energy'/
      call clean(disptype)
      if(type.le.0.or.type.gt.mxdtype) 
     *  disptype='ERROR! No such disptype !'
      do i=1,min(len(disptype),40)
        disptype(i:i)=dtype(type)(i:i)
      end do
      end
