*****************************************************************
* constants module for use in all other modules
* values from http://physics.nist.gov/ 
* and P.J. Mohr, B.N. Taylor: CODATA recommended values of the
* fundamental physical constants,
* Rev. Mod. Phys. 72 (2000), No.2, 351-495.
*
* written by Johannes Neugebauer, Uni Erlangen, 2002
* based on the analogous F90 module 
* by C. Kind, M. Reiher, and J. Neugebauer
*****************************************************************

      real*8 pi,speedoflight,avogadro,planck_h,el_charge,el_mass,
     * mu_0,planck_hbar,amu,epsilon_0,bohr,hartree,bohr2ang,
     * hartree2kj_mol,esu,debey,kjmol2hartree,boltz,invfinestruc,
     * nm2freq,gasconstant,au2deb

* pi
      parameter(pi=3.1415926535897932384626433832795029d0)

* speed of light:
      parameter(speedoflight=2.99792458d08)

* avogadro's number:
      parameter(avogadro=6.02214199d23)

* planck constant h:
      parameter(planck_h=6.62606876d-34)

* elementary charge:
      parameter(el_charge=1.602176462d-19)

* electron mass:
      parameter(el_mass=9.10938188d-31)

* magnetic constant:
      parameter(mu_0=4.0d0*pi*1.0d-07)

* hbar=h/(2*pi) in [Js] (1.054571596d-34):
      parameter(planck_hbar=planck_h/(2.0d0*pi))

* atomic mass unit in [kg] (1.66056d-27)
      parameter(amu=1.0d0/avogadro*1.0d-03)

* electric constant (8.85419d-12)
      parameter(epsilon_0=1.0d0/(speedoflight*speedoflight*mu_0))

* bohr radius in [m] (5.291772083d-11):
      parameter(bohr=4.0d0*pi*epsilon_0*planck_hbar*planck_hbar/
     * (el_mass*el_charge*el_charge))

* hartree energy in [J] (4.35974381d-18):
      parameter(hartree=el_charge**4*el_mass/
     * ((4.0d0*pi*epsilon_0)**2*planck_hbar**2))

* conversion from [bohr] to [Angstroem] (0.529177260d0):
      parameter(bohr2ang=bohr*1.0d10)

* conversion from [hartree] to [kJ/mol] (2.62549992d03):
      parameter(hartree2kj_mol=hartree*avogadro*1.d-03)

* esu (3.33564d-10)
      parameter(esu= 1.0d-01/speedoflight)

* conversion from [esu] to [debey] (3.33564d-30):
      parameter(debey=esu*1.d-2*1.0d-18)

* conversion factor from [kJ/mol] to [hartree]:
      parameter(kjmol2hartree=0.0003809d0)

* boltzmann constant in [J/K]:
      parameter(boltz=1.3806503d-23)

* inverse fine-structure constant in [a.u.]:
      parameter(invfinestruc=137.03599976d0) 

* wavelength in [nm] to angular frequency in [a.u.]:
      parameter(nm2freq=2.d0*pi*invfinestruc/10.0d0*bohr2ang)

* universal gas constant
      parameter(gasconstant=boltz*avogadro)

* conversion factor from [a.u.] to [Debye]:
      parameter(au2deb=2.541765)
