      subroutine getnctm(control,natoms,normmod)
C
C     written by J. Neugebauer, Uni Erlangen, 2001
C     
C     read normal coordinates from TURBOMOLE aoforce output.
C     normmod(modes,coordinates)
C     (written to file control)
C
      include 'defs.h'
      character*(*) control
      character(len=255) form,buf
      integer natoms,i,j,ucf,kbuf,i2,i2end,nvar,nd
      integer kend
      real*8 normmod(3*natoms,3*natoms)
      parameter(ucf=45,nd=5)
      nvar=3*natoms
      form='(5x,5f15.10)'
      write(iosl,*) 'reading normal modes from file: ',control
      open(unit=ucf,file=control,status='old')
10    read(ucf,'(a)',err=900,end=900) buf
      if(index(buf,'$vibrational normal modes').eq.0) goto 10
      i2end=nvar/nd
      if(mod(nvar,nd).ne.0) i2end=i2end+1
      do i=1,nvar
        do i2=1,i2end
         kend=i2*nd
         if(i2.eq.i2end) kend=nvar
         kbuf=(i2-1)*nd+1
         read(ucf,form) (normmod(i,j),j=kbuf,kend)
        end do
      end do
      close(ucf)
      return
900   write(iosl,'(2a)') '** getnctm: error while reading file:',control
      call flush(iosl)
      call flush(idbg)
      stop
      end

C***********************************************************************

      subroutine getfrtm(control,natoms,freq)
C
C     written by J. Neugebauer, Uni Erlangen, 2001
C     
C     read harmonic frequencies from TURBOMOLE aoforce output.
C     (written to file control)
C
      include 'defs.h'
      character*(*) control
      character(len=255) buf
      character(len=3) symtxt
      integer natoms,i,ucf,nvar
      real*8 freq(3*natoms)
      parameter(ucf=45)
      nvar=3*natoms
      write(iosl,*) 'getfrtm: reading frequencies from file: ',control
      write(iosl,*) '-----------------------------------'
      write(iosl,*) 'wavenumbers found in control file: '
      open(unit=ucf,file=control,status='old',err=900)
10    read(ucf,'(a)',err=900,end=900) buf
      if(index(buf,'$vibrational spectrum').eq.0) goto 10
      read(ucf,'(a)',err=900,end=900) buf
      read(ucf,'(a)',err=900,end=900) buf
      do i=1,nvar
        read(ucf,'(a)',err=900,end=900) buf
        read(buf(15:17),'(a3)') symtxt
        read(buf(26:34),'(f9.2)') freq(i)
        write(iosl,'(2x,a3,f9.2)') symtxt, freq(i)
      end do
      close(ucf)
      write(iosl,*) '-----------------------------------'
      return
900   write(iosl,'(2a)') '** getfrtm: error while reading file:',control
      call flush(iosl)
      call flush(idbg)
      stop
      end

C***********************************************************************

      subroutine gethesstm(control,natoms,hessian)
C
C     written by J. Neugebauer, Uni Erlangen, 2001
C     
C     read hessian from TURBOMOLE aoforce output.
C     (written to file control)
C
      include 'defs.h'
      character*(*) control
      character(len=255) form,buf
      integer natoms,i,j,ucf,kbuf,i2,i2end,nvar,nd
      integer kend
      real*8 hessian(3*natoms,3*natoms)
      parameter(ucf=45,nd=5)
      nvar=3*natoms
      form='(5x,5f15.10)'
      write(iosl,*) 'gethesstm: reading hessian from file: ',control
      open(unit=ucf,file=control,status='old',err=900)
10    read(ucf,'(a)',err=900,end=900) buf
      if(index(buf,'$hessian (projected)').eq.0) goto 10
      i2end=nvar/nd
      if(mod(nvar,nd).ne.0) i2end=i2end+1
      do i=1,nvar
        do i2=1,i2end
         kend=i2*nd
         if(i2.eq.i2end) kend=nvar
         kbuf=(i2-1)*nd+1
         read(ucf,form) (hessian(i,j),j=kbuf,kend)
        end do
      end do
      close(ucf)
      return
900   write(iosl,'(2a)') '** gethesstm: error while reading file:',
     * control
      call flush(iosl)
      call flush(idbg)
      close(ucf)
      end

C***********************************************************************

      subroutine getirrtm(control,natoms,irrep)
C
C     written by J. Neugebauer, Uni Erlangen, 2002
C     
C     read irreps of harmonic frequencies from TURBOMOLE aoforce output.
C     (written to file control)
C
      include 'defs.h'
      character*(*) control
      character(len=255) buf
      character(len=4) symtxt
      integer natoms,i,ucf,nvar
      real*8 freq,irrep(4*natoms)
      parameter(ucf=45)
      nvar=3*natoms
      write(iosl,*) 'getirrtm: reading irreps from file: ',control
      write(iosl,*) '-----------------------------------'
      write(iosl,*) 'irreps found in control file: '
      open(unit=ucf,file=control,status='old',err=900)
10    read(ucf,'(a)',err=900,end=900) buf
      if(index(buf,'$vibrational spectrum').eq.0) goto 10
      read(ucf,'(a)',err=900,end=900) buf
      read(ucf,'(a)',err=900,end=900) buf
      do i=1,nvar
        read(ucf,'(a)',err=900,end=900) buf
        read(buf(15:18),'(a4)') symtxt
        read(buf(26:34),'(f9.2)') freq
        read(symtxt,'(a4)') irrep(i)
        write(iosl,'(2x,a4,f9.2)') irrep(i), freq
      end do
      close(ucf)
      write(iosl,*) '-----------------------------------'
      return
900   write(iosl,'(2a)')'** getirrtm: error while reading file:',control
      call flush(iosl)
      call flush(idbg)
      stop
      end

C***********************************************************************

      subroutine getescf(gradient,escf)
C
C read electronic SCF energy from file gradient
C
C written by J. Neugebauer, Uni Erlangen, 2002
C
      include 'defs.h'
      include 'param.h'
      character*(*) gradient
      real*8 escf,getreal
      integer ist,ien,lu,irc
      CHARACTER*128 item(mxitem),rec
      external getreal

      write(iosl,*) 'getescf: reading SCF energy from file: ',gradient
      escf=0.0d0
      lu=1
      OPEN (UNIT=lu,FILE=gradient,STATUS='UNKNOWN',IOSTAT=irc,err=900)
      IF (irc.NE.0) RETURN
90    READ (lu,'(A)',END=99) rec
      CALL split(rec,item,mxitem)
      IF (item(1).EQ.'cycle') GOTO 91
      GOTO 90
91    CONTINUE
      ist=1
      escf=getreal(item(7),ist,ien)
      goto 90
99    close(lu)
      return
900   write(iosl,'(2a)')'** getescf: error while reading file:',gradient
      call flush(iosl)
      call flush(idbg)
      stop
      end

C**********************************************************************

      subroutine getonlye(dscfname,escf)
      character*(*) dscfname
      character*132 row
      real*8 escf
      integer ldscf,lu
      call slen(dscfname,ldscf)
      lu=1
      open(unit=lu,file=dscfname(1:ldscf))
10    read(lu,'(A)',end=900) row
      if(index(row,'|  total energy').eq.0) goto 10
      read(row(40:59),*) escf
900   close(lu)
      end
