      subroutine gredmass(normmod,natoms,ic,redmass)

C calculate reduced mass for a particular normal mode
C written by J. Neugebauer, Uni Erlangen, 2001
 
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),redmass,find_mass
      external find_mass
      redmass=0.0d0
      do i=1,3*natoms
        redmass=redmass+normmod(i)*normmod(i)/find_mass(ic((i+2)/3))
      end do
      redmass=1.d0/redmass
      return
      end 

      subroutine gsiredmass(normmod,natoms,ic,redmass)

C calculate reduced mass for a particular normal mode
C in SI units
C written by J. Neugebauer, Uni Erlangen, 2001

      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),redmass,find_mass,sqmass,scredm
      external find_mass
      redmass=0.0d0
      do i=1,3*natoms
        sqmass=1.d0/find_mass(ic((i+2)/3))
        scredm=normmod(i)*sqmass
        redmass=redmass+scredm*scredm
      end do
      redmass=1.d0/redmass
      return
      end 

******************************************************************************

      subroutine nmod_c2m(normmod,natoms,ic)
*
* converts cartesian normal modes to mass-weighted normal modes
*
* J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),find_mass,sqmass,sum,zthres
      external find_mass
      parameter(zthres=1.0d-12)
      do i=1,3*natoms
        sqmass=sqrt(find_mass(ic((i+2)/3)))
        normmod(i)=normmod(i)*sqmass
      end do

C renormalize norm modes 
      
      sum=0.0d0
      do i=1,3*natoms
        sum=sum+normmod(i)*normmod(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,3*natoms
        normmod(i)=normmod(i)/sum
      end do
      return
      end 

******************************************************************************
   
      subroutine nmod_m2c(normmod,natoms,ic)
*
* converts mass-weighted normal modes to cartesian normal modes
*
* J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer natoms,i,ic(natoms)
      real*8 normmod(3*natoms),find_mass,sqmass,sum,zthres
      external find_mass
      parameter(zthres=1.0d-12)
      do i=1,3*natoms
        sqmass=sqrt(find_mass(ic((i+2)/3)))
        normmod(i)=normmod(i)/sqmass
      end do

C renormalize norm modes 
      
      sum=0.0d0
      do i=1,3*natoms
        sum=sum+normmod(i)*normmod(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,3*natoms
        normmod(i)=normmod(i)/sum
      end do
      return
      end 

******************************************************************************
   
      subroutine print_nmodes(normmod,natoms)
      implicit none
      integer natoms,i,j,kbuf,i2,i2end,nvar,nd,kend
      real*8 normmod(3*natoms,3*natoms)
      character(len=255) form
      parameter(nd=5)
      form='(i2,i3,5f15.10)'
      nvar=3*natoms
      i2end=nvar/nd
      if(mod(nvar,nd).ne.0) i2end=i2end+1
      do i=1,nvar
        do i2=1,i2end
         kend=i2*nd
         if(i2.eq.i2end) kend=nvar
         kbuf=(i2-1)*nd+1
         write(*,form) i,i2, (normmod(i,j),j=kbuf,kend)
        end do
      end do 
      return
      end


******************************************************************************
   
      integer function ipre(step,naus)

* yields the prefactor for displacement  <step> 

      implicit none
      integer step,natoms,naus,imode,im,idum
      external imode
      if(step.eq.0) then
        ipre=0
      else
        im=imode(step,naus)
        idum=(step-(im-1)*naus)
        ipre=naus/2-(idum-1)
        if(ipre.le.0) ipre=ipre-1
      end if
      end

******************************************************************************
   
      integer function imode(step,naus)

* yields the mode number for displacement <step> 

      implicit none
      integer step,natoms,naus
      imode=(step+naus-1)/naus
      end

******************************************************************************
   
      subroutine normalize(vector,nvec)
*
* normalization of <vector> 
*
* written by J. Neugebauer, Uni Erlangen, 2002
*
      implicit none
      integer i,nvec
      real*8 vector(nvec),sum,sqrt,zthres
      parameter(zthres=1.0d-12)
      intrinsic sqrt
   
      sum=0.0d0
      do i=1,nvec
        sum=sum+vector(i)*vector(i)
      end do
      sum=sqrt(sum)
      if(sum.le.zthres) sum=1.0d0
      do i=1,nvec
        vector(i)=vector(i)/sum
      end do
      end 
