CC******************************************************************************
C   Copyright 1999 -- 2002 C. Kind, M. Reiher, J. Neugebauer, and B. A. Hess,
C   Chair of Theoretical Chemistry, Friedrich-Alexander-University 
C   Erlangen-Nuremberg, Egerlandstr. 3, D-91058 Erlangen, Germany.
C
C
C   This file is part of the package SNF.
C
C   SNF is free software; you can redistribute it and/or modify
C   it under the terms of the GNU General Public License as published by
C   the Free Software Foundation; either version 2 of the License, or
C   (at your option) any later version.
C
C   This program is distributed in the hope that it will be useful,
C   but WITHOUT ANY WARRANTY; without even the implied warranty of
C   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C   GNU General Public License for more details.
C
C   You should have received a copy of the GNU General Public License
C   along with this program; if not, write to the Free Software
C   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
C
CC******************************************************************************
      SUBROUTINE rscontrol(tempname,progno,calcno,ipol,omeg,snffast,
     * iclean,icleanl,naus,mxomeg,cstep,idbg)
*-------------------------------------------------------------------
* read <snf_control> file
* only Turbomole-specific information is read from <control>
* modified by M. Reiher, Uni Erlangen, June 2003
*-------------------------------------------------------------------
      implicit none
      integer lu,idbg,lmod,progno,calcno,ipol
      integer iclean,icleanl,mxomeg,mxitem,i,naus
      parameter (mxitem=12)
      real*8 omeg(*),cstep
      character*255 b1,item(mxitem)
      character*20 a20
      character*(*) tempname
      logical snffast
      external getreal

      call flush(idbg)
      lu=1
      IF (mxitem.NE.mxomeg+2) THEN
       CALL errex('set mxitem to mxomeg+2 in rcontrol')
      ENDIF
C@    WRITE (idbg,*) 'reading <snfcontrol> from ',tempname
      call flush(idbg)
      OPEN (UNIT=lu,FILE=tempname,STATUS='UNKNOWN')
      call flush(idbg)
      progno=1
      calcno=1
      ipol=0
      cstep=0.01
      snffast=.false.
      iclean=0
      icleanl=0

1     READ (lu,'(A)',END=999) b1
      call flush(idbg)
      if(index(b1,'$progno').ne.0) then
        read(b1(8:),*,err=900,end=900) progno
      else if(index(b1,'$calcno').ne.0) then
        read(b1(8:),*,err=900,end=900) calcno
      else if(index(b1,'$ramanew').ne.0) then
        read(b1(9:),*,err=900,end=900) ipol
        read(lu,'(a)',err=900,end=900) b1
        if(ipol.gt.1.or.(ipol.eq.1.and.index(b1,'$').eq.0)) then
          read(b1,*,err=900,end=900) (omeg(i),i=1,ipol)
        else if(ipol.eq.1.and.index(b1,'$').ne.0) then
          omeg(1)=0.0d0
        end if
      else if(index(b1,'$vroaew').ne.0) then
        read(b1(8:),*,err=900,end=900) ipol
        ipol=ipol+1000
        read(lu,'(a)',err=900,end=900) b1
        if(ipol.gt.1001.or.(ipol.eq.1001.and.index(b1,'$').eq.0)) then
          read(b1,*,err=900,end=900) (omeg(i),i=1,ipol-1000)
        else if(ipol.eq.1001.and.index(b1,'$').ne.0) then
          omeg(1)=0.0d0
        end if
      else if(index(b1,'$tempdirclean').ne.0) then
        iclean=1
      else if(index(b1,'$logdirclean').ne.0) then
        icleanl=1
      else if(index(b1,'$snffast').ne.0) then
        snffast=.true.
      else if(index(b1,'$cstep').ne.0) then
        read(b1(7:),*,end=900,err=900) cstep
      else if(index(b1,'$numderiv').ne.0) then
        read(b1(10:),*,end=900,err=900) naus
        naus=2*naus
      end if
      goto 1
      call flush(idbg)
900   call errex('error while reading snf_control')
999   CLOSE (UNIT=lu)
      return
      end

***********************************************************************
      subroutine rscontrol2(normmode,nummodes,intonly,freq,
     *                      tempname,idbg)
*-------------------------------------------------------------------
* read normal modes, flags and frequencies from <snf_control> file
* from intonly mode
* written by C. Herrmann, Uni Bonn, 02/2005
*-------------------------------------------------------------------
      implicit none
      include 'param.h'
      integer lu,idbg,nummodes,i,j,intonly
      character*(*) tempname
      character*255 b1,item(mxitem)
      real*8 string2real,normmode(maxmode,maxatoms*3),freq(maxmode)
      lu=1
C@    WRITE (idbg,*) 'reading <snfcontrol> from ',tempname
      OPEN (UNIT=lu,FILE=tempname,STATUS='UNKNOWN')
1     READ (lu,'(A)',END=999) b1
      if(index(b1,'$nummodes').ne.0) then
        read(b1(10:),*,end=900,err=900) nummodes
        if(nummodes.ne.0) intonly = 1
        do j = 1,nummodes
          read(lu,'(A)',END=999) b1
          call split (b1,item,mxitem)
          freq(j)=string2real(item(1))
          normmode(j,1) = string2real(item(2))
          normmode(j,2) = string2real(item(3))
          normmode(j,3) = string2real(item(4))
          do i=2,natoms 
            read(lu,'(A)',END=999) b1
            call split (b1,item,mxitem)
            normmode(j,(i-1)*3+1) = string2real(item(1))
            normmode(j,(i-1)*3+2) = string2real(item(2))
            normmode(j,(i-1)*3+3) = string2real(item(3))
          end do
        end do
      end if
      goto 1
900   call errex('error while reading snf_control')
999   CLOSE (UNIT=lu)
      return
      end

*********************************************************************** 
      subroutine rkeyreal(anfname,anfkey,anfval)
*
* read file anfname; if <anfkey> is present, then read double precision
* variable anfval from the keyword input line
*
* written by J. Neugebauer, Uni Erlangen, 2002.
*
      implicit none
      integer lu,lanfn,lanfk
      real*8 anfval,anfold
      character*(*) anfname,anfkey
      character*132 buf
      logical ex
      lu=1
      call llen(anfname,lanfn)
      call llen(anfkey,lanfk)
      inquire(file=anfname(1:lanfn),exist=ex)
      if(.not.ex) return
      open(unit=lu,file=anfname(1:lanfn))
      anfold=anfval
10    read(lu,'(A)',end=900) buf
      if(index(buf,anfkey(1:lanfk)).ne.0) then
        read(buf(lanfk+1:),*,end=800,err=800) anfval
        goto 900
      end if
800   anfval=anfold
      goto 10
900   close(lu)
      end
 
***********************************************************************
 
      subroutine rkeyint(anfname,anfkey,anfval)
* * read file anfname; if <anfkey> is present, then read integer
* variable anfval from the keyword input line
*
* written by J. Neugebauer, Uni Erlangen, 2002.
*
      implicit none
      integer lu,lanfn,lanfk,anfval,anfvalold
      character*(*) anfname,anfkey
      character*132 buf
      logical ex
      lu=1
      anfvalold=anfval
      call llen(anfname,lanfn)
      call llen(anfkey,lanfk)
      inquire(file=anfname(1:lanfn),exist=ex)
      if(.not.ex) return
      open(unit=lu,file=anfname(1:lanfn))
10    read(lu,'(A)',end=900) buf
      if(index(buf,anfkey(1:lanfk)).ne.0) then
        read(buf(lanfk+1:),*,err=800,end=800) anfval
        goto 900
      end if
800   anfval=anfvalold
      goto 10
900   close(lu)
      end
 
***********************************************************************
 
      subroutine rkeywd(anfname,anfkey,anfval)
 
*------------------------------------------------------------------
* read file anfname; if <anfkey> is present, then set anfval=.true.
*
* written by J. Neugebauer, Uni Erlangen, 2002.
*------------------------------------------------------------------
 
      implicit none
      integer lu,lanfn,lanfk
      character*(*) anfname,anfkey
      character*132 buf
      logical ex,anfval
      lu=1
      anfval=.false.
      call llen(anfname,lanfn)
      call llen(anfkey,lanfk)
      inquire(file=anfname(1:lanfn),exist=ex)
      if(.not.ex) return
      open(unit=lu,file=anfname(1:lanfn))
10    read(lu,'(A)',end=900) buf
      if(index(buf,anfkey(1:lanfk)).ne.0) then
        anfval=.true.
        goto 900
      end if
      goto 10
900   close(lu)
      end
 
                             

