CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C       This routine is based in parts on subroutines from the      C
C       vibrational spectroscopy program SNF, which is distributed  C
C       in terms of the GNU General Public License:                 C
C       http://www.chemie.uni-erlangen.de/hess                      C
C       see reference:                                              C
C       J. Neugebauer, M. Reiher, C. Kind, B. A. Hess,              C
C       J. Comput. Chem., 23 (2002) 895                             C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      subroutine wadf(tdr,natoms,ic,co,iosl)
*
*-------------------------------------------------------------------
*
*  generates ADF-input file 'adfslave.in' on slaves
*  'adfslave.in' differs from 'adf.in' (adf input file provided
*  by the user) only by
*  a) the coordinates (which are replaced by an elongated
*     structure) and
*  b) leaving out the 'Unit' block, if present in 'adf.in'
*
*  written by C.Herrmann, University of Bonn, 01/2005
*
*-------------------------------------------------------------------
*
      implicit none
c
      integer gunit,aunit,natoms,ic(natoms),ltd,i,j,ibefore
      integer iend,k,maxrow,mxitem,iafter,info
      parameter(gunit=10,aunit=11,maxrow=100,mxitem=10)
      real*8 co(3,natoms)
      character*(*) tdr
      character(len=132) row(maxrow),item(mxitem),scratch
      character*2 find_element
      external find_element
      character*255 fname, aname, element(natoms)
      logical valbohr
      integer lrow(maxrow),iosl
      include 'fconstants.h'
c
      call llen(tdr,ltd)
      aname=tdr(1:ltd)//'/adf.in'
      fname=tdr(1:ltd)//'/adfslave.in'
c
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      call llen(row(i),lrow(i))
      write(iosl,*) 'lrow, i, row: ', lrow(i), i, row(i)
      call flush(iosl)
      call split(row(i),item,mxitem)
c read over equilibrium atom coordinates
      if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms'
     *    ) then
c mark beginning of coordinate section
        ibefore=i
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of coordinate section
          iafter = i
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      else if(item(1).eq.'Unit'.or.item(1).eq.'UNIT'
     *.or.item(1).eq.'unit') then
        i = i-1
        do k = 1,maxrow
25      read(aunit,'(a)') scratch
        call split(scratch,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          goto 30
        else
          goto 25
        end if
        end do
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fname,status='unknown')
CCH writing rows in adf-input file with blanks at the end
CCH makes ADF crash for some reason ...
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
        do i=1,natoms
          write(gunit,40) find_element(ic(i)), (co(j,i)*bohr2ang,j=1,3)
        end do
40    format(a4,2x,3F18.12)
      write(gunit,'(a)') 'End'
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      close(gunit)
c
      info=system('chmod u+x '//fname)
c
      end subroutine wadf


*************************************************************************
      subroutine radf(fname,natoms)

*-------------------------------------------------------------------
*
*  reads ADF input file fname
*
*  written by C. Herrmann, Universitaet Jena 04/2005
*
*-------------------------------------------------------------------
c
      implicit none
c
      integer iunit,natoms,lfname,mxitem,i,mxatoms
      parameter(mxitem=10,mxatoms=200)
      character*255 fname,row,item(mxitem)
      iunit=10
      call llen(fname,lfname)
      open(unit=iunit,file=fname(1:lfname))
10    read(iunit,'(a132)',end=30) row
      call split(row,item,mxitem)
c if Atom-block found, determine number of atoms 
      if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms'
     *    ) then
20      read(iunit,'(a132)') row
        call split(row,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          goto 10
        else
          natoms = natoms+1
          i=i+1
          goto 20
        end if
      end if
      goto 10
30    continue
      close(iunit)
      end

************************************************************************
c
      subroutine radfcoord(fname,natoms,co,ic,iosl)
c
c  written by C. Herrmann, Uni Jena 04/2005
c
      implicit none
c
      integer lf,iosl,find_nucl_charge,ic(100)
      character*255 fname
      integer gunit,mxitem,lrow,nempty,natoms,i,j,coordunit
      integer ist,end,k,maxrow
      parameter(mxitem=10,maxrow=100)
      character(len=132) row,item(mxitem)
      character(len=2) el(100)
      real*8 co(3,100),string2real
      logical valbohr
      include 'fconstants.h'
c
      gunit=10
      valbohr=.false.
      i=1
c
      call llen(fname,lf)
      open(gunit,file=fname(1:lf),status='unknown')
10    read(gunit,'(a132)',end=30) row
          write(iosl,*) 'WADF',row
          write(*,*) 'WADF',row
          call flush(iosl)
      call split(row,item,mxitem)
      if (item(1).eq.'Atoms'.or.item(1).eq.'ATOMS'.or.item(1).eq.'atoms'
     *    ) then
20      read(gunit,'(a132)') row
        call split(row,item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
          goto 10
        else
          read(row,*) el(i), co(1,i), co(2,i), co(3,i)
          ic(i) = find_nucl_charge(el(i))
          i=i+1
          goto 20
        end if
c----------------------------------------------------------------------
c if there is a unit block specifying other than standard units (Bohr),
c set logical variable valbohr = .true.
c---------------------------------------------------------------------
      else if(item(2).eq.'Bohr'.or.item(2).eq.'BOHR'.or.item(2).eq.
     *    'bohr') then
          valbohr=.true.
      end if
      goto 10
30    continue
      close(gunit)
c----------------------------------------------------------------------
c if coordinates are given in Angstroem (ADF default), convert them to Bohr
c (Akira internal standard)
c----------------------------------------------------------------
      if(.not.valbohr) then
        do i = 1,natoms
        do j=1,3
         write(*,*) 'i,j ', i,j
           co(j,i) =  co(j,i)*(1/bohr2ang)
        end do
        end do
      end if

      end subroutine
c
c
c***********************************************************************
c
      subroutine wadfdip(tdr,fname,iosl)
*
*-------------------------------------------------------------------
*
*  generates ADF single-point input file name on slaves
*
*  written by C.Herrmann, University of Jena, 04/2005
*
*-------------------------------------------------------------------
*
      implicit none
c
      integer gunit,aunit,ltd,i,j,ibefore,lf
      integer iend,k,maxrow,mxitem,iafter,info
      parameter(gunit=10,aunit=11,maxrow=100,mxitem=10)
      character*(*) tdr
      character(len=132) row(maxrow),item(mxitem),scratch
      character*2 find_element
      external find_element
      character*255 fname, aname,fnamefull 
      logical valbohr
      integer lrow(maxrow),iosl
      include 'fconstants.h'
c
      call llen(tdr,ltd)
      call llen(fname,lf)
      fnamefull=tdr(1:ltd)//'/'//fname(1:lf)
      aname=tdr(1:ltd)//'/adfslave.in'
      call llen(fnamefull,lf)
c
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if (item(1).eq.'Geometry'.or.item(1).eq.'GEOMETRY'.or.item(1).eq.
     *'geometry') then
c mark beginning of geometry section
        ibefore=i-1
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of geometry section
          iafter = i+1
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fnamefull(1:lf),
     *status='unknown')
CCH writing rows in adf-input file with blanks at the end
CCH makes ADF crash for some reason ...
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      call flush(iosl)
      close(gunit)
c
      info=system('chmod u+x '//fnamefull(1:lf))
c
      end subroutine wadfdip


c***********************************************************************
c
      subroutine wadfpol(tdr,fname,iosl)
*
*-------------------------------------------------------------------
*
*  generates ADF single-point input file name on slaves
*  for calculation of polarizabilities
*
*  written by C.Herrmann, University of Jena, 04/2005
*
*-------------------------------------------------------------------
*
      implicit none
c
      include 'param.h'
      integer gunit,aunit,ltd,i,j,ibefore,lf
      integer iend,k,maxrow,iafter,info
      parameter(gunit=10,aunit=11,maxrow=100)
      character*(*) tdr
      character(len=132) row(maxrow),item(mxitem),scratch
      character*2 find_element
      external find_element
      character*255 fname, aname,fnamefull 
      logical valbohr
      integer lrow(maxrow),iosl
      include 'fconstants.h'
c
      call llen(tdr,ltd)
      call llen(fname,lf)
      fnamefull=tdr(1:ltd)//'/'//fname(1:lf)
      aname=tdr(1:ltd)//'/adfslave.in'
      call llen(fnamefull,lf)
c
      i = 1
      open(aunit,file=aname,status='unknown')
10    read(aunit,'(a)',end = 35) row(i)
      call llen(row(i),lrow(i))
      call split(row(i),item,mxitem)
      if (item(1).eq.'Geometry'.or.item(1).eq.'GEOMETRY'.or.item(1).eq.
     *'geometry') then
c mark beginning of geometry section
        ibefore=i-1
        i = i+1
20      read(aunit,'(a)') row(i)
        call split(row(i),item,mxitem)
        if(item(1).eq.'End'.or.item(1).eq.'END'.or.item(1).eq.'end')then
c mark end of geometry section
          iafter = i+1
          goto 30
        end if
        i=i+1
        goto 20
c if there is a unit block specifying other than standard units (Angstrom),
c leave it out
      end if
30    continue
      i=i+1
      goto 10
35    continue
      close(aunit)
      iend=i-1
c
      open(unit=gunit,file=fnamefull(1:lf),
     *status='unknown')
CCH writing rows in adf-input file with blanks at the end
CCH makes ADF crash for some reason ...
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=1,ibefore)
      write(gunit,'(a)') 'Response'
      write(gunit,'(a)') '  allcomponents'
      if(omeg(1).ne.0.0d0) then
        write(gunit,'(a7,i3)') ' nfreq ',ipol 
        write(gunit,'(a8,f16.8)') ' frqbeg ',omeg(1) 
        write(gunit,'(a8,f16.8)') ' frqend ',omeg(ipol)
        write(gunit,'(a)') ' hartree'  
      end if
      write(gunit,'(a)') 'End' 
      write(gunit,'(a)') (row(i)(1:lrow(i)),i=iafter,iend)
      call flush(iosl)
      close(gunit)
c
      info=system('chmod u+x '//fnamefull(1:lf))
c
      end subroutine wadfpol


